////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : AbstractModule.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:24 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 14:29:28 2000
//  Update count     : 27
//  CVS Info         : $Id: AbstractModule.java,v 1.2 2002/03/18 12:28:37 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		    Changes/Extensions
//  ----	    ----		    ------------------
//  20.02.01    vigelius        implemented setShowing()
//  21.02.01    vigelius        removed viewer/sort module window bounds
//  26.02.01    vigelius        ehanced documentation
//  08.03.01    vigelius        changed ChangeEvent to ModuleChangeEvent
//  08.03.01    vigelius        declared abstract in order to force ModuleWindows
//                              to overwrite newModuleWindow()
//  14.03.01    vigelius        added method to set module name
//  17.05.01    vigelius        Implemented afterDeserialisation()
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.ModuleChangeEvent;

/**
 * AbstractModule is the recommended superclass for all modules. It provides
 * methods for accessing private member states as well as managing the
 * associated ModuleListeners.
 */
abstract public class AbstractModule implements Module {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Default Icon.
     */
    private static ImageIcon s_icon = new ImageIcon("images/module.gif");

    /**
     * Name of the module. This is for the module window as well as for the
     * description in the project window.
     * @serial Name of the module
     */
    protected String m_name;

    /**
     * List of all associated ModuleListeners.
     * @see visalg.basics.ModuleListener
     * @serial List of associated ModuleListeners
     */
    private Vector m_moduleListeners;

    /**
     * Reference to the associated ModuleManager.
     * @see visalg.basics.ModuleManager
     * @serial Reference to the associated ModuleManager
     */
    protected ModuleManager m_moduleManager;

    /**
     * Reference to the ModuleWindow.
     * @see visalg.graphics.ModuleWindow
     */
    protected transient ModuleWindow m_moduleWindow;

    /**
     * The window bounds of the ModuleWindow.
     * @see visalg.graphics.ModuleWindow
     * @serial Window bounds of the ModuleWindow
     */
    protected Rectangle m_moduleWindowBounds;

    /**
     * If the ModuleWindow is visible, this state is TRUE. This is needed
     * to reconstruct the module window after serialisation.
     * @serial TRUE, if the ModuleWindow is visible.
     */
    private boolean m_showing;

    /**
     * If the module can react on a stepping request, this state is TRUE.
     * @serial TRUE, if the module can step.
     */
    private boolean m_stepping;

    /**
     * Contains the number of all performed steps of this module.
     * @serial Performed steps of this module
     */
    protected int m_time;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * The module will be created with a certain name and an associated module
     * manager. The window bounds will be set to default value and all member
     * states will be initialised.
     *
     * @param label Name of the module
     * @param mm Reference to the ModuleManager
     *
     * @see visalg.basics.ModuleManager
     */
    public AbstractModule(String label, ModuleManager mm) {
    	m_time = 0;
	    m_name = label;
	    m_stepping = true;
	    m_moduleWindowBounds = new Rectangle(0,0,400,200);
    	m_moduleListeners = new Vector();
	    m_moduleManager = mm;
    }//AbstractModule

    /**
     * This method will be called, if the module state has changed. It notifies
     * all connected ModuleListener as well as the ModuleWindow.
     *
     * @see visalg.basics.ModuleListener
     * @see visalg.graphics.ModuleWindow
     * @see visalg.basics.AbstractModule#step()
    **/
    protected void fireEvent() {
        // Notify the moduleWindow if exists
	    if (m_moduleWindow != null) {
	        m_moduleWindow.moduleChanged(new ModuleChangeEvent(this));
	    }

    	// Notify all other listeners
	    for (Enumeration e = m_moduleListeners.elements() ; e.hasMoreElements() ;) {
	        ModuleListener a = (ModuleListener)e.nextElement();
	        a.moduleChanged(new ModuleChangeEvent(this));
	    }
    }// fireEvent

    /**
     * This method allows easy access to all data modules, that are connected
     * to the associated ModuleManager.
     *
     * @see visalg.basics.DataModule
     *
     * @return List of DataModules
     */
    public Vector getDataModules() {
    	return m_moduleManager.getDataModules();
    }//getDataModules

    /**
     * This method allows easy acces to all modules connected to the associated
     * ModuleManager.
     *
     * @return List of all modules.
     */
    public Vector getModules() {
    	return m_moduleManager.getModules();
    }//getModules

    /**
     * This method can set the state of the module window either to "visible"
     * or "invisible". This will not affect the window itself, it is just used,
     * if all windows need to be reconstructed, for example after serialisation.
     * This method will be called by the ApplicationWindow, if a module window
     * registers itself.
     *
     * @see visalg.graphics.ApplicationWindow#addModuleWindow(visalg.graphics.ModuleWindow,java.awt.Rectangle)
     * @see visalg.graphics.ApplicationWindow#removeModuleWindow(visalg.graphics.ModuleWindow)
     *
     * @param b True, if the window should be visible
     */
    public void setShowing(boolean b) {
    	m_showing = b;
    }//setShowing


    // ----------- from Interface OtherModule  ------------------


    /**
     * This method adds a ModuleListener to the list of connected listeners.
     * This listener will be notified, if a change occured.
     *
     * @param a Reference to the ModuleListener to add
     *
     * @see visalg.basics.ModuleListener
     */
    public void addModuleListener(ModuleListener a) {
    	if (a instanceof Serializable)
	        m_moduleListeners.add(a);
    }//addModuleListener

    /**
     * Returns the number of all performed steps of this module.
     *
     * @return Number of performed steps
     */
    public int getTime() {
    	return m_time;
    }//getTime

    /**
     * Removes a ModuleListener from the list of the connected listeners. This
     * listener will not be notified of a change anymore.
     *
     * @param a Reference to the ModuleListener to be removed
     *
     * @see visalg.basics.ModuleListener
     */
    public void removeModuleListener(ModuleListener a) {
    	m_moduleListeners.remove(a);
    }//removeModuleListener

    /**
     * Returns the name of the module.
     *
     * @return Name of the module
     *
     * @see visalg.basics.AbstractModule#AbstractModule(String, ModuleManager)
     */
    public String toString() {
    	return m_name;
    }//toString

    /**
     * Sets the modules name.
     *
     * @param name The new name for this module
     * @see AbstractModule#toString()
     */
    public void setString(String name) {
        m_name = name;
        m_moduleManager.repaint();
    }//setString
    // ----------- from Interface Module  ------------------

    /**
     * Close the associated module window, if it exists.
     *
     * @see visalg.graphics.ModuleWindow
     */
    public void discardWindows() {
    	if (m_moduleWindow!=null) {
	        m_moduleManager.getModuleWindowContainer().removeModuleWindow(m_moduleWindow);
	        m_moduleWindow=null;
	    }
    }//discardWindows

    /**
     * Returns the ModuleManager.
     *
     * @return Reference to the ModuleManager
     *
     * @see visalg.basics.ModuleManager
     */
    public ModuleManager getModuleManager() {
    	return m_moduleManager;
    }//getModuleManager

    /**
     * Returns the icon for the module. If this method is not overwritten, it
     * will return a default icon.
     *
     * @return The icon of this module
     */
    public Icon getNodeIcon() {
    	return s_icon;
    }//getNodeIcon

    /**
     * Returns a list of all connected sub-modules. It will return NULL by
     * default, and must be overwritten, if the derived class acts as a
     * ModuleManager.
     *
     * @return List of all connected sub-modules
     *
     * @see visalg.basics.ModuleManager
     */
    public Vector getSubModules() {
    	return null;
    }//getSubModules

    /**
     * Returns TRUE, if the module window is visible.
     *
     * @return TRUE, if the module window is visible.
     *
     * @see visalg.basics.AbstractModule#setShowing(boolean)
     */
    public boolean isShowing() {
    	return m_showing;
    }//isShowing

    /**
     * Returns TRUE, if the module reacts on stepping requests.
     *
     * @return TRUE, if the module is "steppable"
     */
    public boolean isStepping() {
    	return m_stepping;
    }//isStepping

    /**
     * Creates a new ModuleWindow. This method must be overwritten in derived
     * classes in order to create the appropriate ModuleWindow object. It will
     * create a (useless) ModuleWindow by default.
     *
     * @see visalg.graphics.ModuleWindow
     * Neues ModuleWindow. Unbedingt berschreiben!
     */
    abstract public void newModuleWindow();
    /**
     * Sets a new ModuleManager.
     *
     * @param mm Reference to the new ModuleManager
     *
     * @see visalg.basics.ModuleManager
`     */
    public void setModuleManager(ModuleManager mm) {
    	if (m_moduleManager!=null) m_moduleManager = mm;
    }//setModuleManager

    /**
     * Sets new window bounds of the module, as well as the state of the window
     * ("visible" or "invisible").
     *
     * @param visible TRUE, if the window should be visible.
     * @param r The new window bounds of the module
     */
    public void setModuleWindowBounds(boolean visible, Rectangle r) {
    	m_moduleWindowBounds = r;
	    setShowing(visible);
    }//setModuleWindowBounds

    /**
     * Sets the state of this module to either "stepping" or "not stepping".
     *
     * @param b TRUE, if the module reacts on stepping events.
     *
     * @see visalg.basics.AbstractModule#isStepping()
     */
    public void setStepping(boolean b) {
    	m_stepping = b;
    }//setStepping

    /**
     * This method will be called by the ModuleManager, if either the user or
     * the StepThread invokes a global step. If the modules contains data, which
     * are needed by other modules and this data changes, the other modules need
     * to be notified. If this method is overwritten for some reason, the
     * superclass method has to be called up at the end.
    **/
    public void step() {
    	m_time += 1;
	    fireEvent();
    }//step

    /**
     * Does nothing at all.
     */
    public void afterDeserialisation() { }

    /**
     * Returns a reference to the module window.
     *
     * @return Reference to the module window, null if none is open
     */
    public ModuleWindow getModuleWindow() {
        if (isShowing()) {
            return m_moduleWindow;
        } else {
            return null;
        }
    }
} // class AbstractModule
