////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : Algorithm.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:19 1999
//  Last modified by : ahr
//  Last modified on : Wed Oct  4 11:42:57 2000
//  Update count     : 4
//  CVS Info         : $Id: Algorithm.java,v 1.2 2002/03/18 12:28:37 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
// 29.12.2000 vigelius  Konstruktor setzt m_done auf false.
//  08.03.01    vigelius        declared abstract (see AbstractModule)
//  04.04.01    vigelius    added abstract method setDataModule/getDataModule
//  09.04.01    vigelius    added ModuleManager Interface
//                          added newModuleWindow() method
//  17.05.01    vigelius    removed ModuleManager Interface :-))
//  17.05.01    vigelius    Added implementation of afterDeserialisation()
//                          and new function registerProperties()
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import visalg.basics.AbstractModule;
import visalg.basics.AbstractCommand;
import visalg.basics.ModuleManager;
import visalg.graphics.ModuleWindowContainer;
import visalg.graphics.MenuBarContainer;
import visalg.types.ModuleChangeEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This is the recommended base class for a module, that represents some kind
 * of algorithm. It contains an array of all available commands, that can be
 * requested by other modules, for example the trigger module. Furthermore, it
 * implements a step method, that automatically processes the next command.
 *
 * @see visalg.basics.Command
 * @see visalg.modules.PrimitiveTrigger
 */
abstract public class Algorithm extends AbstractModule implements ModuleListener {
    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * This array contains the different commands of the algorithm.
     * @serial All available commands.
     */
    protected Command[] m_commands;

    /**
     * The last performed command of this algorithm (null, if the algorithm
     * is at the beginning).
     * @serial Last performed algorithm of this algorithm.
     */
    protected Command m_lastCommand;

    /**
     * Next command of the algorithm (null, if done).
     * @serial Next command to be performed (null, if done)
     */
    protected Command m_nextCommand;

    /**
     * Indicates if the algorithm has finished yet.
     * @serial Indicates if the algorithm is done (TRUE).
     */
    protected boolean m_done;

    /**
     * Contains the icon for this algorithm. If none is set, it contains a
     * default icon.
     *
     * @serial The icon for this algorithm.
     */
    private static ImageIcon s_icon = new ImageIcon("images/algorithm.gif");


    /**
     * List of all connected sub modules. All viewers are sub modules, but
     * there could of course be other sub modules.
     */
    protected Vector m_subModules;

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * This calls up the superclass constructor and additionally sets up some
     * member states.
     *
     * @see visalg.basics.AbstractModule#AbstractModule(String,visalg.basics.ModuleManager)
     */
    public Algorithm(String s, ModuleManager mm) {

    	super(s, mm);

	    m_done = true;
    	m_commands = null;
	    m_lastCommand = null;
    	m_nextCommand = null;

        m_subModules = new Vector();
    }//Algorithm

    /**
     * Returns the classes of all available commands for this algorithm.
     * @return All commands
     *
     * @see visalg.basics.Command
     */
    public Command[] getCommands() {
    	return m_commands;
    }//getCommands

    /**
     * Returns the last processed command.
     * @return The last command
     *
     * @see visalg.basics.Command
     */
    public Command getLastCommand() {
    	return m_lastCommand;
    }//getLastCommand

    /**
     * Gives Information if the algorithm is still working (FALSE) or
     * if it is done (TRUE).
     * @return True, if the algorithm has finished.
     */
    public boolean isDone() {
    	return m_done;
    }//isDone

    /**
     * Sets the state of the algorithm (done or not).
     * @param b True, if the algorithm is done.
     */
    protected void setDone(boolean b) {
    	m_done = b;
    }//setDone

    /**
     * Sets the next command to be processed.
     *
     * @param ac Reference to the next command
     */
    protected void setNextCommand(Command ac) {
    	m_nextCommand = ac;
    }//setNextCommand

    // ----------- from class AbstractModule (resp. interface Module) ------------------

    /**
     * The call of step will execute the command saved in nextCommand.
     */
    public void step() {
    	// Do nothing if algorithm is finished
	    if (m_done) {
	        return;
    	}

	    // Perform nextCommand
    	if (m_nextCommand != null) {
	        Command c = m_nextCommand;
	        c.doIt();
    	    m_lastCommand = c;
	    }

    	setProperties();

        // Calls step method of AbstractModule class. The time will be
	    // incremented, the Module window and all other listeners will be
    	// notified.
	    super.step();
    }//step

    /**
     * Returns icon for Algorithm.
     *
     * @return The icon
     */
    public Icon getNodeIcon() {
    	return s_icon;
    }//getNodeIcon

    /**
     * Sets a new data module. The Algorithm must register itself as a
     * module listener in order to make sure, it can react on changes of
     * the data module. This can be done by calling super() in derived classes.
     * After that, fireEvent() should be called in order
     * to notify all connected listeners.
     *
     * @param dm The new data module
     *
     * @see visalg.basics.AbstractModule#fireEvent()
     */
    public void setDataModule(DataModule dm) {
        if (dm!=null) {
            dm.addModuleListener(this);
        }
    }//setDataModule(DataModule)

    /**
     * Returns the current data module.
     *
     * @return The data module
     */
    abstract public DataModule getDataModule();

    /**
     * Returns all submodules.
     *
     * @return All submodules
     */
    public Vector getSubModules() {
        return m_subModules;
    }//getModules()

    /**
     * Browses the list of all submodules and calls newModuleWindow() on them,
     * if they are currently visible.
     */
    public void newModuleWindow() {
        for (Enumeration e = m_subModules.elements() ; e.hasMoreElements() ;) {
	        Module m = (Module)e.nextElement();
            if (m.isShowing()) {
		        m.newModuleWindow();
            }
	    }
    }//newModuleWindow()

    /**
     * Will be called, after a step has performed. Derived classes should set
     * up the properties to the according values in this method.
     */
    public void setProperties() {}

    /**
     * Will be called in order to tell the algorithm, to register its properties.
     * This method must be overwritten by all algorithm classes, since otherwise
     * serialisation will not work properly.
     */
    public void registerProperties() {}

    /**
     * Just calls registerProperties() of this module. Derived classes should
     * invoke the superclass method first.
     */
    public void afterDeserialisation() {
        //registerProperties();
    }

    /**
     * This method should reset all internal data of the algorithm and register
     * the new properties. It will be called by the standard implementation of
     * moduleChanged().
     */
    abstract protected void reset();

    /**
     * Will be called if the data module has been changed. This standard
     * implementation calls up reset(). If the algorithm also listens to other
     * modules than the DataModule, it has to check the source of the
     * ModuleChangeEvent.
     */
    public void moduleChanged(ModuleChangeEvent evt) {
        reset();
    }//moduleChanged(ModuleChangeEvent)
} // class Algorithm