////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : Application.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:15 1999
//  Last modified by : ahr
//  Last modified on : Wed Oct  4 16:57:37 2000
//  Update count     : 21
//  CVS Info         : $Id: Application.java,v 1.2 2002/03/18 12:28:37 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  26.02.01    vigelius    enhanced documentation
//  07.03.01    vigelius    removed dynamic menubar bug
//  02.04.01    vigelius    added JHelpSystem
//  18.06.01    vigelius    introduce a sensible initial divider location
//
///////////////////////////////////////////////////////////////////////////////

package visalg.basics;

import java.io.File;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;

import javax.swing.UIManager;
import javax.swing.JFileChooser;
import java.net.URL;

import java.awt.Rectangle;

import javax.help.*;

import visalg.basics.History;
import visalg.graphics.ApplicationWindow;
import visalg.graphics.MenuBarContainer;
import visalg.graphics.HistoryWindowContainer;
import visalg.graphics.ProjectWindowContainer;
import visalg.graphics.ModuleWindowContainer;

/**
 * The VisAlg main class. This is the top class of the hierarchy. On creation,
 * it creates all necessary classes, like the History or the Project, including
 * the window classes. This class is the bridge, between the model and the view.
 * @author Martin Spoden
 */
public class Application {
    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the application window.
     */
    private ApplicationWindow m_applicationWindow;

    /**
     * Reference to the History.
     */
    private History m_history;

    /**
     * Reference to the Help System.
     */
    private HelpSet m_hs;

    /**
     * The initial position and size of the application window.
     */
    private Rectangle m_startDimension = new Rectangle(20,40,800,600);

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * The constructor sets up its window as well as the LookAndFeel and then
     * creates a new history by calling NewHistory(). If it is not possible to
     * create a new window object, the application stops with an error message.
     * @see visalg.graphics.ApplicationWindow
     * @see visalg.basics.Application#newHistory()
     */
    public Application() {
    	// Windows-LookAndFeel, wenn mglich.
	    try {
	        UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
    	} catch (Exception e) {
	        System.err.println(e);
    	}

        /**
         * Tries to locate the JHelp *.hs.
         */
        try {
          URL hsURL = new URL("file:./doc/visalg.hs");
          /*while (hsURL == null) {
            JFileChooser d = new JFileChooser();
            d.setCurrentDirectory(new File("."));
            d.setMultiSelectionEnabled(false);

            d.showOpenDialog(null);

            File f = d.getSelectedFile();

            hsURL = HelpSet.findHelpSet(Application.class.getClassLoader(), f.getAbsolutePath());
          }*/
          m_hs = new HelpSet(null, hsURL);
        }
        catch (Exception ee) {
          System.err.println("Could not locate Help System !");
          System.err.println("Due to : "+ee);
          System.exit(0);
        }

        /* Applikationsfenster erzeugen, wenn nicht mglich, dann
         * Programmende mit Fehlermeldung
    	 */
	    try {
	        m_applicationWindow = new ApplicationWindow(this, m_startDimension);
    	    m_applicationWindow.setVisible(true);
	    } catch (Exception e) {
	        System.err.println(e);
    	    System.err.println("VisAlg-Fenster konnte nicht dargestellt werden.");
	        System.exit(0);
    	}

    	//neue History und damit neues Projekt starten...
	    newHistory();
    } // Application()

    /**
     * If the constructor is invoked with a file as argument, it tries to load
     * the given history. If this is not possible, a new history will be
     * created without notification. It then processes the same actions as the
     * standard constructor.
     * @see visalg.basics.Application#Application()
     *
     * @param f The file, from which the History should be loaded.
     */
    public Application(File f) {
    	// Windows-LookAndFeel, wenn mglich.
	    try {
    	    UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
    	} catch (Exception e) {
    	    System.err.println(e);
    	}

        /**
         * Tries to locate the JHelp *.hs.
         */
        try {
          URL hsURL = HelpSet.findHelpSet(null, "doc/visalg.hs");
          m_hs = new HelpSet(null, hsURL);
        }
        catch (Exception ee) {
          System.err.println("Could not locate Help System !");
        }

    	/* Applikationsfenster erzeugen, wenn nicht mglich, dann
    	 * Programmende mit Fehlermeldung
    	 */
	    try {
	        m_applicationWindow = new ApplicationWindow(this, new Rectangle(20,40,600,400));
    	    m_applicationWindow.setVisible(true);
	    } catch (Exception e) {
            System.err.println(e);
    	    System.err.println("VisAlg-Fenster konnte nicht dargestellt werden.");
	        System.exit(0);
	    }


    	/* History laden. wenn fehlerhaft neue History und damit neues
    	 * Projekt starten.
	     */
    	if (!(loadHistory(f))) {
	        newHistory();
    	}
    }// Application(File f)

    /**
     * This method will exit VisAlg.
     */
    public void exit() {
    	System.exit(0);
    }//exit()

    /**
     * Creates a new History object and opens new windows. The latter one is
     * done by calling the method newWindows().
     *
     * @see visalg.basics.Application#newWindows()
     */
    public void newHistory() {

    	m_history = new History(this);

    	/* Es werden nun neue Fenster eingesetzt, um alle Referenzen
	     * auf die alten Objekte zu lschen.  Damit kann der
    	 * Garbage-Collector diese einsammeln.
	     */
    	newWindows();

        m_applicationWindow.setLeftWindowDivider(m_startDimension.height/2);
    }//newHistory()

    /**
     * Tries to load a History out of the given file. If this was successfull,
     * new windows are created by invoking newWindows().
     * @param f The file, that contains the History.
     * @return True, if the History could be loaded.
     */
    public boolean loadHistory(File f) {

    	boolean returnValue = true;

    	// Try to read object from file f
	    Object object = new Object();
    	try {
	        ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
	        object = in.readObject();
    	    in.close();
	    } catch (Exception e) {
	        System.err.println(e);
    	    returnValue = false;
	    }

    	// Is it a history object?
	    if (object instanceof History) {
	        m_history = (History)object;
    	    m_history.setApplication(this);
	        newWindows();
    	}
	    else {
	        returnValue = false;
    	}

	    return returnValue;
    }//loadHistory(File)

    /**
     * Tries to save the current History into the given File.
     * @param f The file, the History should be stored in
     * @return True, if the saving has been successful.
     */
    public boolean saveHistory(File f) {

    	boolean returnValue = true;

	    // Try to write history into file f.
    	try {
	        ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(f));
	        out.writeObject(m_history);
    	    out.close();
	    } catch (IOException e) {
    	    System.err.println(e);
	        returnValue = false;
    	}

	    return returnValue;
    }//saveHistory(File)

    /**
     * Lets the History create new windows. This is done by calling
     * History.newWindows().
     *
     * @see visalg.basics.History#newWindows()
     */
    public void newWindows() {
        m_applicationWindow.newMenu();
    	m_history.newWindows();
	    //(Dies wird z.B. nach dem Laden von Projekten gebraucht.)
    }//newWindows()

    /**
     * Gives back the container for the menu
     * @return Menubar container
     */
    public MenuBarContainer getMenuBarContainer() {
	return m_applicationWindow;
    }//getMenuBarContainer

    /**
     * Returns a reference to the current history.
     * @return The current History
     */
     public History getHistory()
     {
        return m_history;
    }//getHistory

   /**
    * Returns a referencet to the Help System.
    * @return The Help Set
    */
    public HelpSet getHelpSet() {
      return m_hs;
    }//getHelpSystem

   /**
    * Returns the container for the HistoryWindow.
    * @return The HistoryWindow container
    * @see visalg.graphics.HistoryWindow
    */
    public HistoryWindowContainer getHistoryWindowContainer() {
    	return m_applicationWindow;
    }//getHistoryWindowContainer

    /**
     * Returns the ProjectWindow container.
     * @return The ProjectWindow container
     * @see visalg.graphics.ProjectWindow
     */
    public ProjectWindowContainer getProjectWindowContainer() {
    	return m_applicationWindow;
    }//getProjectWindowContainer

    /**
     * Returns the container for the ModuleWindow.
     *
     * @return The ModuleWindow container.
     *
     * @see visalg.graphics.ModuleWindow
     */
    public ModuleWindowContainer getModuleWindowContainer() {
    	return m_applicationWindow;
    }//getModuleWindowContainer

    /**
     * Will be called, after a DeSerialisation (Open Project) has taken place.
     * The call will be passed to the History.
     */
    public void afterDeserialisation() {
        newWindows();
        m_history.afterDeserialisation();
    }//AfterDeserialisation
} // class Application
