////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : DataModule.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:55 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 10:16:42 2000
//  Update count     : 15
//  CVS Info         : $Id: DataModule.java,v 1.3 2002/03/18 12:28:37 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  10.01.01    vigelius    methoden thread safe synchroniziert
//  26.02.01    vigelius    enhanced documentation
//  14.03.01    vigelius    module now writes the filename of the data to its
//                          string
//  16.05.01    vigelius    data module is now module manager
//  17.05.01    vigelius    data module is now PropertyManager
//  27.06.01    vigelius    closes windows on discardWindows()
//  29.10.01    vigelius    Added reset API
//  20.11.01    vigelius    Removed serialisation bugs
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import java.lang.reflect.Array;
import java.io.File;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import visalg.basics.AbstractModule;
import visalg.basics.ObjectReader;
import visalg.graphics.DataModuleWindow;
import visalg.types.VisAlgMatrix;
import visalg.types.PropertyObject;
import visalg.graphics.ModuleWindowContainer;
import visalg.graphics.MenuBarContainer;
import visalg.types.VisAlgData;

/**
 * The DataModule is a simple module, that manages data for other modules.
 * The DataModuleWindow allows the user to read in data or to convert it from
 * ASCII files.
 *
 * @see visalg.basics.Converter
 * @see visalg.graphics.DataModuleWindow
 */
public class DataModule extends AbstractModule
    implements ModuleManager, PropertyManager {

    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Special Icon for DataModule.
     *
     * @serial The icon for the data module
     **/
    private static ImageIcon s_icon = new ImageIcon("images/dataModule.gif");

    /**
     * Reference to the data which is hold.
     *
     * @serial Referenz to the data which is hold
     */
    protected Object m_data;

    /**
     * Reference to a copy of the data. This is needed by the reset() method.
     *
     * @serial Reference to a copy of the data.
     */
    protected Object m_clonedData;

    /**
     * List of all properties of the data module.
     */
    private  Vector m_properties = new Vector();

    /**
     * All ViewerModules that are associated with the current data.
     */
    private Vector m_viewerModules = new Vector();

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * The constructor sets up the module with the name "Data Module" an
     * sets the data to null.
     *
     * @param mm The module manager, which is associated with this module
     */
    public DataModule(ModuleManager mm) {
    	super("Data Module", mm);
	    m_data = null;
    }//DataModule(ModuleManager)

    /**
     * Returns the data.
     *
     * @return The data, null if none is loaded yet.
     */
    synchronized public Object getData() {
    	return m_data;
    }//getData()

    /**
     * Returns the data type of the data object hold.
     *
     * @return The data type, null if none is loaded yet.
     */
    synchronized public Class getDataClass() {
    	if (m_data != null) {
	        return m_data.getClass();
    	}
	    return null;
    }//getDataClass()

    /**
     * Tries to read out data of the given file. After that, fireEvent() is
     * called in order to notify the connected listeners.
     *
     * @see visalg.basics.AbstractModule#fireEvent()
     *
     * @param f Reference to the file, out of which the data should be loaded
     *
     * @return True, if the loading process has been successfull
     */
    synchronized public boolean loadData(File f) {

    	ObjectReader or = new ObjectReader(f);

	    if (or.object == null) {
	        return false;
    	}

	    m_data = or.object;
        cloneData();

        setString(f.getName());
        closeViewers();
        registerViewers();
        fireEvent();

	    return true;
    }//loadData(File)

    /**
     * Tries to save the current data to the given file.
     *
     * @param f The file, that the data should be stored in.
     *
     * @return True, if the saving process has been successfull.
     */
    synchronized public boolean saveData(File f) {

	boolean returnValue = true;

    	try {
	        ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(f));
	        out.writeObject(m_data);
    	    out.close();
    	}
	    catch (IOException e) {
	        System.err.println(e);
    	    returnValue = false;
	    }

    	return returnValue;
    }//saveData(File)

    /**
     * Sets a new data object. After that, fireEvent() is called.
     *
     * @see visalg.basics.AbstractModule#fireEvent()
     *
     * @param o The data object to be set
     */
    synchronized public void setData(Object o) {
        m_data = o;
        fireEvent();
    }//setData(Object)

    /**
     * Sets a new data object. The difference to setData(Object) is, that
     * after setting a new object, cloneData() will be called. This function
     * should be used rather than setData() to create an initial data set.
     */
    synchronized public void setNewData(Object o) {
        m_data = o;
        cloneData();
        closeViewers();
        registerViewers();
        fireEvent();
    }// setNewData(Object)

    /**
     * Resets the data.
     */
    synchronized public void reset() {
        m_data = m_clonedData;
        cloneData();
        m_properties.clear();
        fireEvent();
    }//reset();

    /**
     * Closes all viewer windows and unregisters them as well as all
     * properties.
     */
    public void closeViewers() {
        Vector viewerModules = m_viewerModules;
        for (Enumeration e=m_viewerModules.elements(); e.hasMoreElements();) {
            ViewerModule vm = (ViewerModule) e.nextElement();
            vm.discardWindows();
        }

        m_viewerModules.removeAllElements();
        m_properties.removeAllElements();
    }//closeViewers()

    /**
     * Registers the viewer modules of the currently set data.
     */
    public void registerViewers() {
        if (m_data instanceof VisAlgData) {
            ((VisAlgData)m_data).registerViewerModules(this,this);
        }
    }
    /**
     * Prints the data to stdout.
     */
    synchronized public void printData(){
        if (m_data == null) {
            System.out.println("No data available !");
        } else {
            ((VisAlgData)m_data).printData();
        }
    	/**f (m_data == null) {
	        System.err.println("No data available!");
	        return;
	    }

    	// VisAlgMatrix
	    if (m_data instanceof VisAlgMatrix) {

	        VisAlgMatrix matrix = (VisAlgMatrix)m_data;

    	    for (int i = 0; i < matrix.rows; i++) {
	        	for (int j = 0; j < matrix.columns; j++) {
		            System.out.print("" + matrix.getValue(i, j) + "\t");
        		}
		        System.out.println("");
    	    }
	        return;
        }

    	// Array
	    if (m_data.getClass().isArray()) {

	        Object[] array = (Object[])m_data;

    	    for (int i = 0; i < array.length; i++) {
	        	System.out.println("" + array[i]);
	        }
    	    return;
	    }

    	// Type unknown
	    System.err.println("DataModule.java: Type of data unknown.");
    	System.err.println("" + m_data); **/
    } // printData()

    /**
     * Makes a copy of the current data. The data can therefore be reset
     * later by calling reset().
     *
     * This method makes use of the clone() API method. Therefore, every
     * type of VisAlg data must take care of the special agreements for the
     * clone() method. This is, that calling up clone(), performs a field copy
     * of the cloned object.
     */
    private void cloneData() {
        try {
            m_clonedData = ((VisAlgData) m_data).cloneData();
        }
        catch (Exception e) {
            System.err.println(e);
        }
    }
    // ----------- from class AbstractModule (resp. interface Module) ------------------

    /**
     * The step() call will be ignored.
     */
    public void step() {
    }

    /**
     * Creates a DataModuleWindow. If the creation failed, an error message
     * will be sent to the standard error output.
     * @see visalg.graphics.DataModuleWindow
     */
    public void newModuleWindow() {
    	try {
	        m_moduleWindow =
		    new DataModuleWindow(m_name,
			    	     this,
				         m_moduleWindowBounds,
				         m_moduleManager.getModuleWindowContainer());
    	}
	    catch (Exception e) {
	        System.err.println(e);
    	}

        // Reopen the viewer Modules
        Enumeration enum = m_viewerModules.elements();
        while (enum.hasMoreElements()) {
            ViewerModule vm = (ViewerModule) enum.nextElement();
            if (vm.isShowing()) {
                vm.newModuleWindow();
            }
        }

    }//newModuleWindow()

    /**
     * Returns icon for DataModule.
     *
     * @return The icon
     */
    public Icon getNodeIcon() {
    	return s_icon;
    }//getNodIcon()

    // -------------- Methods for handling properties ------------------

    /**
     * Adds a property to the list of properties.
     *
     * @PARAM property Property to add
     */
    public void addProperty(PropertyObject property)
    {
        if (m_properties == null) {
            System.out.println("Error : m_properties = null");
        } else if (property == null) {
            System.out.println("Error ; property = null");
        } else {
            m_properties.add(property);
        }
        propertyChanged();
    }//addProperty(PropertyObject)

    /**
     * Removes the specified property from the list.
     *
     * @PARAM property Property to remove
     */
    public void removeProperty(PropertyObject property)
    {
        m_properties.remove(property);
        propertyChanged();
    }//removeProperty(PropertyObject)

    /**
     * Throws an property changed event.
     */
    public void propertyChanged() {
        for (Enumeration e = m_viewerModules.elements(); e.hasMoreElements();) {
            ((ViewerModule)e.nextElement()).propertyChanged();
        }
    }
    /**
     * Returns a list of all properties, that are specified by a certain name.
     *
     * @PARAM name The name of the queried properties
     *
     * @RETURN All properties, that contain the specified name
     */
    public Vector queryPropertyObjects(String name)
    {
        Vector returnVector = new Vector();
        PropertyObject current;

        for (Enumeration e = m_properties.elements() ; e.hasMoreElements() ;) {
            current = (PropertyObject) e.nextElement();
            if (current.getPropertyName().equals(name)) {
                returnVector.add(current);
            }
        }
        return returnVector;
    }//QueryPropertyObjects(String)

    // --------------- from Interface : ModuleManager -----------------------
    /**
     * Adds a sub module to the list of submodules and invokes repaint() at the
     * associated ModuleManager.
     *
     * @return True
     */
    public boolean addModule(Module m) {
        m_viewerModules.add(m);
        getModuleManager().repaint();
        return true;
    }//addModule(Module)

    /**
     * Removes the specified submodule out of the list.
     *
     * @return True, if the submodule could be removed, else : false.
     */
    public boolean removeModule(Module m) {
        boolean b;

        b=m_viewerModules.remove(m);
        getModuleManager().repaint();
        return b;
    }//removeModule(Module)

    /**
     * Swaps the position of two specified modules.
     */
    public void swapModules(int a, int b) {
	    if ((a<0)
            || (b<0)
            || (a>=m_viewerModules.size())
            || (b>=m_viewerModules.size())) {
                return;
            }
    	m_viewerModules.set(a, m_viewerModules.set(b, m_viewerModules.get(a)));
        getModuleManager().repaint();
    }//swapModules(int,int)

    /**
     * Returns all submodules. This method overrides getModules() of
     * AbstractModule.
     *
     * @see AbstractModule#getModules()
     */
    public Vector getModules() {
        return m_viewerModules;
    }
    /**
     * Returns null, since all submodules are viewermodules, that just need to
     * know one data module : this one.
     * @return All Data modules.
     */
    public Vector getDataModules() {
        return null;
    }//getDataModules()

    /**
     * Returns the ModuleWindowContainer. By default, the MWC of this module
     * will be returned. This function can be overwritten by derived classes,
     * if the derived class provides an MWC itself.
     *
     * @return The ModuleWindowContainer
     */
    public ModuleWindowContainer getModuleWindowContainer() {
        return getModuleManager().getModuleWindowContainer();
    }//getModuleWindowContainer

    /**
     * Does nothing in this implementation, since all submodules of the data
     * module are viewermodules.
     */
    public void historyStep() {
    }//historyStep()

    /**
     * Repaints the list of sub modules. This call will just be passed to the
     * ModuleManager of this module.
     */
    public void repaint() {
        getModuleManager().repaint();
    }//repaint()

    /**
     * Returns the MenuBarContainer for the submodules. This is by default
     * the MBC of the associated ModuleManager. This function may be overwritten,
     * if a derived class provides an own MBC.
     */
    public MenuBarContainer getMenuBarContainer() {
        return getModuleManager().getMenuBarContainer();
    }//getMenuBarContainer()

    /**
     * Will be called to close the windows. This calls up superclass method and
     * also traverses the viewer modules and calls discardWindows() on them.
     */
    public void discardWindows() {
        super.discardWindows();

        Enumeration enum = m_viewerModules.elements();
        ViewerModule next;

        while (enum.hasMoreElements()) {
            next = (ViewerModule) enum.nextElement();
            next.discardWindows();
        }
    }//discardWindows()

    public void afterDeserialisation() {
        /**m_viewerModules.removeAllElements();
        ((VisAlgData)m_data).registerViewerModules(this, this);**/
    }//afterDeserialisation()
} // class DataModule