////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : History.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:49 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 22 16:49:13 2000
//  Update count     : 12
//  CVS Info         : $Id: History.java,v 1.2 2002/03/18 12:28:37 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  08.01.01    vigelius    m_isSteppable hinzugefuegt mit methode
//  10.01.01    vigelius    step() synchronized
//  21.01.01    vigelius    fixed small bug at loadProject
//  21.01.01    vigelius    fixed undo() method
//  26.01.01    vigelius    enhanced documentation
//  15.03.01    vigelius    increased thread performance
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import java.io.Serializable;
import java.util.Vector;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.awt.Dimension;

import visalg.basics.Application;
import visalg.basics.Project;
import visalg.graphics.HistoryWindow;
import visalg.graphics.MenuBarContainer;
import visalg.graphics.HistoryWindowContainer;
import visalg.graphics.ProjectWindowContainer;
import visalg.graphics.ModuleWindowContainer;


/**
 * The History is responsible for managing projects, which means, it can,
 * besides the current project, keep different other projects as well. This
 * allows the user to test different alternatives or save important moments.
 * The user thereby gains the possibility to create more complex lessons.
 *
 * @see visalg.basics.Project
 */
public class History implements Serializable {
    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the HistoryWindow (the view).
     */
    private transient HistoryWindow m_historyWindow;

    /**
     * Reference to the Application. Is for example needed to get
     * information where to put the HistoryWindow. */
    private transient Application m_application;

    /**
     * Projects, which are saved automatically in order to realise an
     * undo-functionality.
     * @serial Array of the projects.
     */
    private Project[] m_undoHistory;

    /**
     * Number of projects, which are saved for the undo-function.
     *
     * @serial Number of projects
     */
    private static int s_undoHistorySize = 0;

    /**
     * Projects, which haven been saved by the user.
     *
     * @serial User-saved projects.
     */
    private Vector m_history;

    /**
     * The current project.
     * @serial The current project.
     */
    private Project m_currentProject;

    /**
     * Indicates, if the history is currently stepping (True, if this is
     * the case).
     *
     * @see visalg.basics.StepThread
     *
     * @serial True, if the History is currently stepping
     */
     private boolean m_isStepping;

    /**
     * Reference to the thread which is needed for stepping.
     */
    private transient StepThread m_stepThread;

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * The constructor sets up a new project and creates a new StepThread,
     * if currently none is available. It creates a new array for the undo
     * function, if needed, as well.
     *
     * @param The application which creates the History.
     *
     * @see visalg.basics.StepThread
     */
    public History(Application a) {
    	// Applikation merken
	    m_application = a;

    	// leeres Projekt anfangen
	    m_currentProject = new Project(this);

    	// history initialisieren...
	    m_history = new Vector(0,1);

        // initialise the undoHistory
    	if (s_undoHistorySize > 0) {
	        m_undoHistory = new Project[s_undoHistorySize];
    	} else {
	        m_undoHistory = null;
    	}

          // The history is not stepping at the beginning

          m_isStepping=false;

        /*
          // Set new thread, if not already existing

          if (m_stepThread==null) {
            m_stepThread=new StepThread(this, 100);
            m_stepThread.start();
          } else
          {
            System.err.println("Error in History:History : Thread already existing !");
          }*/
    } // History(Application a)

    /**
     * Sets up the reference to the application and sets the History of the
     * current project. This is necessary after deserialisation.
     *
     * @param a The application for this History
     */
    public void setApplication(Application a) {
    	m_application = a;
	    m_currentProject.setHistory(this);
    }//setApplication(Application)

    /**
     * Creates a new HistoryWindow and calls up Project.newWindows() in order
     * to restore the module windows.
     *
     * @see visalg.graphics.HistoryWindow
     * @see visalg.basics.Project#newWindows()
     */
    public void newWindows() {

    	// Create HistoryWindow
	    if (m_application != null) {

	        // Get the HistoryWindowContainer from the Application
    	    HistoryWindowContainer hwc =
	    	m_application.getHistoryWindowContainer();

	        // If it exists, create HistoryWindow
    	    if (hwc != null) {
	    	    try {
		            m_historyWindow = new HistoryWindow(this, hwc);
        		} catch (Exception e) {
	        	    System.err.println(e);
		        }
    	    }

	        // Create the windows of the current project
	        m_currentProject.newWindows();
    	}
    }//newWindows()

    /**
     * Discards all module windows. This is done by calling
     * Project.discardWindows() and after that
     * ModuleWindowContainer.discardWindows(). It should work also by just
     * calling one of these two.
     *
     * @see visalg.basics.Project#discardWindows()
     * @see visalg.graphics.ModuleWindowContainer#discardModuleWindows()
     */
    public void discardWindows() {
    	m_currentProject.discardWindows();
	    ModuleWindowContainer mwc = getModuleWindowContainer();
    	if (mwc==null) return;
	    mwc.discardModuleWindows();
    }//discardWindows()

    /**
     * Restores the windows of the current project. This is done by first
     * calling Project.discardWindows() and then restoring it by calling
     * Project.newWindows()
     *
     * @see visalg.basics.Project#discardWindows()
     * @see visalg.basics.Project#newWindows()
     */
    public void restoreWindows() {
     	m_currentProject.discardWindows();
        m_currentProject.newWindows();
    }//restoreWindows()


    /**
     * Gives back a reference to the container for the menu.
     *
     * @return The menubar container, null if none is available
     * @see visalg.basics.Application#getMenuBarContainer()
     */
    public MenuBarContainer getMenuBarContainer() {
    	if (m_application != null) {
	        return m_application.getMenuBarContainer();
    	}
	    return null;
    }//getMenuBarContainer

    /**
     * Gives back the container for the ProjectWindow.
     *
     * @return Reference to the ProjectWindowContainer, null if none is
     * available.
     *
     * @see visalg.basics.Application#getProjectWindowContainer()
     */
    public ProjectWindowContainer getProjectWindowContainer() {
    	if (m_application != null) {
	        return m_application.getProjectWindowContainer();
    	}
	    return null;
    }//getProjectWindowContainer

    /**
     * Returns the container for the module windows.
     *
     * @return Reference to the ModuleWindowContainer, null if none is
     * available.
     *
     * @see visalg.basics.Application#getModuleWindowContainer()
     */
    public ModuleWindowContainer getModuleWindowContainer() {
    	if (m_application != null) {
	        return m_application.getModuleWindowContainer();
    	}
	    return null;
    }//getModuleWindowContainer()

    /**
     * Returns a reference to the current history window.
     *
     * @return The current History window
     */

     public HistoryWindow getHistoryWindow() {
        return m_historyWindow;
    }//getHistoryWindow

    /**
     * Gives back the state of the history.
     * @return True, if the history is currently stepping. False, if not.
     */
    public boolean isStepping() {
        return m_isStepping;
    }//isStepping()

    /**
     * Sets the state of the history.
     * @param True, if the history is stepping, else : false
     */
    public void setStepping(boolean b) {
      m_isStepping=b;
        if (b==true) {
            // create a new thread and start it
            m_stepThread = new StepThread(this,100);
            m_stepThread.start();
        } else {
            m_stepThread.setActive(false);
        }
    }//setStepping(boolean)

    /**
     * Sets the delay time for the step thread.
     * @param time The delay time in milliseconds
     * @see visalg.basics.StepThread
     * @see visalg.basics.StepThread#setDelayTime(int)
     */
    public void setDelayTime(int time) {
      if (m_stepThread!=null) {
        m_stepThread.setDelayTime(time);
      }
    }//setDelayTime(int)

    /**
     * Steps every module in the History. This is done by calling the
     * member method Module.step(). Furthermore, the current project is
     * stored in the undo arary.
     *
     * @see visalg.basics.Module#step()
     * @see visalg.basics.History#undo()
     */
    synchronized public void step() {
    	historyStep();
    }//step()

    /**
     * Loads the last project of the undo History and restores its windows.
     *
     * @see visalg.basics.Project#newWindows()
     */
    public void undo() {
    	//wenn noch etwas in der undoHistory ist...
	    Project p = m_undoHistory[0];
    	if (p!=null) {
	        //...alles nach vorne schieben (hinten wird null eingefgt...)
	        for (int i=0; i < m_undoHistory.length - 1; i++) {
    		    m_undoHistory[i] = m_undoHistory[i+1];
	        }
	        m_undoHistory[m_undoHistory.length - 1] = null;

            // discard old windows
            m_currentProject.discardWindows();

            // set new project and the right history (this one)
            m_currentProject = p;
            m_currentProject.setHistory(this);

            // create new windows
    	    m_currentProject.newWindows();
	    }
    }//undo()

    //HISTORY/PROJECT-VERWALTUNG

    /**
     * Returns the current Project.
     *
     * @return Reference to the current Project
     */
    public Project getProject() {
    	return m_currentProject;
    }//getProject()

    /**
     * Returns the number of saved projects. This is not affected by the
     * undoHistory.
     *
     * @return Number of user-saved projects
     */
    public int getHistorySize() {
    	return m_history.size();
    }//getHistorySize()

    /**
     * Returns the project at a certain place.
     *
     * @param i The place of the project
     * @return Reference to the i. project
     */
    public Project getProject(int i) {

    	Project p = null;
	    try {
	        p = (Project)m_history.get(i);
    	} catch (Exception e) {
	        System.err.println(e);
	        return null;
    	}
	    return p;
    }//getProject(int)

    /**
     * Duplicates a certain project and sets it as the current one. After that,
     * the windows are restored.
     *
     * @param i Number of the chosen project
     * @return True, if successfull
     *
     * @see visalg.basics.Project#newWindows()
     */
    public boolean loadProject(int i) {
	    //erst Projekt abholen...
    	Project p = null;
	    try {
	        p = (Project)m_history.get(i);
    	} catch (Exception e) {
	        return false;
    	}
	    //...erneut serialisieren...
    	if (p==null) return false;
	    try {
	        ByteArrayOutputStream bout = new ByteArrayOutputStream();
    	    ObjectOutputStream out = new ObjectOutputStream(bout);
	        out.writeObject(p);
	        out.close();
    	    p = null;
	        ByteArrayInputStream bin = new ByteArrayInputStream(bout.toByteArray());
	        ObjectInputStream in = new ObjectInputStream(bin);
    	    p = (Project)in.readObject();
	        in.close();
    	} catch (Exception e) {
	        return false;
    	}
	    if (p==null) return false;
      	//... wenn alles OK: setzen und Fenster wiederherstellen...
        // zuerst alle alten Fenstern schliessen
        m_currentProject.discardWindows();
	    p.setHistory(this);
        // set new project
	    m_currentProject = p;
        // restore old windows
        restoreWindows();
     	return true;
    }//loadProject(int)

    /**
     * SaveProject() duplicates the current project and appends it to the
     * list.
     *
     * @return True, if successfull
     */
    public boolean saveProject() {
    	Project p = null;
	    try {
	        ByteArrayOutputStream bout = new ByteArrayOutputStream();
    	    ObjectOutputStream out = new ObjectOutputStream(bout);
	        out.writeObject(m_currentProject);
    	    out.close();
    	    ByteArrayInputStream bin = new ByteArrayInputStream(bout.toByteArray());
	        ObjectInputStream in = new ObjectInputStream(bin);
    	    p = (Project)in.readObject();
    	    in.close();
	    } catch (Exception e) {
	        return false;
    	}
	    if (p==null) return false;

    	p.setHistory(this);
	    m_history.add(p);

    	return true;
    }//saveProject()

    /**
     * Deletes a certain project.
     *
     * @param i The number of the project to be deleted
     */
    public void deleteProject(int i) {
    	Project p = null;
	    try {
	        p = (Project)m_history.remove(i);
    	} catch (Exception e) {
    	}
    }//deleteProject(int)

    /**
     * Sets a new size for the undo array.
     *
     * @param size The new size
     */
     public void setUndoSize(int size)
     {
        s_undoHistorySize = size;
        if (s_undoHistorySize > 0) {
    	    m_undoHistory = new Project[s_undoHistorySize];
	    } else {
	        m_undoHistory = null;
    	}
     }//setUndoSize(int)

    /**
     * Returns the current size of the undo array.
     *
     * @return Current size
     */
    public int getUndoSize() {
        return s_undoHistorySize;
    }//getUndoSize()

    // -----------------------------------------------------
    // ------- M e t h o d s ( p r i v a t e ) -------------
    // -----------------------------------------------------

    //STEPs historyStep() gibt den step() einfach an das aktuelle
    //Projekt weiter.  ... und kmmert sich um die undoHistory

    /**
     * Passes the step() to the modules by calling Module.step(). Furthermore,
     * the current project will be appended to the undo array.
     *
     * @see visalg.basics.Module#step()
     */
    private void historyStep() {
    	if (m_currentProject!=null) {
            if (m_undoHistory!=null) {
				//alle eins nach hinten schieben...
    		    for (int i=m_undoHistory.length - 1; i > 0; i--) {
	    	        m_undoHistory[i] = m_undoHistory[i-1];
    		    }
				//... und am Anfang das serialisierte wieder hinein...
	        	try {
		            ByteArrayOutputStream bout = new ByteArrayOutputStream();
		            ObjectOutputStream out = new ObjectOutputStream(bout);
        		    out.writeObject(m_currentProject);
	        	    out.close();
		            ByteArrayInputStream bin = new ByteArrayInputStream(bout.toByteArray());
		            ObjectInputStream in = new ObjectInputStream(bin);
        		    Project p = (Project)in.readObject();
	        	    in.close();
		            m_undoHistory[0] = p;
    		    } catch (Exception e) {
	    	        return;
    		    }
    	    }
	        m_currentProject.step();
    	}
    }//historyStep()

    /**
     * Will be called by the Application after a Deserialisation has taken
     * place. The call will be passed to the current Project.
     */
    public void afterDeserialisation() {
        m_currentProject.afterDeserialisation();
    }//AfterDeserialisation()
} // class History

