////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : visalg/basics/Module.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:31:45 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 10:58:49 2000
//  Update count     : 8
//  CVS Info         : $Id: Module.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		    Changes/Extensions
//  ----	    ----		    ------------------
// 20.02.01     vigelius        new abstract method : setShowing
// 26.02.01     vigelius        enhanced documentation
// 17.05.01     vigelius        added function afterDeserialisation()
//
///////////////////////////////////////////////////////////////////////////////


package visalg.basics;


import visalg.basics.OtherModule;
import visalg.basics.ModuleManager;

import visalg.graphics.ModuleWindow;
import java.awt.Rectangle;
import java.util.Vector;
import javax.swing.Icon;


/**
 * Module is the interface, that every module must implement. It provides
 * functionality for the ModuleManager. Most of these functions are implemented
 * in the class AbstractModule, which should be used rather than this one.
 *
 * @see visalg.basics.AbstractModule
 * @see visalg.basics.ModuleManager
 */
public interface Module extends OtherModule {


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * discardWindows() should close all associated windows. This is normally
     * the module window itself, but all other windows, that might have been
     * opened by the module, must be closed, too.
     */
    public void discardWindows();

    /**
     * Returns a reference to the ModuleManager.
     *
     * @return The associated ModuleManager
     */
    public ModuleManager getModuleManager();

    /**
     * Returns the icon of this module.
     *
     * @return The icon
     */
    public Icon getNodeIcon();

    /**
     * Returns all connected submodules.
     *
     * @return List of all submodules
     */
    public Vector getSubModules();

    /**
     * Indicates, if the window of this module is currently showing.
     *
     * @return True, if the window is open.
     */
    public boolean isShowing();

    /**
     * Sets the module state to visible/invisible. This does not affect any
     * real windows.
     *
     * @param b True, if the module window is showing
     */
     public void setShowing(boolean b);

    /**
     * Indicates, if the module reacts on stepping requests of the project.
     *
     * @param b True, if it reacts
     */
    public void setStepping(boolean b);

    /**
     * Indicates, if the module reacts on stepping requests.
     *
     * @return True, if it reacts
     */
    public boolean isStepping();

    /**
     * Creates a new ModuleWindow.
     */
    public void newModuleWindow();

    /**
     * Sets a new ModuleManager.
     *
     * @param mm The new ModuleManager
     */
    public void setModuleManager(ModuleManager mm);

    /**
     * Sets new window bounds for the module window. This method is invoked by
     * the module manager, if the user has changed the window properties. If
     * the window changes its properties itself, it should call this function,
     * to make sure, that it can be restored (for example after serialisation).
     *
     * @param visible True, if the window is visible
     * @param r The window bounds
     */
    public void setModuleWindowBounds(boolean visible, Rectangle r);

    /**
     * This function processes the specific code for this module.
     * It is called, if a either the user or the StepThread invokes a
     * step() on the History.
     *
     * @see visalg.basics.StepThread
     * @see visalg.basics.History#step()
     */
    public void step();

    /**
     * This method will be called, after the object has been deserialised. All
     * necessary initialisation should be done here.
     */
    public void afterDeserialisation();

    /**
     * This method returns the currently opened window (if one), otherwise null.
     *
     * @return Reference to its window (if one), otherwise null
     */
    public ModuleWindow getModuleWindow();
} // interface Module