////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : StepThread.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            : mvigelius@gmx.de
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Feb 27 13:43:00 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  15.03.01    vigelius    increased thread performance
//
///////////////////////////////////////////////////////////////////////////////

package visalg.basics;

/**
 * The StepThread class is simply a Thread, which continously calls the step()
 * method of the associated History. This thread must be started first by calling
 * start(). If the thread is set to "inactive", it cannot be revived, due to the
 * Java API Securicy Police. It has to be recreated.  After invoking
 * a History step, the thread delays a predefined time.
 *
 * @see visalg.basics.History#History(visalg.basics.Application)
 * @see visalg.basics.History#step()
 */
public class StepThread extends Thread {

    /**
     * The time, for which the thread delays.
     */
    private int m_delayTime;

    /**
     * Referenc to the History.
     */
    private History m_history;

    /**
     * Indicates, if the thread is currently active (True) or not (False).
     */
    private boolean m_isStepping;

    /**
     * Constructs a new StepThread class, that is connected to a History and
     * delays a predefined time. In order to work, the thread has to be
     * started by calling run() first.
     *
     * @param h Reference to the History
     * @param time Initial delay time
     *
     * @see visalg.basics.StepThread#run()
     */
    public StepThread(History h, int time) {
        super("Step Thread !");
        m_delayTime=time;
        m_history=h;
        m_isStepping=true;
    }//StepThread(History, time)

    /**
     * Starts the execution of the thread. The thread has to be set active to
     * perform steps.
     *
     * @see visalg.basics.StepThread#setActive(boolean)
     */
    public void run() {
        while (m_isStepping==true) {
            m_history.step();
            try {
                sleep(m_delayTime);
            } catch (InterruptedException e) {}
        }
    }//run()

    /**
     * Changes the delay time of the thread.
     *
     * @param value Delay time in milliseconds
     */
    public void setDelayTime(int value) {
        m_delayTime=value;
    }//setDelayTime(int)

    /**
     * Sets the threads state to "active" or "inactive".
     *
     * @param b True, if the thread should be active
     */
    public void setActive(boolean b) {
        m_isStepping=b;
    }//setActive(boolean)
}//class StepThread