////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ViewerModule.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Wed May 16 15:10:00 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
///////////////////////////////////////////////////////////////////////////////

package visalg.basics;

import visalg.types.ModuleChangeEvent;

/**
 * This class is the superclass for all viewer modules. A viewer module is
 * connected to exactly one data module, that must be specified at the
 * constructor.
 *
 * This implementation contains a function to return the associated data module
 * and also implements propertyChanged(), which invokes a repaint() on the
 * window.
 */
public class ViewerModule extends AbstractModule implements ModuleListener {

    /**
     * Reference to the associated DataModule.
     */
    private DataModule m_dataModule;

    /**
     * Creates a new viewer module, that is connected to the specified
     * ModuleManager and the DataModule, which are in most cases identically.
     *
     * @param name The name of the viewer module
     * @param mm The ModuleManager (mostly the DataModule itself)
     * @param dm The DataModule
     */
    public ViewerModule(String name, ModuleManager mm, DataModule dm) {
        super(name, mm);
        m_dataModule = dm;
        // register as a listener to the data module
        m_dataModule.addModuleListener(this);
    }//ViewerModule(String, ModuleManager, DataModule)

    /**
     * Must be overwritten in order to open the convenient window.
     */
    public void newModuleWindow() {
    }//newModuleWindow

    /**
     * Will be called by the data module, if one of its properties has changed.
     * This implementation invokes repaint() on the associated window (if one).
     */
    public void propertyChanged() {
        if (m_moduleWindow != null) {
            m_moduleWindow.repaint();
        }
    }//propertyChanged()

    /**
     * Returns the associated DataModule.
     *
     * @return The DataModule
     */
    public DataModule getDataModule() {
        return m_dataModule;
    }//getDataModule()

    /**
     * Can be overwritten to do some initialisation stuff here.
     */
    public void afterDeserialisation() {
    }//afterDeserialisation

    /**
     * Just calls fireEvent() in order to notify the connect window (if any).
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
        fireEvent();
    }//moduleChanged(ModuleChangeEvent)
}//class ViewerModule