////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : OpenViewerDialog.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            : mvigelius@gmx.de
//  Copyright        : Uni Heidelberg
//  Created on       : Mon Apr 09 2001
//  Last modified by : -
//  Last modified on : -
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  18.06.01    vigelius    viewer modules are represented by a list box.
//
///////////////////////////////////////////////////////////////////////////////

package visalg.components;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.Vector;

import visalg.basics.*;
import visalg.modules.*;
import javax.swing.*;

/**
 * This class represents a dialog, wich allows the user to choose a viewer
 * module. The available viewer modules are provided by the appropriate
 * data model. This dialog will be opened by the DataModule.
 *
 * @see visalg.types.VisAlgData
 * @see visalg.basics.DataModule
 */
public class OpenViewerDialog extends JDialog {

    /**
     * The exit panel containing the ok and the cancel button.
     */
    private JPanel m_exitPanel = new JPanel();

    /**
     * The layout of the dialog.
     */
    private BorderLayout borderLayout1 = new BorderLayout();

    /**
     * The OK button.
     */
    private JButton m_okButton = new JButton();

    /**
     * The cancel button.
     */
    private JButton m_cancelButton = new JButton();

    /**
     * The flow layout for the exit panel.
     */
    private FlowLayout flowLayout1 = new FlowLayout();

    /**
     * The panel for the components.
     */
    private JPanel m_viewerPanel = new JPanel();

    /**
     * The layout for the component panel.
     */
    private GridLayout gridLayout1 = new GridLayout();

    /**
     * The dialog text.
     */
    private JLabel m_openLabel = new JLabel();

    /**
     * The list of the available modules.
     */
    private JList m_viewerList;

    /**
     * All available modules.
     */
    private Vector m_viewerModules;

    /**
     * Sets up the main panel and reads out all available modules. This is done
     * by asking the ModuleManager. In order to make this possible, the viewer
     * modules need to be registered by the ModuleManager first.
     *
     * @see visalg.basics.DataModule
     * @see visalg.basics.ModuleManager
     * @param mm The associated ModuleManager
     */
    public OpenViewerDialog(ModuleManager mm) {

        m_viewerModules = mm.getModules();

        m_viewerList = new JList(m_viewerModules);
        m_viewerList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }//OpenViewerDialog(ModuleManager)

    /**
     * Sets up all components.
     */
    private void jbInit() throws Exception {
        this.setModal(true);
        this.setTitle("Open Viewer ...");
        this.getContentPane().setLayout(borderLayout1);
        m_exitPanel.setLayout(flowLayout1);
        m_okButton.setText("Open");
        m_okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okActionPerformed(e);
            }
        });
        m_cancelButton.setText("Cancel");
        m_cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelActionPerformed(e);
            }
        });
        m_viewerPanel.setLayout(gridLayout1);
        gridLayout1.setRows(2);
        m_openLabel.setText("Open Viewer :");
        this.getContentPane().add(m_exitPanel, BorderLayout.SOUTH);
        m_exitPanel.add(m_okButton, null);
        m_exitPanel.add(m_cancelButton, null);
        this.getContentPane().add(m_viewerPanel, BorderLayout.CENTER);
        m_viewerPanel.add(m_openLabel, null);
        m_viewerPanel.add(new JScrollPane(m_viewerList), null);
        setBounds(50,50,300,200);
    }//jbInit()

    /**
     * Calls openSelectedViewer() and closes the dialog,
     * if the user presses the ok button.
     *
     * @see OpenViewerDialog#openSelectedViewer()
     * @param e The ActionEvent
     */
    void okActionPerformed(ActionEvent e) {
        if (m_viewerList.getSelectedValue()!=null) {
            openSelectedViewer();
        }

        dispose();
    }//okActionPerformed(ActionEvent)

    /**
     * Closes the dialog without anything done.
     * @param e The ActionEvent
     */
    void cancelActionPerformed(ActionEvent e) {
        dispose();
    }//cancelActionPerformed(ActionEvent)

    /**
     * Opens the selected viewer module window by calling
     * Module.newModuleWindow().
     *
     * @see visalg.basics.Module#newModuleWindow()
     */
    private void openSelectedViewer() {
        Module m = (Module) m_viewerList.getSelectedValue();
        m.newModuleWindow();
    }//openSelectedViewer()
}//OpenViewerDialog