////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PreferencesDialog.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            : mvigelius@gmx.de
//  Copyright        : Uni Heidelberg
//  Created on       : Mon Jan 15 2001
//  Last modified by : -
//  Last modified on : -
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  21.02.01    vigelius        added undo size tab
//  27.02.01    vigelius        Enhanced documentation
//
///////////////////////////////////////////////////////////////////////////////
package visalg.components;

import java.lang.Number.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import visalg.basics.*;
import visalg.graphics.*;

/**
 * This class represents the preferences dialog, which is shown to the user in
 * order to let him set up certain values. It consists of different tabs, that
 * can hold seperate logically connected options. If the user chooses the OK
 * button, saveValues() is called, which stores the values using the appropriate
 * member functions.
 *
 * @see visalg.components.PreferencesDialog#saveValues()
 */
public class PreferencesDialog extends JDialog {

    /**
     * Reference to the History, that is passed to the constructor.
     */
    private History m_history;
    /**
     * Reference to the HistoryWindow passed to the constructor
     */
    private HistoryWindow m_historyWindow;

    // ------------------------Finish Panel ----------------------------------
    /**
     * The panel containing the ok and the cancel button.
     */
    private JPanel m_finishPanel;

    /**
     * The OK Button.
     */
    private JButton m_okButton;

    /**
     * The CANCEL button.
     */
    private JButton m_cancelButton;

    // -------------------------Main Panel -----------------------------------
    /**
     * The main pane containing the panels.
     */
    private JTabbedPane m_mainPane;

    // ----------------------- Slider Panel ----------------------------------

    /**
     * The panel for the set up of the slider borders.
     */
    private JPanel m_delayPanel;

    /**
     * The start value for the slider.
     */
    private JTextField m_delayStartText;

    /**
     * The end value for the slider.
     */
    private JTextField m_delayEndText;

    // ------------------------- Undo Panel ---------------------------------
    /**
     * The panel containing the undoHistory size
     */
    private JPanel m_undoPanel;

    /**
     * Value of the undoHistory size
     */
    private JTextField m_undoSize;

    /**
     * Creates the components of the dialog and shows it.
     *
     * @param owner Owner of the dialog (usually the ApplicationWindow)
     * @param history Reference to the History
     */
    public PreferencesDialog(JFrame owner, History history) {
        super(owner,"Preferences", true);
        m_history=history;
        m_historyWindow=m_history.getHistoryWindow();
        createComponents();
        setBounds(50,50,300,200);
        show();
    }//PreferencesDialog(JFrame, History)

    /**
     * Creates the GUI components of the dialog. The inner class FinishAction
     * is connected to the OK and the CANCEL button and is thereby invoked, if
     * the user presses one of these.
     */
    private void createComponents()
    {
        // Create OK/Cancel Buttons
        m_okButton=new JButton("OK");
        m_okButton.setActionCommand("OK");
        m_okButton.addActionListener(new FinishAction());
        m_cancelButton=new JButton("Cancel");
        m_cancelButton.setActionCommand("Cancel");
        m_cancelButton.addActionListener(new FinishAction());

        // Setup delay Panel
        m_delayPanel = new JPanel();
        m_delayPanel.setLayout(new GridLayout(2,2));
        m_delayStartText =
            new JTextField(
                (new Integer(m_historyWindow.getDelayTimeMinimum())).toString(),
                 20);
        m_delayEndText =
            new JTextField(
                (new Integer(m_historyWindow.getDelayTimeMaximum())).toString(),
                 20);
        m_delayPanel.add(new JLabel("Minimum Value (ms) :"));
        m_delayPanel.add(m_delayStartText);
        m_delayPanel.add(new JLabel("Maximum Value (ms) :"));
        m_delayPanel.add(m_delayEndText);

        // Setup Undo Panel
        m_undoPanel = new JPanel();
        m_undoPanel.setLayout(new GridLayout(1,2));
        Integer i = new Integer(m_history.getUndoSize());
        m_undoSize = new JTextField(i.toString(),20);
        m_undoPanel.add(new JLabel("Number of undos :"));
        m_undoPanel.add(m_undoSize);

        // Setup finish Panel
        m_finishPanel=new JPanel();
        m_finishPanel.add(m_okButton);
        m_finishPanel.add(m_cancelButton);

        // Setup tabbed Panel
        m_mainPane=new JTabbedPane();
        m_mainPane.addTab("Play delay time", m_delayPanel);
        m_mainPane.addTab("Undo size", m_undoPanel);

        // Add panels to container
        getContentPane().add(m_finishPanel, BorderLayout.SOUTH);
        getContentPane().add(m_mainPane, BorderLayout.NORTH);
    }//createComponents()

    /**
     * Saves the values of the dialog using the appropriate member methods.
     */
    public void saveValues() {
        // Save delaytime values
        int min = (new Integer(m_delayStartText.getText())).intValue();
        int max = (new Integer(m_delayEndText.getText())).intValue();
        m_historyWindow.setDelayTimeProperties(min, max);
        // Save undo size
        int size = (new Integer(m_undoSize.getText())).intValue();
        m_history.setUndoSize(size);
    }//saveValues()

    /**
     * The ActionListener for the OK and the CANCEL button. If the user chooses
     * OK, then all values are saved, before disposing the dialog.
     */
    class FinishAction implements ActionListener {

        public void actionPerformed(ActionEvent evt) {
            if (evt.getActionCommand().equals("OK")) {
                saveValues();
            }
            dispose();
        }//actionPerformed(ActionEvent)
    }//class FinishAction
}//class PreferencesDialog