////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : TSPGraphConverter.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////

package visalg.converter;


import visalg.basics.Converter;
import visalg.basics.ObjectReader;
import visalg.types.VisAlgGraph.*;

import java.awt.Color;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;

import javax.swing.JOptionPane;

import java.util.StringTokenizer;
import java.util.Vector;


/**
 * Converts an ASCII file containing different values into a VisAlgGraph object.
 *
 * @see visalg.types.VisAlgGraph.VisAlgGraph
 */
public class TSPGraphConverter implements Converter {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
      *
      */
    private Vector m_nodesVector     = new Vector();
    private Vector m_edgesVector     = new Vector();
    private Vector m_intervalVector = new Vector();
    private VisAlgGraph m_visAlgGraph = new VisAlgGraph();


    // -----------------------------------------------------
    // ------- M e t h o d s  ( p u b l i c ) --------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public TSPGraphConverter() {
    }

    /**
     * Builds a VisAlgGraph- object from a given File.
     * @param f   file to convert
     *
     * @return visAlgGraph   the convertet object, if it worked
     * @return null   if does not work
     */
    public Object getData(File f) {
        String aLine;
        String nameEN; // name of edge or node

        // initializes all with null, that later no error occures
        m_visAlgGraph.setVisAlgEdges(null);
        m_visAlgGraph.setVisAlgIntervals(null);
        m_visAlgGraph.setVisAlgNodes(null);
        m_visAlgGraph.setTSPLibFile(null);
        m_visAlgGraph.setTexFile(null);
        // sets the referenz to the File f
        m_visAlgGraph.setTSPGraphFile(f);

        try {
            BufferedReader dataInFile = new BufferedReader(new FileReader(f));

            // search for first line specifies the type of data
            while (    (aLine = dataInFile.readLine()) != null
                       && !aLine.startsWith("NODE_SECTION")
                       && !aLine.startsWith("EDGE_SECTION")) {
            }

            // return if file end already reached
            if (aLine == null) {
                dataInFile.close();
                System.err.println("Unexpected end of file.\nNo data- sections found!");
                JOptionPane.showMessageDialog(null,
                                              "Unexpected end of file.\nNo data- sections found!",
                                              "TSPConverter error: NO DATA SECTIONS",
                                              JOptionPane.ERROR_MESSAGE);
                dataInFile.close();
                return null;
            } // if (aLine == null)

        // looks what kind of section
            if (aLine.startsWith("NODE_SECTION")) {
                boolean loadedFile = false;
                int temp_i;
                int X = 0;
                int Y = 0;
                StringTokenizer t = new StringTokenizer("");
                VisAlgNode node;
                String name;

                // determin source for data
                while (    (aLine = dataInFile.readLine()) != null
                           && !aLine.startsWith("EDGE_SECTION")
                           && !aLine.startsWith("EDGE_VIS_SECTION")
                           && !aLine.startsWith("EOF")) {
                    t = new StringTokenizer(aLine);

                    switch (t.countTokens()) {
                    case 0: // line without data => jumps to next line
                        break;

                    case 1:
                        aLine = t.nextToken();

                        if(!extractNodes(aLine, f.getAbsoluteFile().getParent())) {
                            m_visAlgGraph.setTSPLibFile(null);
                            dataInFile.close();
                            return null;
                        }

                        // sets no TSPLIB- File if more than one is loaded
                        if (loadedFile) {
                            m_visAlgGraph.setTSPLibFile(null);
                        } // if (loadedFile)
                        else {
                            loadedFile = true;
                        } // else: if (loadedFile)
                        break;

                    case 3:  // aLine should contain the data of a node
                        if(!addNodes(aLine)) {
                            dataInFile.close();
                            return null;
                        } // if(!addNodes(aLine)

                        // is set, that no wrong file- reference will occure
                        loadedFile = true;
                        m_visAlgGraph.setTSPLibFile(null);

                        break;

                    default:  // error
                        System.err.println("Invalid kind of node- data. Three values are needed:");
                        System.err.println("name  ( a String )");
                        System.err.println("x- value  ( Double )");
                        System.err.println("y- value ( Double )");
                        JOptionPane.showMessageDialog(null,
                                                      "Invalid kind of node- data. Three values are needed:"
                                                      + "\nname  ( a String )"
                                                      + "\nx- value  ( Double )"
                                                      + "\ny- value  ( Double )",
                                                      "TSPConverter error: UNKNOWN DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        dataInFile.close();
                        return null;
                    } // switch
                } // while ( !null && !EOF && !EDGE_SECTION)

                // return if file end already reached
                if (aLine == null) {
                    dataInFile.close();

                    // error but nodes ok
                    if (m_nodesVector.size() > 0) {
                        System.err.println("Unexpected end of file. But data available.");

                        if (  JOptionPane.YES_OPTION ==
                              JOptionPane.showConfirmDialog(null,
                                                            "Unexpected end of file. But data available.\nContinue process ?",
                                                            "TSPConverter error: INVALID DATA",
                                                            JOptionPane.YES_NO_OPTION)) {

                            m_nodesVector.trimToSize();

                            // determins the extrem- points
                            if (determineMinMaxXY() == null) {
                                System.err.println("Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.");
                                JOptionPane.showMessageDialog(null,
                                                              "Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.",
                                                              "TSPConverter error: INVALID DATA",
                                                              JOptionPane.ERROR_MESSAGE);
                                dataInFile.close();
                                return null;
                            }  // if (determineMinMaxXY() == null)
                            m_visAlgGraph.setMinMaxXY(determineMinMaxXY());

                            // stores the nodes
                            m_visAlgGraph.setVisAlgNodes(m_nodesVector.toArray());

                            // initializes the interval [0,1] in white
                            m_visAlgGraph.setLeftOpen(false);
                            m_visAlgGraph.setRightOpen(false);
                            m_intervalVector.add(new VisAlgInterval(0,
                                                                    1,
                                                                    Color.white));
                            m_intervalVector.trimToSize();

                            // stores the Objects in the vector as intervals
                    	    VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                            VisAlgInterval vai;

                            for (int i = m_intervalVector.size(); i > 0; i--) {
                                vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                	        iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                                   vai.getUpperLimit(),
                                                                   vai.getColor());
                            } // for (int i = m_edgesVector.size(); i > 0; i--)

                            m_visAlgGraph.setVisAlgIntervals(iArray);

                            return m_visAlgGraph;
                        } // if (JOptionPane.YES_OPTION == ... )
                        else {
                            return null;
                        } // else: if (JOptionPane.YES_OPTION == ... )
                    } // if (m_nodesVector.size() > 0)
                    // error and nodes not ok
                    else {
                        System.err.println("Unexpected end of file.");
                        System.err.println("Not able to read any data from file " + f.getName() + " !");
                        JOptionPane.showMessageDialog(null,
                                                      "Unexpected end of file.\nNot able to read any data from file "
                                                      + f.getName() + " !",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        return null;
                    } // else: if (m_nodesVector.size() > 0)
                } // if (aLine == null)

                if (m_nodesVector.size() != 0) {
                    m_nodesVector.trimToSize();

                    // determins the extrem- points
                    if (determineMinMaxXY() == null) {
                        System.err.println("Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.");
                        JOptionPane.showMessageDialog(null,
                                                      "Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        dataInFile.close();
                        return null;
                    }  // if (determineMinMaxXY() == null)
                    m_visAlgGraph.setMinMaxXY(determineMinMaxXY());

                    // stores the nodes
                    m_visAlgGraph.setVisAlgNodes(m_nodesVector.toArray());
                } // if (m_nodesVector.size() != 0)
            } // end if (aLine.indexOf("NODE_SECTION") != -1)

        // looks what kind of section
            if (aLine.startsWith("EDGE_SECTION")) {
                String start;
                String end;
                StringTokenizer t = new StringTokenizer(aLine);
                int startPos;
                int endPos;
                int temp_i;
                int counter = m_nodesVector.size();
                double weight = 0.0;
                boolean newVec = false;

                // checks if m_nodesVector exist
                // true    means just write the edges
                // false   means build new nodes
                if (counter > 0) {
                  newVec = true;
                }

                // read data for edges
                while (    (aLine = dataInFile.readLine()) != null
                           && !aLine.startsWith("EOF")
                           && !aLine.startsWith("EDGE_VIS_SECTION")) {
                    t = new StringTokenizer(aLine);

                    if (t.countTokens() != 0) {

                        if (  t.countTokens() != 3) {
                            System.err.println("Invalid kind of edge- data. Three values are needed:");
                            System.err.println("start- node  ( a String )");
                            System.err.println("end- node  ( a String )");
                            System.err.println("weight ( Double )");
                            JOptionPane.showMessageDialog(null,
                                                          "Invalid kind of node- data. Three values are needed:"
                                                          + "\nstart- node  ( a String )"
                                                          + "\nend- node  ( a String )"
                                                          + "\nweight  ( Double )",
                                                          "TSPConverter error: UNKNOWN DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if (  t.countTokens() != 3)

                        start = t.nextToken();
                        end = t.nextToken();

                        startPos = findNode(start);
                        endPos   = findNode(end);

                        if (startPos == -2 || endPos == -2) { // wrong instance
                            dataInFile.close();
                            System.err.println("Wrong Object occured in findNode() !\nNot instance of VisAlgNode.");
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong Object occured in findNode() !\nNot instance of VisAlgNode.",
                                                          "TSPConverter error: INVALID DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            return null;
                        } // if (startPos == -2 || endPos == -2)

                        if ( startPos == -1  && !newVec) { // start- edge does not exists
                            VisAlgNode van = new VisAlgNode();

                            van.setName(start);
                            van.setAbsoluteX(0);
                            van.setAbsoluteY(0);

                            m_nodesVector.add(van);
                            counter++;
                            startPos = m_nodesVector.size() - 1;

                            if (endPos == -1 && start.compareTo(end) == 0) {
                                endPos = startPos;
                            }
                        }  // if ( startPos == -1  && !newVec)

                        if ( endPos == -1 && !newVec) { // end- edge does not exists
                            VisAlgNode van = new VisAlgNode();

                            van.setName(end);
                            van.setAbsoluteX(0);
                            van.setAbsoluteY(0);

                            m_nodesVector.add(van);
                            counter++;
                            endPos = m_nodesVector.size() - 1;
                        }  // if ( endPos == -1  && !newVec)

                    // not existing node and the m_nodesVector exists
                        if ( startPos == -1 ) {
                            dataInFile.close();
                            System.err.println("Node " + start + " not found while adding edges.");
                            JOptionPane.showMessageDialog(null,
                                                          "Node " + start + " not found while adding edges.",
                                                          "TSPConverter error: INVALID DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            return null;
                        } // if ( startPos == -1 )

                    // not existing node and the m_nodesVector exists
                        if ( endPos == -1 ) {
                            dataInFile.close();
                            System.err.println("Node " + end + " not found while adding edges. Node");
                            JOptionPane.showMessageDialog(null,
                                                          "Node " + end + " not found while adding edges.",
                                                          "TSPConverter error: INVALID DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            return null;
                        } // if ( endPos == -1 )

                        if (  startPos != endPos) {

                            try {
                                weight = Double.parseDouble(t.nextToken());
                            } // try
                            catch (NumberFormatException e) {
                                System.err.println("Error: " + e);
                                System.err.println("Exception while parsing weight in EDGE_SECTION. Wrong type.");
                                JOptionPane.showMessageDialog(null,
                                                              "Exception while parsing weight in EDGE_SECTION.\nWrong type.",
                                                              "TSPConverter error: INVALID DATA",
                                                              JOptionPane.ERROR_MESSAGE);
                                dataInFile.close();
                                return null;
                            } // catch

                            if (weight < 0.0 || weight > 1.0) {
                                System.err.println("Weight has wrong value. Needed is");
                                System.err.println("weight ( double of [0,1] ).");
                                JOptionPane.showMessageDialog(null,
                                                              "Weight has wrong value. Needed is\nweight ( double of [0,1] ).",
                                                              "TSPConverter error: INVALID DATA",
                                                              JOptionPane.ERROR_MESSAGE);
                                dataInFile.close();
                                return null;
                            } // if (weight < 0.0 || weight > 1.0)

                            temp_i = checkEdges(startPos,
                                                endPos,
                                                weight);

                            switch (temp_i) {
                            case -3: // wrong instance
                                System.err.println("Wrong Object occured in checkEdges() !\nNot instance of VisAlgEdge.");
                                JOptionPane.showMessageDialog(null,
                                                              "Wrong Object occured in checkEdges() !\nNot instance of VisAlgEdge.",
                                                              "TSPConverter error: INVALID DATA",
                                                              JOptionPane.ERROR_MESSAGE);
                                dataInFile.close();
                                return null;

                            case -2:  // identical edge with duoble weight
                                System.err.println("Error occured while adding edge: ambiguous edge.");
                                JOptionPane.showMessageDialog(null,
                                                              "Error occured while adding edge: ambiguous edge.",
                                                              "TSPConverter error: INVALID DATA",
                                                              JOptionPane.ERROR_MESSAGE);
                                dataInFile.close();
                                return null;

                            case -1:  // already exists
                                break;

                            case 0:  // no error
                                VisAlgEdge vae = new VisAlgEdge();
                                vae.setStartID(startPos);
                                vae.setEndID(endPos);
                                vae.setWeight(weight);
                                m_edgesVector.add(vae);
                                break;
                            } // switch
                        } // if (startPos != endPos)
                    } // if (t.countTokens() != 0)
                } //while((?? != null) && (?? != EOF ) && (?? != E_V_S))

                // draw as circle if no node- data has been given
                if ( !newVec) {
                    temp_i = m_nodesVector.size();

                    for (int j = 0; j < temp_i; j++) {
                        VisAlgNode van = (VisAlgNode) m_nodesVector.get(j);
                        van.setAbsoluteX( (int) (Math.cos( Math.PI * 2 * j / temp_i) * 1000));
                        van.setAbsoluteY( (int) (Math.sin( Math.PI * 2 * j / temp_i) * 1000));
                    } // for
                } // if ( !newVec)

                // return if file end already reached
                if (aLine == null) {
                    dataInFile.close();
                    System.err.println("Unexpected end of file. But correct data.");
                    if (  JOptionPane.YES_OPTION ==
                          JOptionPane.showConfirmDialog(null,
                                                        "Unexpected end of file. But correct data.\nContinue process ?",
                                                        "TSPConverter error: INVALID DATA",
                                                        JOptionPane.YES_NO_OPTION)) {
                        m_nodesVector.trimToSize();

                            // determins the extrem- points
                        if (determineMinMaxXY() == null) {
                            System.err.println("Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.");
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.",
                                                          "TSPConverter error: INVALID DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        }  // if (determineMinMaxXY() == null)
                        m_visAlgGraph.setMinMaxXY(determineMinMaxXY());

                        // stores the nodes
                        m_visAlgGraph.setVisAlgNodes(m_nodesVector.toArray());

                        // strores the edges
                        m_edgesVector.trimToSize();
                        // nd hier ID setzen
                        m_visAlgGraph.setVisAlgEdges(m_edgesVector.toArray());

                        // initializes the interval [0,1] in black
                        m_visAlgGraph.setLeftOpen(false);
                        m_visAlgGraph.setRightOpen(false);
                        m_intervalVector.add(new VisAlgInterval(0,
                                                                1,
                                                                Color.black));
                        m_intervalVector.trimToSize();

                        // stores the Objects in the vector as intervals
                        VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                        VisAlgInterval vai;

                        for (int i = m_edgesVector.size(); i > 0; i--) {
                            vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                            iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                               vai.getUpperLimit(),
                                                               vai.getColor());
                        } // for (int i = m_edgesVector.size(); i > 0; i--)

                        m_visAlgGraph.setVisAlgIntervals(iArray);

                        return m_visAlgGraph;
                    } // if (  JOptionPane.YES_OPTION == ...
                    else {
                        return null;
                    } // else: if (  JOptionPane.YES_OPTION ==...
                } // if (aLine == null)

                // all OK till here
                m_edgesVector.trimToSize();
                // nd hier ID setzen
                m_visAlgGraph.setVisAlgEdges(m_edgesVector.toArray());

                if ( !newVec) {

                    if (determineMinMaxXY() == null) {
                        System.err.println("Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.");
                        JOptionPane.showMessageDialog(null,
                                                      "Wrong Object occured in determineMinMaxXY() !\nNot instance of VisAlgNode.",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        dataInFile.close();
                        return null;
                    } // if (determineMinMaxXY() == null)
                    m_visAlgGraph.setMinMaxXY(determineMinMaxXY());
                    m_nodesVector.trimToSize();
                    m_visAlgGraph.setVisAlgNodes(m_nodesVector.toArray());

                } // if ( !newVec)
            } // if (aLine.startsWith("EDGE_SECTION"))

            if (  m_edgesVector.size() == 0) {
                m_visAlgGraph.setVisAlgEdges(null);
                m_visAlgGraph.setVisAlgIntervals( new VisAlgInterval(0,
                                                                     1,
                                                                     Color.white));
            } // if (  m_edgesVector.size() == 0)
            else {
                m_visAlgGraph.setVisAlgIntervals( new VisAlgInterval(0,
                                                                     1,
                                                                     Color.black));
            } // else: if (  m_edgesVector.size() == 0)

            if (  (m_nodesVector.size() == 0)) {
                System.err.println("No edge- data or node- data available.");
                JOptionPane.showMessageDialog(null,
                                              "No edge- data or node- data available.",
                                              "TSPConverter error: NO DATA",
                                              JOptionPane.ERROR_MESSAGE);
                dataInFile.close();
                return null;
            } // if (  m_nodesVector == 0)

           if (aLine.startsWith("EOF")) {
                m_visAlgGraph.setLeftOpen(false);
                m_visAlgGraph.setRightOpen(false);
           }

            if (aLine.startsWith("EDGE_VIS_SECTION")) {
                StringTokenizer t = new StringTokenizer("");
                boolean noType = true; // false if correct INTERVAL_TYPE has been found

                double from;
                double to;
                int red;
                int green;
                int blue;
                int temp_i;

                while (    (aLine = dataInFile.readLine()) != null
                           && !aLine.startsWith("EOF")) {
                    t = new StringTokenizer(aLine);

                    if (t.countTokens() != 0) {

                        if (  aLine.startsWith("INTERVAL_TYPE :")) {

                            if (aLine.indexOf("LEFT_OPEN") != -1) {
                                m_visAlgGraph.setLeftOpen(true);
                                m_visAlgGraph.setRightOpen(false);
                                noType = false;
                            } // else if (aLine.indexOf("LEFT_OPEN") != -1)

                            else if (aLine.indexOf("RIGHT_OPEN") != -1) {
                                    m_visAlgGraph.setLeftOpen(false);
                                    m_visAlgGraph.setRightOpen(true);
                                noType = false;
                            } // else if (aLine.indexOf("RIGHT_OPEN") != -1)

                            else if (aLine.indexOf("OPEN") != -1) {
                                m_visAlgGraph.setLeftOpen(true);
                                m_visAlgGraph.setRightOpen(true);
                                noType = false;
                            } // if (aLine.indexOf("OPEN") != -1)

                            else if (aLine.indexOf("COMPACT") != -1) {
                                m_visAlgGraph.setLeftOpen(false);
                                m_visAlgGraph.setRightOpen(false);
                                noType = false;
                            } // else if (aLine.indexOf("COMPACT") != -1)
                            else {
                                m_visAlgGraph.setLeftOpen(true);
                                m_visAlgGraph.setRightOpen(true);
                                noType = false;
                            } // else
                        } // if (aLine.startsWith("INTERVAL_TYPE :"))
                        else {
                            m_visAlgGraph.setLeftOpen(true);
                            m_visAlgGraph.setRightOpen(true);
                            noType = false;
                        } // else: if ( start="INTERVAL_TYPE :")
                    } // if (t.countTokens() != 0)

                    if (  !noType) {
                        break;
                    }
                } // while ( ?? != null && ?? !EOF )

                // return if file end already reached
                if (aLine == null) {
                    dataInFile.close();
                    System.err.println("Unexpected end of file. But data available.");

                    if (  JOptionPane.YES_OPTION ==
                          JOptionPane.showConfirmDialog(null,
                                                        "Unexpected end of file. But data available.\nContinue process ?",
                                                        "TSPConverter error: INVALID DATA",
                                                        JOptionPane.YES_NO_OPTION)){
                        // initializes the intervall [0,1] in black or white

                        if (m_edgesVector.size() == 0) {
                            m_intervalVector.add(new VisAlgInterval(0,
                                                                    1,
                                                                    Color.white));
                        } // if (m_edgesVector.size() == 0)
                        else {
                            m_intervalVector.add(new VisAlgInterval(0,
                                                                    1,
                                                                    Color.black));
                        } // else: if (m_edgesVector.size() == 0)
                        m_intervalVector.trimToSize();

                        // stores the Objects in the vector as intervals
                        VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                        VisAlgInterval vai;

                        for (int i = m_intervalVector.size(); i > 0; i--) {
                            vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                            iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                               vai.getUpperLimit(),
                                                               vai.getColor());
                        } // for (int i = m_intervalVector.size(); i > 0; i--)

                        m_visAlgGraph.setVisAlgIntervals(iArray);

                        if (m_intervalVector.size() == 1) {
                            m_visAlgGraph.setLeftOpen(false);
                            m_visAlgGraph.setRightOpen(false);
                        } // if (m_intervalVector.size() <= 1)

                        return m_visAlgGraph;
                    } // if (  JOptionPane.YES_OPTION == ...
                    else {
                        return null;
                    } // else: if (  JOptionPane.YES_OPTION == ...
                } // if (aLine == null)

                if (aLine.startsWith("INTERVAL_TYPE :")) {

                    aLine = dataInFile.readLine();
                    t = new StringTokenizer(aLine);
                    while (  (t.countTokens() == 0)
                             && (aLine != null)
                             && (!aLine.startsWith("EOF"))) {
                        aLine = dataInFile.readLine();
                        t = new StringTokenizer(aLine);
                    } // while ( t.c = 0 && !null && !EOF)

                    // return if file end already reached
                    if (aLine == null) {
                        dataInFile.close();
                        System.err.println("Unexpected end of file. But data available.");

                        if (  JOptionPane.YES_OPTION ==
                              JOptionPane.showConfirmDialog(null,
                                                            "Unexpected end of file. But data available.\nContinue process ?",
                                                            "TSPConverter error: INVALID DATA",
                                                            JOptionPane.YES_NO_OPTION)){
                            // initializes the intervall [0,1] in black or white

                            if (m_edgesVector.size() == 0) {
                                m_intervalVector.add(new VisAlgInterval(0,
                                                                        1,
                                                                        Color.white));
                            } // if (m_edgesVector.size() == 0)
                            else {
                                m_intervalVector.add(new VisAlgInterval(0,
                                                                        1,
                                                                        Color.black));
                            } // else: if (m_edgesVector.size() == 0)
                            m_intervalVector.trimToSize();

                            // stores the Objects in the vector as intervals
                    	    VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                            VisAlgInterval vai;

                            for (int i = m_intervalVector.size(); i > 0; i--) {
                                vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                	        iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                                   vai.getUpperLimit(),
                                                                   vai.getColor());
                            } // for (int i = m_intervalVector.size(); i > 0; i--)

                            m_visAlgGraph.setVisAlgIntervals(iArray);

                            if (m_intervalVector.size() == 1) {
                                m_visAlgGraph.setLeftOpen(false);
                                m_visAlgGraph.setRightOpen(false);
                            } // if (m_intervalVector.size() <= 1)

                            return m_visAlgGraph;
                        } // if (  JOptionPane.YES_OPTION == ...
                        else {
                            return null;
                        } // else: if (  JOptionPane.YES_OPTION == ...
                    } // if (aLine == null)

                    // now tries to find a line with five tokens
                    if (t.countTokens() != 5
                        && (!aLine.startsWith("EOF"))) {
                        System.err.println("Invalid kind of interval- data. Five values are needed:");
                        System.err.println("from   ( double of [0,1] )");
                        System.err.println("to   ( double of [0,1] )");
                        System.err.println("red-   value ( int of [0,255] )");
                        System.err.println("green-   value ( int of [0,255] )");
                        System.err.println("blue-   value ( int of [0,255] )");
                        JOptionPane.showMessageDialog(null,
                                                      "Invalid kind of interval- data. Five values are needed:"
                                                      + "\nfrom   ( double of [0,1] )"
                                                      + "\nto   ( double of [0,1] )"
                                                      + "\nred-   value ( int of [0,255] )"
                                                      + "\ngreen-   value ( int of [0,255] )"
                                                      + "\nblue-   value ( int of [0,255] )",
                                                      "TSPConverter error: UNKNOWN DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        dataInFile.close();
                        return null;
                    } // if (t.countTokens() != 5)
                } // if (aLine.startsWith("INTERVAL_TYPE :"))
                // there are only thre possibilities of aLine now
                // 5 tokens or EOF or aLine == null

                if (aLine == null) {
                    dataInFile.close();
                    System.err.println("Unexpected end of file. But data available.");

                    if (  JOptionPane.YES_OPTION ==
                          JOptionPane.showConfirmDialog(null,
                                                        "Unexpected end of file. But data available.\nContinue process ?",
                                                        "TSPConverter error: INVALID DATA",
                                                        JOptionPane.YES_NO_OPTION)){
                        // initializes the intervall [0,1] in black or white

                        if (m_edgesVector.size() == 0) {
                            m_intervalVector.add(new VisAlgInterval(0,
                                                                    1,
                                                                    Color.white));
                        } // if (m_edgesVector.size() == 0)
                        else {
                            m_intervalVector.add(new VisAlgInterval(0,
                                                                    1,
                                                                    Color.black));
                        } // else: if (m_edgesVector.size() == 0)
                        m_intervalVector.trimToSize();

                        // stores the Objects in the vector as intervals
                        VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                        VisAlgInterval vai;

                        for (int i = m_intervalVector.size(); i > 0; i--) {
                            vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                            iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                               vai.getUpperLimit(),
                                                               vai.getColor());
                        } // for (int i = m_intervalVector.size(); i > 0; i--)

                        m_visAlgGraph.setVisAlgIntervals(iArray);

                        if (m_intervalVector.size() == 1) {
                            m_visAlgGraph.setLeftOpen(false);
                            m_visAlgGraph.setRightOpen(false);
                        } // if (m_intervalVector.size() <= 1)

                        return m_visAlgGraph;
                    } // if (  JOptionPane.YES_OPTION == ...
                    else {
                        return null;
                    } // else: if (  JOptionPane.YES_OPTION == ...
               } // if (aLine == null)

                do {

                    if (aLine.startsWith("EOF")) {
                        break;
                    } // if (aLine.startsWith("EOF"))

                    t = new StringTokenizer(aLine);

                    if ( t.countTokens() != 0) {

                        if (  t.countTokens() != 5) {
                            System.err.println("Invalid kind of interval- data. Five values are needed:");
                            System.err.println("from   ( double of [0,1] )");
                            System.err.println("to   ( double of [0,1] )");
                            System.err.println("red-   value ( int of [0,255] )");
                            System.err.println("green-   value ( int of [0,255] )");
                            System.err.println("blue-   value ( int of [0,255] )");
                            JOptionPane.showMessageDialog(null,
                                                          "Invalid kind of interval- data. Five values are needed:"
                                                          + "\nfrom   ( double of [0,1] )"
                                                          + "\nto   ( double of [0,1] )"
                                                          + "\nred-   value ( int of [0,255] )"
                                                          + "\ngreen-   value ( int of [0,255] )"
                                                          + "\nblue-   value ( int of [0,255] )",
                                                          "TSPConverter error: UNKNOWN DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // else if (  t.countTokens() != 5)

                        try {
                            from  = Double.parseDouble(t.nextToken());
                            to    = Double.parseDouble(t.nextToken());
                            red   = Integer.parseInt(t.nextToken());
                            green = Integer.parseInt(t.nextToken());
                            blue  = Integer.parseInt(t.nextToken());
                        } // try
                        catch (NumberFormatException e) {
                            System.err.println("Error: " + e);
                            System.err.println("Exception while parsing in EDGE_VIS_SECTION. Wrong type.");
                            JOptionPane.showMessageDialog(null,
                                                          "Exception while parsing in EDGE_VIS_SECTION.\nWrong type.",
                                                          "TSPConverter error: INVALID DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // catch

                        if (from > to) {
                            System.err.println("No correct interval bounds in EDGE_VIS_SECTION!");
                            JOptionPane.showMessageDialog(null,
                                                          "No correct interval bounds in EDGE_VIS_SECTION!",
                                                          "TSPConverter error: INVALID INTERVALL",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if (from > to)

                        if (  (from < 0) || (from > 1)
                              || (to < 0) || (to > 1)) {
                            System.err.println("In EDGE_VIS_SECTION: The intervals have to be a part of [0,1] !");
                            JOptionPane.showMessageDialog(null,
                                                          "In EDGE_VIS_SECTION: The intervals have to be a part of [0,1] !",
                                                          "TSPConverter error: INVALID INTERVALL",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if (from < 0.0 || from > 1.0)

                        if (  from == to
                              && (  m_visAlgGraph.isLeftOpen()
                                    || m_visAlgGraph.isRightOpen())) {
                            System.err.println("Empty interval detected !");
                            System.err.println("Only allowed, if compact intervalls.");
                            JOptionPane.showMessageDialog(null,
                                                          "Empty interval detected !"
                                                          + "\nOnly allowed, if compact intervalls.",
                                                          "TSPConverter error: INVALID INTERVALL",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if from == to and not compact

                        if ( (red < 0) || (red > 255)) {
                            System.err.println("Wrong red- value detected !");
                            System.err.println("Red- value needs an integer of [0,255] !");
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong red- value detected !"
                                                          + "\nThe red- value needs an integer of [0,255] !",
                                                          "TSPConverter error: INVALID COLOR",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if ( red < 0 || red > 255)

                        if ( (green < 0) || (green > 255)) {
                            System.err.println("Wrong green- value detected !");
                            System.err.println("Green- value needs an integer of [0,255] !");
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong green- value detected !"
                                                          + "\nThe green- value needs an integer of [0,255] !",
                                                          "TSPConverter error: INVALID COLOR",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if ( green < 0 || green > 255)

                        if ( (blue < 0) || (blue > 255)) {
                            System.err.println("Wrong blue- value detected !");
                            System.err.println("Blue- value needs an integer of [0,255] !");
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong blue- value detected !"
                                                          + "\nThe blue- value needs an integer of [0,255] !",
                                                          "TSPConverter error: INVALID COLOR",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // if ( blue < 0 || blue > 255)

                        temp_i = checkInterval( from,
                                                to,
                                                m_visAlgGraph.isLeftOpen(),
                                                m_visAlgGraph.isRightOpen());

                        switch (temp_i) {
                        case -2: // wrong instance
                            System.err.println("Wrong Object occured in checkInterval() !\nNot instance of VisAlgInterval.");
                            JOptionPane.showMessageDialog(null,
                                                          "Wrong Object occured in checkInterval() !\nNot instance of VisAlgInterval.",
                                                          "TSPConverter error: INVALID DATA",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;

                        case -1: // all ok
                            m_intervalVector.add(new VisAlgInterval(from,
                                                                    to,
                                                                    new Color(red, green, blue)));
                            break;

                        default: // error with interval number ??
                            String tmp_l = "[";
                            String tmp_r = "]";

                            VisAlgInterval vai = (VisAlgInterval) m_intervalVector.get(temp_i);

                            if (m_visAlgGraph.isLeftOpen()) {
                                tmp_l = "(";
                            }

                            if (m_visAlgGraph.isRightOpen()) {
                                tmp_r = ")";
                            }

                            System.err.println("The current interval :");
                            System.err.println(tmp_l + from + "," + to + tmp_r);
                            System.err.println("intersects the existing interval :");
                            System.err.println(tmp_l + vai.getLowerLimit()
                                               + "," + vai.getUpperLimit()
                                               + tmp_r + " !");
                            JOptionPane.showMessageDialog(null,
                                                          "The current interval :"
                                                          + "\n\n" + tmp_l + from + "," + to + tmp_r
                                                          + "\n\nintersects the existing interval :"
                                                          + "\n\n" + tmp_l + vai.getLowerLimit()
                                                          + "," + vai.getUpperLimit()
                                                          + tmp_r + " !",
                                                          "TSPConverter error: INTERVALL INTERSECTION",
                                                          JOptionPane.ERROR_MESSAGE);
                            dataInFile.close();
                            return null;
                        } // switch (temp_i)
                    } // if ( t.countTokens() != 0)
                } while (    (aLine = dataInFile.readLine()) != null);
            } // if (aLine.startsWith("EDGE_VIS_SECTION"))

            // return if file end already reached
            if (aLine == null) {
                dataInFile.close();
                System.err.println("Unexpected end of file. But data available.");

                if (  JOptionPane.YES_OPTION ==
                      JOptionPane.showConfirmDialog(null,
                                                    "Unexpected end of file. But data available.\nContinue process ?",
                                                    "TSPConverter error: INVALID DATA",
                                                    JOptionPane.YES_NO_OPTION)){

                    if (m_intervalVector.size() == 0) {
                        // init the interval [0,1] in white

                        System.err.println("No interval- data available. Standart- interval is set:");
                        System.err.println("compact interval");
                        System.err.println("from :  0");
                        System.err.println("to :  1");
                        System.err.println("color is WHITE !");
                        JOptionPane.showMessageDialog(null,
                                                      "No interval- data available. Standart- interval is set:"
                                                      + "\ncompact interval"
                                                      + "\nfrom :  0"
                                                      + "\nto :  1"
                                                      + "\n color is WHITE !",
                                                      "TSPConverter error: NO DATA",
                                                      JOptionPane.ERROR_MESSAGE);

                        m_visAlgGraph.setVisAlgIntervals( new VisAlgInterval(0,
                                                                             1,
                                                                             Color.white));
                    } // if (m_intervalVector.size() == 0)
                    else {
                        // stores the current intervals
                        m_intervalVector.trimToSize();

                        // stores the Objects in the vector as intervals
                        VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                        VisAlgInterval vai;

                        for (int i = m_intervalVector.size(); i > 0; i--) {
                            vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                            iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                               vai.getUpperLimit(),
                                                               vai.getColor());
                        } // for (int i = m_intervalVector.size(); i > 0; i--)

                        if (m_intervalVector.size() == 1) {
                            m_visAlgGraph.setLeftOpen(false);
                            m_visAlgGraph.setRightOpen(false);
                        } // if (m_intervalVector.size() <= 1)

                        m_visAlgGraph.setVisAlgIntervals(iArray);
                    } // if (m_intervalVector.size() == 0)

                    return m_visAlgGraph;
                } // if (  JOptionPane.YES_OPTION == ...
                else {
                    return null;
                } // else: if (  JOptionPane.YES_OPTION == ...
            } // if (aLine == null)

            if (aLine.startsWith("EOF")) {

                if (m_intervalVector.size() == 0) {
                    // initializes the intervall [0,1] in black or white

                    if (m_edgesVector.size() == 0) {
                        m_intervalVector.add(new VisAlgInterval(0,
                                                                1,
                                                                Color.white));
                    } // if (m_edgesVector.size() == 0)
                    else {
                        m_intervalVector.add(new VisAlgInterval(0,
                                                                1,
                                                                Color.black));
                    } // else: if (m_edgesVector.size() == 0)
                    m_intervalVector.trimToSize();

                    // stores the Objects in the vector as intervals
                    VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                    VisAlgInterval vai;

                    for (int i = m_intervalVector.size(); i > 0; i--) {
                        vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                        iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                           vai.getUpperLimit(),
                                                           vai.getColor());
                    } // for (int i = m_intervalVector.size(); i > 0; i--)

                    m_visAlgGraph.setLeftOpen(false);
                    m_visAlgGraph.setRightOpen(false);

                    m_visAlgGraph.setVisAlgIntervals(iArray);
                }  // if (m_intervalVector.size() == 0)
                else {
                    // stores the current intervals
                    m_intervalVector.trimToSize();

                    // stores the Objects in the vector as intervals
                    VisAlgInterval[] iArray = new VisAlgInterval[m_intervalVector.size()];
                    VisAlgInterval vai;

                    for (int i = m_intervalVector.size(); i > 0; i--) {
                        vai = (VisAlgInterval) m_intervalVector.get(i - 1);
                        iArray[i - 1] = new VisAlgInterval(vai.getLowerLimit(),
                                                           vai.getUpperLimit(),
                                                           vai.getColor());
                    } // for (int i = m_intervalVector.size(); i > 0; i--)

                    if (m_intervalVector.size() == 1) {
                        m_visAlgGraph.setLeftOpen(false);
                        m_visAlgGraph.setRightOpen(false);
                    } // if (m_intervalVector.size() <= 1)

                    m_visAlgGraph.setVisAlgIntervals(iArray);
                } // else: if (m_intervalVector.size() == 0)
            } // if (aLine.startsWith("EOF"))

        // only if all ok in
            dataInFile.close();

        } // try
        catch (IOException e) {
            System.err.println("Error: " + e);
            JOptionPane.showMessageDialog(null,
                                          "Error: " + e,
                                          "TSPConverter error: INPUT/OUTPUT",
                                          JOptionPane.ERROR_MESSAGE);
            return null;
        } // catch

        return m_visAlgGraph;
    } // getData (File f)

    /**
     * This function determines minimum and maximum of the absoluteX- and
     * absoluteY- values.
     *
     * @return int[]   xy[0] = minX, xy[1] = maxX, xy[2] = minY, xy[3] = maxY
     */
    protected int[] determineMinMaxXY() {
        int[] xy = {0, 0, 0, 0};    // xy[0] = minX, xy[1] = maxX, xy[2] = minY, xy[3] = maxY
        int temporary;
        VisAlgNode van;

        if (m_nodesVector.size() == 0){
            return null;
        }

        if (m_nodesVector.get(0) instanceof VisAlgNode) {
            van = (VisAlgNode) m_nodesVector.get(0);
            xy[0] = xy[1] = van.getAbsoluteX();
            xy[2] = xy[3] = van.getAbsoluteY();
        }//end if (nodes.getClass(0) instanceof VisAlgNode)
        else {
            return null;
        }

        for (int i = 1; i < m_nodesVector.size(); i++) {
            if (m_nodesVector.get(i) instanceof VisAlgNode) {
                van = (VisAlgNode) m_nodesVector.get(i);
            }//end if (nodes.get(i) instanceof VisAlgNode)
            else {
                return null;
            }//end else

            temporary = van.getAbsoluteX();

            xy[0] = (xy[0] <= temporary)
                           ? xy[0]
                           : temporary;

            xy[1] = (xy[1] >= temporary)
                           ? xy[1]
                           : temporary;

            temporary = van.getAbsoluteY();
            xy[2] = (xy[2] <= temporary)
                           ? xy[2]
                           : temporary;


            xy[3] = (xy[3] >= temporary)
                           ? xy[3]
                           : temporary;
        }//end for (int i = 1; i < nodes.length; i++)

        if (xy[1] == xy[0]) {
            xy[1] = xy[1] + 1000;
            xy[0] = xy[0] - 1000;
        }//end if (xy[1] == xy[0])

        if (xy[3] == xy[2]) {
            xy[3] = xy[3] + 1000;
            xy[2] = xy[2] - 1000;
        }//end if (xy[3] == xy[2])
        return xy;
    }//end public void determineMinMaxAbsoluteXY()

    /**
     * Checks the given interval, discripted by f and t, with regard to basic
     * configuration errors or intersections with other intervals.
     * @param  f   the start of the interval
     * @param  t   the end of the interval
     * @param lo   true if left open
     * @param ro   true if right open
     *
     * @return  -2   m_intervalVector contains an element, that is not an VisAlgInterval,
     *          -1   all OK,
     *           0   the interval, the intersects the given interval.
     */
    protected int checkInterval(double f, double t, boolean lo, boolean ro) {
        VisAlgInterval vai;

        for (int i = 0; i < m_intervalVector.size(); i++) {
            if (m_intervalVector.get(i) instanceof VisAlgInterval) {
                vai = (VisAlgInterval) m_intervalVector.get(i);

                if ( (f > vai.getLowerLimit()) && (f < vai.getUpperLimit())) {
                    return i;
                } // if ( (f > vai.getLowerLimit()) &&  (f < vai.getUpperLimit()))

                if ( (t > vai.getLowerLimit()) && (t < vai.getUpperLimit())) {
                    return i;
                } // if ( (t > vai.getLowerLimit()) &&  (t < vai.getUpperLimit()))

                // compact and intersection at one of the edges
                if (!lo && !ro  // compact
                    && (  (f == vai.getUpperLimit())
                          || (t == vai.getLowerLimit()))) {  // intersection
                    return i;
                } // if (!lo && !ro)

            }//end if (nodes.get(i) instanceof VisAlgNode)
            else {
                return -2;
            }//end else
        } // for (int i = 0; i < v.size(); i++)

        return -1;
    } // public int checkInterval(double f, double t, Vector v)

    /**
     * Searches the in s specified file and tries to extract the data for
     * the nodes. The specified file has to be a TSPLIB- file.
     * @param s   String, that discribes the TSPLIB- file
     * @param p   String, that discribed the parent of the TSPGraph- file
     *
     * @return boolean   true if it worked
     */
    protected boolean extractNodes(String s, String p) {
        File f = new File(s);
        StringTokenizer t = new StringTokenizer("");
        String aLine;
        String name;
        int X;
        int Y;
        int iTemp;
        int delete = 0;

        // If just the file- name is given, tries to find the file in the same
        // directory as the TSPGraph- file.
        if (!f.isFile()) {
            f = new File(p + f.separator + s);
        } // if (!f.isFile())

        // is not a file
        if (!f.isFile()) {
            System.err.println("The given string does not describes a file !");

            if (  JOptionPane.YES_OPTION ==
                  JOptionPane.showConfirmDialog(null,
                                                "The given string does not describes a file !"
                                                + "\nTry to convert rest of the TSPGraph- file ?",
                                                "TSPConverter error: TSPLIB- FILE",
                                                JOptionPane.YES_NO_OPTION)) {
                m_visAlgGraph.setTSPLibFile(null);
                return true;
            } // if (  JOptionPane.YES_OPTION ==...
            else {
                return false;
            } // else: if (  JOptionPane.YES_OPTION ==...
        } // if (!f.isFile())

        // file does not exists
        if (!f.exists()) {
            System.err.println("The specified file does not exists !");

            if (  JOptionPane.YES_OPTION ==
                  JOptionPane.showConfirmDialog(null,
                                                "The specified file does not exists !"
                                                + "\nTry to convert rest of the TSPGraph- file ?",
                                                "TSPConverter error: TSPLIB- FILE",
                                                JOptionPane.YES_NO_OPTION)) {
                m_visAlgGraph.setTSPLibFile(null);
                return true;
            } // if (  JOptionPane.YES_OPTION ==...
            else {
                return false;
            } // else: if (  JOptionPane.YES_OPTION ==...
        }  // if (!f.exists())

        try {
            BufferedReader tspFile = new BufferedReader(new FileReader(f));

           // jumps to "NAME :"
            while (    (aLine = tspFile.readLine()) != null
                       && !aLine.startsWith("NAME :")) {
            }

            // return if file end already reached
            if (aLine == null) {
                System.err.println("Unexpected end of file.\nNo NAME found!");
                if (  JOptionPane.YES_OPTION ==
                      JOptionPane.showConfirmDialog(null,
                                                  "Unexpected end of file.\nNo NAME found!"
                                                  + "\nTry to convert rest of the TSPGraph- file ?",
                                                  "TSPConverter error: NO DATA",
                                                  JOptionPane.YES_NO_OPTION)) {
                    tspFile.close();
                    return true;
                } // if (  JOptionPane.YES_OPTION ==
                else {
                    tspFile.close();
                    return false;
                }
            } // if (aLine == null)

            // jumps to "EDGE_WEIGHT_TYPE"
            while (    (aLine = tspFile.readLine()) != null
                       && !aLine.startsWith("EDGE_WEIGHT_TYPE")) {
            }

            // return if file end already reached
            if (aLine == null) {
                System.err.println("Unexpected end of file.\nNo EDGE_WEIGHT_TYPE found!");
                if (  JOptionPane.YES_OPTION ==
                      JOptionPane.showConfirmDialog(null,
                                                  "Unexpected end of file.\nNo EDGE_WEIGHT_TYPE found!"
                                                  + "\nTry to convert rest of the TSPGraph- file ?",
                                                  "TSPConverter error: NO DATA",
                                                  JOptionPane.YES_NO_OPTION)) {
                    tspFile.close();
                    return true;
                } // if (  JOptionPane.YES_OPTION ==...
                else {
                    tspFile.close();
                    return false;
                } // else: if (  JOptionPane.YES_OPTION ==...
            } // if (aLine == null)

            // not a specified data - EUC_2D does not exists
            if (aLine.indexOf("EUC_2D") == -1) {
                System.err.println("No correct data.\nNo EUC_2D found!");
                int tmp = JOptionPane.showConfirmDialog(null,
                                                        "No correct data.\nNo EUC_2D found!"
                                                        + "\nTry to convert rest of the TSPLIB- file ?"
                                                        + "\\nnYes  -  try to convert, but maybe error will occure."
                                                        + "\nNo  -  try to convert rest of the TSPGraph- file."
                                                        + "\nCancel  -  stop converting any data.",
                                                        "TSPConverter error: INVALID DATA",
                                                        JOptionPane.YES_NO_CANCEL_OPTION);

                if ( tmp == JOptionPane.NO_OPTION) {
                    tspFile.close();
                    return true;
                } // if ( tmp = JOptionPane.NO_OPTION)

                if ( tmp == JOptionPane.CANCEL_OPTION) {
                    tspFile.close();
                    return false;
                } // if ( tmp = JOptionPane.NO_OPTION)
            } // if (aLine.indexOf("EUC_2D") == -1)

            // jumps till NODE_COORD_SECTION
            while (    (aLine = tspFile.readLine()) != null
                       && !aLine.startsWith("NODE_COORD_SECTION")) {
            } // while (null && !NODE_COORD_SECTION)

            if (aLine == null) {
                System.err.println("Unexpected end of file.\nNo NODE_COORD_SECTION found!");
                if (  JOptionPane.YES_OPTION ==
                      JOptionPane.showConfirmDialog(null,
                                                  "Unexpected end of file.\nNo NODE_COORD_SECTION found!"
                                                  + "\nTry to convert rest of the TSPGraph- file ?",
                                                  "TSPConverter error: NO DATA",
                                                  JOptionPane.YES_NO_OPTION)) {
                    tspFile.close();
                    return true;
                } // if (  JOptionPane.YES_OPTION ==
                else {
                    tspFile.close();
                    return false;
                }
            } // if (aLine == null)

            while (    (aLine = tspFile.readLine()) != null
                       && !aLine.startsWith("EOF")) {
                t = new StringTokenizer(aLine);

                if (t.countTokens() != 0) {

                    if (  t.countTokens() != 3) {
                        System.err.println("Invalid kind of node- data. Three values are needed:");
                        System.err.println("name  ( a String )");
                        System.err.println("x- value  ( Double )");
                        System.err.println("y- value ( Double )");
                        JOptionPane.showMessageDialog(null,
                                                      "Invalid kind of node- data. Three values are needed:"
                                                      + "\nname  ( a String )"
                                                      + "\nx- value  ( Double )"
                                                      + "\ny- value  ( Double )",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        tspFile.close();
                        return false;
                    } // if (  t.countTokens() != 3)
                    name = t.nextToken();

                    // searches for ambigouse nodes
                    iTemp = findNode(name);
                    switch (iTemp) {
                    case -2: // wrong instance
                        System.err.println("Wrong Object occured in findNode() !\nNot instance of VisAlgNode.");
                        JOptionPane.showMessageDialog(null,
                                                      "Wrong Object occured in findNode() !\nNot instance of VisAlgNode.",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        return false;

                    case -1: // not existing - is OK
                        break;

                    default: // already exists
                        System.err.println("Ambiguous node in findNode().");
                        System.err.println("Node '" + name + "' already exists.");
                        JOptionPane.showMessageDialog(null,
                                                      "Ambiguous node in findNode().\nNode '" + name + "' already exists.",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        return false;
                    } // switch

                    try {
                        X = (int) Double.parseDouble(t.nextToken());
                        Y = (int) Double.parseDouble(t.nextToken());
                    } // try
                    catch (NumberFormatException e) {
                        System.err.println("Error: " + e);
                        System.err.println("Exception while parsing coordinates in NODE_SECTION. Wrong type.");
                        JOptionPane.showMessageDialog(null,
                                                      "Exception while parsing coordinates in NODE_SECTION.\nWrong type.",
                                                      "TSPConverter error: INVALID DATA",
                                                      JOptionPane.ERROR_MESSAGE);
                        tspFile.close();
                        return false;
                    } // catch

                    VisAlgNode van = new VisAlgNode();

                    van.setName(name);
                    van.setAbsoluteX(X);
                    van.setAbsoluteY(Y);

                    m_nodesVector.add(van);
                } // if (t.countTokens() == 0)
            }// while (EOF or null)

            // return if file end already reached
            if (aLine == null) {
                System.err.println("Unexpected end of file. But data available.");

                if (  JOptionPane.YES_OPTION ==
                      JOptionPane.showConfirmDialog(null,
                                                    "Unexpected end of file. But data available.\nContinue process ?",
                                                    "TSPConverter error: INVALID DATA",
                                                    JOptionPane.YES_NO_OPTION)) {
                    tspFile.close();
                    return true;
                } // if (  JOptionPane.YES_OPTION ==...
                else {
                    tspFile.close();
                    return false;
                } // else: if (  JOptionPane.YES_OPTION ==...
            } // if (aLine == null)
        } // try
        catch (IOException e) {
            System.err.println("Error: " + e);
            JOptionPane.showMessageDialog(null,
                                          "Error: " + e,
                                          "TSPConverter error: INPUT/OUTPUT",
                                          JOptionPane.ERROR_MESSAGE);
            return false;
        } // catch

        m_visAlgGraph.setTSPLibFile(f);
        return true;
    } // protected Vector extractNodes(String s)

    /**
     * Adds the node, specified in the String, to the current nodes.
     * @param s   Line from file containing the node- data.
     *
     * @return true   if it worked
     */
    protected boolean addNodes(String s) {
        StringTokenizer t = new StringTokenizer(s);
        int X;
        int Y;
        int iTemp;

        s = t.nextToken();

        try {
            X = (int) Double.parseDouble(t.nextToken());
            Y = (int) Double.parseDouble(t.nextToken());
        } // try
        catch (NumberFormatException e) {
            System.err.println("Error: " + e);
            System.err.println("Exception while parsing coordinates in NODE_SECTION.\nWrong type.");
            JOptionPane.showMessageDialog(null,
                                          "Exception while parsing coordinates in NODE_SECTION.\nWrong type.",
                                          "TSPConverter error: INVALID DATA",
                                          JOptionPane.ERROR_MESSAGE);
            return false;
        } // catch

        iTemp = findNode(s);
        switch (iTemp) {
        case -2: // wrong instance
            System.err.println("Wrong Object occured in findNode() !\nNot instance of VisAlgNode.");
            JOptionPane.showMessageDialog(null,
                                          "Wrong Object occured in findNode() !\nNot instance of VisAlgNode.",
                                          "TSPConverter error: INVALID DATA",
                                          JOptionPane.ERROR_MESSAGE);
            return false;

        case -1: // not existing - is OK
            break;

        default: // already exists
            System.err.println("Ambiguous node in findNode().");
            System.err.println("Node '" + s + "' already exists.");
            JOptionPane.showMessageDialog(null,
                                          "Ambiguous node in findNode().\nNode '" + s + "' already exists.",
                                          "TSPConverter error: INVALID DATA",
                                          JOptionPane.ERROR_MESSAGE);
            return false;
        } // switch

        VisAlgNode node = new VisAlgNode();

        node.setName(s);
        node.setAbsoluteX(X);
        node.setAbsoluteY(Y);

        m_nodesVector.add(node);
        return true;
    } // protected boolean addNodes(String s)

    /**
     * This function searches within the given vector for an edge with the
     * following specifications:
     * @param s   Start- ID
     * @param e   End- ID
     * @param w   Weight
     *
     * @return  0 No error occured,
     *         -1 Two completely identical edges found,
     *         -2 Two identical edges with different weights found,
     *         -3 Wrong instance.
     */
    private int checkEdges(int s, int e, double w) {
        int temporary = m_edgesVector.size();
        VisAlgEdge vae;

        for (int j = 0; temporary > j; j++) {
            if (m_edgesVector.get(j) instanceof VisAlgEdge) {
                vae = (VisAlgEdge) m_edgesVector.get(j);

                if (    (s == vae.getStartID() || e == vae.getStartID())
                        && (s == vae.getEndID() || e == vae.getEndID()) ) {
                    if (w == vae.getWeight()) {
                        return -1;
                    }// if (w == vae.getWeight())
                    else {
                        return -2;
                    }// else (w == vae.getWeight())
                }
            }// if instance of VAE
            else {
                return -3;
            }
        }//end for (int j = 0; temporary > j; j++)
        return 0;
    }//end public int checkEdges(int s, int e, double w, Vector v)

    /**
     * This function searches for the ID corresponding to the given string
     * within the given Vector.
     * @param in   Nodename
     *
     * @return j Node- ID,
     *        -1 Node not found,
     *        -2 Wrong instance.
     */
    private int findNode(String in) {
        int temporary = m_nodesVector.size();
        VisAlgNode van;

        for (int j = 0; temporary > j; j++) {
            if (m_nodesVector.get(j) instanceof VisAlgNode) {
                van = (VisAlgNode) m_nodesVector.get(j);

                if (in.compareTo(van.getName()) == 0) {
                    return j;
                }//end if (in.compareTo(van.getName()) == 0)
            }//end if (v.get(j) instanceof VisAlgNode)
            else {
                return -2;
            }//end else
        }//end for (int j = 0; temporary > j; j++)
        return -1;
    }//end public int findNode(String in,Vector v)
} // end class TSPGraphConverter