////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : TuringMachineDataConverter.java
//  Description      : converting a program text file into VisAlg-
//                                                     TuringMachineData
//  Author           : Heiner Violet
//  Email            : hviolet@ix.urz.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : 30.09.01
//  Last modified by :
//  Last modified on :
//  Update count     : V 1.0
//
///////////////////////////////////////////////////////////////////////////////
//
// Date          Name               Change/Extensions
// ----          ----               -----------------
//
////////////////////////////////////////////////////////////////////////////


package visalg.converter;


import visalg.basics.ObjectReader;
import visalg.basics.Converter;
import visalg.types.VisAlgTuringMachineData;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.ArrayList;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;

import java.lang.NumberFormatException;

/**
 * Converts a text file which contains a Turing program
 * file into a VisAlgTuringMachineData-Object.
 * @see visalg.types.VisAlgTuringMachineData.
 */
public class TuringMachineDataConverter implements Converter {

    // -----------------------------------------------------
    // --------------- V A R I A B L E S -------------------
    // -----------------------------------------------------

    /*
     * ArrayList that contains the text of the Turing program
     */
    private ArrayList m_turingProgramText;

    /**
     * ArrayList that contains a copy of 'm_turingProgramText'
     */
    private ArrayList m_turingProgramTextCopy;

    /**
     * ArrayList that contains the coded Turing program
     */
    private ArrayList m_turingProgramCode;

    /**
     * The syntax type of the Turing program
     */
    private int m_turingProgramSyntaxType;

    /**
     * The tape alphabet of the Turing machine
     */
    private char[] m_tapeAlphabet;

    /**
     * The length of the tape alphabet
     */
    private int m_tapeAlphabetLength;

    /*
     * VisAlgTuringMachineData-Object where the read data and the
     * configuration of the Turing Machine will be stored
     */
    private VisAlgTuringMachineData m_turingMachineData;

    // -----------------------------------------------------
    // ------- M E T H O D S   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Does nothing.
     */
    public TuringMachineDataConverter() {

    } //TuringMachineDataConverter()

    /**
     * Reads in the program data out of the specified file.
     * Calls method programCheck() where the convertion takes place
     */
    public Object getData(File f) {

        m_turingProgramText = new ArrayList();
        try {
            BufferedReader in = new BufferedReader(new FileReader(f));
	    String line;

           // Scan lines
            while ((line = in.readLine()) != null) {
                line = line + " ";
	        m_turingProgramText.add(line);
            }
            in.close();
        } catch (IOException e) {
  	      System.out.println("An error occurred while reading file " +
                                                  f.getAbsolutePath() + ".");
              return null;
          }

        m_turingProgramText.remove(0);   // cut the first line of the program
        if(programCheck() != 0) {
            System.out.println(" Conversion of data failed.");
            return null;
        }
        m_turingMachineData = new VisAlgTuringMachineData(
                        m_turingProgramCode, m_turingProgramSyntaxType,
                                        m_tapeAlphabet, m_turingProgramText);
        return m_turingMachineData;

    } // getData(File f)

    // -----------------------------------------------------
    // --------  M E T H O D S  ( p r i v a t e )-----------
    // -----------------------------------------------------

    /**
     * Tries to convert the program text stored in m_turingProgramText into
     * an object VisAlgTuringMachineData by following steps:
     *  + Makes a copy of m_turingProgramText.
     *  + Simplifies this copy by cutting all redundant lines and spaces.
     *  + Reads the syntax-type and the alphabet of the program.
     *  + Makes a syntax control i.e. testing brackets, number of args,
     *    length of args.
     *  + Translates the program lines into vectors of integers
     *  + Returns 0 if the m_turingProgramText represents a valid TM program
     *    and -1 otherwise (in this case the convertion has failed!).
     */
    private int programCheck() {
        String line;
        m_turingProgramTextCopy = new ArrayList(m_turingProgramText);

   // remove all blank lines, comment lines, comments and spaces
        for(int i = 0; i < m_turingProgramTextCopy.size(); ++i) {
            line = (String)m_turingProgramTextCopy.get(i);
            line = line.trim();
            if(line.equals("") || (line.charAt(0) == '/')) {
                m_turingProgramTextCopy.remove(i--);
                continue;
            }
            m_turingProgramTextCopy.remove(i);
            line = removeComment(line);
            line = cutBlank(line);
            m_turingProgramTextCopy.add(i, line);
        }
   // test if program empty:
        if(m_turingProgramTextCopy.size() < 3) {
            System.out.println("Program contains no command lines.");
            return -1;
        }
   // read the syntax-type of the program
        line =  (String)m_turingProgramTextCopy.get(0);
        if(readSyntaxType(line) != 0) {
            return -1;
        }
        m_turingProgramTextCopy.remove(0);
   // read the alphabet of the program
        line =  (String)m_turingProgramTextCopy.get(0);
        if(readAlphabet(line) != 0) {
            return -1;
        }
        m_turingProgramTextCopy.remove(0);
   // translate m_turingProgramTextCopy into vectors of integers
        m_turingProgramCode = new ArrayList();
        for(int i = 0; i < m_turingProgramTextCopy.size(); ++i) {
            if(translateLine((String)m_turingProgramTextCopy.get(i)) != 0) {
                return -1;
            }
        }
        return 0;
    } // method programCheck

    /*
     * Removes the comment at the end of the command line if there is one.
     */
    private String removeComment(String line) {
        int j;
        for(j = 0; j < line.length(); ++j) {
            if ((line.charAt(j) == '/') && (line.length() > j+1)) {
                if(line.charAt(j+1) == '/') {
                    break;
                }
            }
        }
        return line.substring(0, j);
    } // method removeComment

    /*
     * cuts all blank symbols from a command line
     */
    private String cutBlank(String line) {
        String newLine = "";
        char[] c = new char[line.length()];
        int ctr = 0;

        line = line.trim();
        for(int i = 0; i < line.length(); ++i) {
            if((line.charAt(i) != ' ') && (line.charAt(i) != '\t')) {
                c[ctr++] = line.charAt(i);
            }
        }
        newLine = new String(c);
        return newLine.trim();
    } // method cutBlank(String)  */

    /*
     * Reads the syntax-type out of the String line.
     * Returns -1 if the definition is wrong
     */
    private int readSyntaxType(String line) {
        if(line.length() < 5) {
            System.out.println("Missing keyword \"TYPE:\".");
            return -1;
        }
        if( !(line.substring(0,5).equals("TYPE:")) ) { // missing keywd "TYPE:"
            System.out.println("Missing keyword \"TYPE:\".");
            return -1;
        }
        line = line.substring(5);
        if(line.length() != 7) {                 // illegal syntax-type
            System.out.println("Illegal syntax-type " + line + ".");
            return -1;
        }
        if(line.equals("states1")) {
            m_turingProgramSyntaxType = 1;
        }
        else if(line.equals("states2")) {
            m_turingProgramSyntaxType = 2;
        }
        else {
            System.out.println("Illegal syntax-type " + line + ".");
            return -1;
        }
        return 0;
    } // method readSyntaxType

    /*
     * Reads the tape alphabet out of the String line.
     * Returns -1 if the alphabet isn't defined correctly, 0 otherwise.
     */
    private int readAlphabet(String line) {
        if(line.length() < 9) {
            System.out.println("Missing keyword \"ALPHABET:\".");
            return -1;
        }
        if(!(line.substring(0,9).equals("ALPHABET:"))) {
            System.out.println("Missing keyword \"ALPHABET:\".");
            return -1;                          // missing keyword "ALPHABET:"
        }
        line = line.substring(9);
        if(line.length() < 1) {       // alphabet contains no symbol at all
            System.out.println("Alphabet must contain the symbol '#'.");
            return -1;
        }
        m_tapeAlphabet = new char[line.length()];
        boolean cFlag = false;     // control flag for searching the symbol
        int i;                     // '#' which must be contained in alphabet
        for(i = 0; i < line.length(); ++i) {
            m_tapeAlphabet[i] = line.charAt(i);
            ++m_tapeAlphabetLength;
            if(line.charAt(i) == '#') {
                cFlag = true;
            }
        }
        if(!cFlag) {
            System.out.println("Alphabet must contain the symbol '#'.");
            return -1;
        }
        return 0;                   // alphabet correctly defined
    } // method readAlphabet

    /*
     * Translates the received String line into a coded TM command i.e.
     * a five Tupel (z0, a, b, c, z1) where z0, z1 are program states
     * and a, b, c depend on the syntax-type.
     * syntaxType = 1:  a = 1: move command -  then b is the move command.
     *                  a = 2: print command - then b is the symbol to print.
     *                  a = 3: test command - then b and c are program states.
     * syntaxType = 2:  'a' is the symbol to be on the heads cell.
     *                  'b' the symbol to print on the heads cell.
     *                  'c' the move command to do.
     *
     * To do this translation this method checks the brackets, the number
     * of args, the contents of the args and returns -1 if there is an error.
     */
    private int translateLine(String line) {
        int[] code;
        String[] args;

        if(line.length() < 7) {
            System.out.println("Program contains invalid command.");
            return -1;
        }
        if(!(line.charAt(0) == '(') ||
                            !(line.charAt(line.length() - 1) == ')')) {
            System.out.println("Missing bracket");
            return -1;
        }
    // testing if the line is of the form (..)->(..)
        if(m_turingProgramSyntaxType == 2) {
            line = testArrow(line);
            if(line.equals("error")) {
                System.out.println("Program contains invalid command.");
                return -1;
            }
        }
    // counting the arguments of the command:
        int num = 1;  // counting variable for number of args
        for(int i = 1; i < line.length(); ++i) {
            if(line.charAt(i) == ',') {
                ++num;
            }
        }
    // Reading the arguments of the command into args[]:
        args = new String[num];
        num = 0;
        int i, iOld = 1;
        for(i = 1; i < line.length(); ++i) {
            if( (line.charAt(i) == ',')  || (i == line.length()-1) ) {
                args[num++] = line.substring(iOld, i);
                iOld = i+1;
            }
        }
    // translate them into five-tupels of integers
        code = new int[5];
        String arg = "";

        arg = args[0];        // analysing the 1. argument (initial state).
        try {
            i = Integer.parseInt(arg);
            code[0] = i;
        }
        catch(NumberFormatException e) {
            System.out.println("Illegal argument");
            return -1;
        } // first argument

      // ***************** 1. SYNTAX:states1 ******************
        if(m_turingProgramSyntaxType == 1) {
            if( (num < 3) || (num > 4) ) {
                System.out.println("Wrong number of arguments.");
                return -1;
            }

            if(num == 3) {        // three args: print or move command
                arg = args[1];      // analysing the 2. argument (P or M).
                if( (arg.length() == 0) || (arg.length() > 2) ) {
                    System.out.println("Illegal argument");
                    return -1;
                }
                if (arg.length() == 1) {     // in case of a Move-command
                    code[1] = 1;
                    if (arg.equals("L") || arg.equals("R") || arg.equals("S")) {
                        code[2] = arg.charAt(0);
                    }
                    else {
                        System.out.println("Illegal argument.");
                        return -1;
                    }
                }
                else if (arg.length() == 2) {  // in case of a Write-/Read-comm.
                    if (arg.charAt(0) == 'P') {
                        code[1] = 2;
                        if(isInAlphabet(arg.charAt(1))) {
                            code[2] = arg.charAt(1);
                        }
                        else {
                            System.out.println("Illegal character in program line");
                            return -1;
                        }
                    }
                    else {
                        System.out.println("Illegal argument.");
                        return -1;
                    }
                }

                arg = args[2];      // analysing the 3. argument (follow state).
                try {
                    i = Integer.parseInt(arg);
                    code[4] = i;
                }
                catch(NumberFormatException e) {
                    System.out.println("Illegal argument");
                    return -1;
                } // third argument

            }
            else if(num == 4) {    // four args: test command
                arg = args[1];      // analysing the 2. argument (T).
                if(arg.length() != 2) {
                    System.out.println("Illegal argument");
                    return -1;
                }
                code[1] = 3;
                if ( !(arg.charAt(0) == 'T')) {
                    System.out.println("Illegal argument");
                    return -1;
                }
                if(isInAlphabet(arg.charAt(1))) {
                    code[2] = arg.charAt(1);
                }
                else {
                    System.out.println("Illegal character in program line");
                    return -1;
                }
                arg = args[2];        // 3. arg. (follow state 'then').
                try {
                    i = Integer.parseInt(arg);
                    code[4] = i;
                }
                catch(NumberFormatException e) {
                    System.out.println("Illegal argument");
                    return -1;
                } // third argument
                 arg = args[3];        // 4. argument (follow state 'else').
                try {
                    i = Integer.parseInt(arg);
                    code[3] = i;
                }
                catch(NumberFormatException e) {
                    System.out.println("Illegal argument");
                    return -1;
                } // fourth argument
            }
        } // syntax-type states1

    // ***************** 2. SYNTAX:states2 ******************
        else if(m_turingProgramSyntaxType == 2) {
            if(num != 5) {
                System.out.println("Wrong number of arguments.");
                return -1;
            }
            arg = args[1];        // analysing the 2. argument (symbol).
            if(arg.length() != 1) {
                System.out.println("Illegal argument.");
                return -1;
            }
            if(isInAlphabet(arg.charAt(0))) {
                code[1] = arg.charAt(0);
            }
            else {
                System.out.println("Illegal character in program line");
                return -1;
            }
            arg = args[2];        // analysing the 3. argument (symbol).
            if(arg.length() !=1) {
                System.out.println("Illegal argument.");
                return -1;
            }
            code[2] = arg.charAt(0);

            arg = args[3];        // analysing the 4. argument (move command).
            if(arg.length() !=1) {
                System.out.println("Illegal argument.");
                return -1;
            }
            code[3] = arg.charAt(0);

            arg = args[4];        // analysing the 5. argument (follow state).
            try {
                i = Integer.parseInt(arg);
                code[4] = i;
            }
            catch(NumberFormatException e) {
                System.out.println("Illegal argument");
                return -1;
            } // fifth argument

        } // syntax-type states2

        m_turingProgramCode.add(code);
        return 0;
    } // method transLateLine

    /*
     * Tests if the received String line contains a part ")->(".
     * If it is so, this part is cut and the remaining string returned,
     * otherwise a String "error" is returned.
     */
    private String testArrow(String line) {
        if(line.length() < 14) {
            return "error";
        }
        char[] a = new char[4];
        for(int i = 0; i < line.length() - 4; ++i) {
            a[0] = line.charAt(i);
            a[1] = line.charAt(i+1);
            a[2] = line.charAt(i+2);
            a[3] = line.charAt(i+3);
            String s = new String(a);
            if( s.equals(")->(") ) {
                line = line.substring(0,i) + "," + line.substring(i+4);
                return line;
            }
        }
        return "error";
    } // method testArrow

    /**
     * Checks wether a given symbol c is contained in the alphabet
     */
    private boolean isInAlphabet(char c) {
        for(int i = 0; i < m_tapeAlphabetLength; ++i) {
            if(m_tapeAlphabet[i] == c) {
                return true;
            }
        }
        return false;
    } // method isInAlphabet

} // class TuringMachineDataConverter