////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : AlgorithmWindow.java
//  Description      : A superclass for all algorithm windows
//  Author           : Matthias Vigelius
//  Email            : mvigelius@gmx.de
//  Copyright        : Uni Heidelberg
//  Created on       : Wed Apr 04 16:01:00 2000
//  Last modified by :
//  Last modified on :
//  Update count     : 0
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//
///////////////////////////////////////////////////////////////////////////////

package visalg.graphics;

import javax.swing.JLabel;
import javax.swing.JComboBox;
import java.awt.Rectangle;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.AbstractAction;

import visalg.basics.*;
import visalg.types.*;

/**
 * This class is thought to be a superclass for all algorithm module windows.
 * Such a window class can be derived from this one and just has to overwrite
 * the method openViewer(), that is called, if the user chooses the menu item
 * "Open Viewer ...". The derived class must then construct a new viewer window
 * object and register it in the ModuleWindowContainer.
 *
 * It might be also usefull to overwrite the method onLoadMenu(JMenu) in order
 * to add own menu items to the module menu.
 *
 * @see visalg.graphics.ModuleWindowContainer
 */
public class AlgorithmWindow extends ModuleWindow {

    /**
     * The label showing the module time.
     */
    private JLabel m_stepTester;

    /**
     * A combobox containing all available data modules, the user can choose
     * of.
     */
    private JComboBox m_dataModules;

    /**
     * The menu that will be added to the container menu bar.
     */
    private JMenu m_algorithmMenu;

    /**
     * Reference to the ModuleWindowContainer.
     */
    private ModuleWindowContainer m_moduleWindowContainer;

    /**
     * Creates a standard window and loads the menu. Before the menu is added
     * to the MenuBarContainer, onLoadMenu(JMenu) is called in order to allow
     * derived classes to add own menu items.
     *
     * @param name The name of the window
     * @param alg The algorithm module, this window belongs to
     * @param bounds The initial window bounds of this window
     * @param mwc The associated ModuleWindowContainer
     */
    public AlgorithmWindow(String name, Algorithm alg, Rectangle bounds,
            ModuleWindowContainer mwc) {

        super(name, alg, bounds, mwc);

        //Initialisierung der (lokalen) Variablen
        m_moduleWindowContainer = mwc;

		m_contentPane.setLayout(new BorderLayout());
		m_stepTester = new JLabel("Time : " + getModule().getTime());
		m_dataModules = new JComboBox(((Algorithm) getModule()).getDataModules());
        m_dataModules.addActionListener(new DataListListener());
		JPanel northPanel = new JPanel();
		northPanel.add(m_stepTester);
		northPanel.add(m_dataModules);
		m_contentPane.add(northPanel,BorderLayout.NORTH);
    }//AlgorithmWindow(String,Algorithm,Rectangle,ModuleWindowContainer)

    /**
     * If the algorithm module has changed, this function shows the correct
     * module time and invokes a repaint(). Might be overwritten, but the
     * superclass function should be called at the end.
     *
     * @param cEvt The ModuleChangeEvent
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
        m_stepTester.setText("Time : " + ((Algorithm)getModule()).getTime());
		revalidate();
		repaint();
    }//moduleChanged(ModuleChangeEvent cEvt)

    /**
     * Will be called by the ModuleWindowContainer, if the window is about to
     * be closed.
     *
     * @see visalg.graphics.ModuleWindowContainer
     */
    public void onClose() {
    }//onClose()

    public void onFocusGained() {
        loadMenu();
    }

    public void onFocusLost() {
        getModule().getModuleManager().getMenuBarContainer().removeMenu(m_algorithmMenu);
    }

    /**
     * Loads the menu for this window. Before the menu is added to the menubar,
     * onLoadMenu() will be called.
     */
    private void loadMenu() {
        m_algorithmMenu = new JMenu(getName());

        onLoadMenu(m_algorithmMenu);

        getModule().getModuleManager().getMenuBarContainer().addMenu(m_algorithmMenu);
    }//loadMenu()

    /**
     * The ActionListener class for the combobox. If the user selects another
     * data module, Algorithm.setDataModule(DataModule) is called in order to
     * set the new data module.
     *
     * @see visalg.basics.Algorithm#setDataModule(DataModule)
     */
    class DataListListener implements ActionListener {
        public DataListListener() {}

        public void actionPerformed(ActionEvent evt) {
            DataModule dm = (DataModule)m_dataModules.getSelectedItem();
		    //if (dm != ((Algorithm)getModule()).getDataModule())
            ((Algorithm)getModule()).setDataModule(dm);
		    m_dataModules.invalidate();
		    validate();
        }
    }//class DataListListener

    /**
     * Returns a reference to the ModuleWindowContainer.
     *
     * @return The ModuleWindowContainer
     */
    protected ModuleWindowContainer getModuleWindowContainer() {
        return m_moduleWindowContainer;
    }//getModuleWindowContainer()

    /**
     * Will be called, if the menu is created. Derived classes can add own
     * items to the passed menu.
     *
     * @param menu The module menu
     */
    protected void onLoadMenu(JMenu menu) {
    }//onLoadMenu(JMenu)

    /**
     * Returns a reference to the associated algorithm module.
     *
     * @return The algorithm module
     */
    protected Algorithm getAlgorithm() {
        return (Algorithm) getModule();
    }//getAlgorithm()
}//class AlgorithmWindow