////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ApplicationWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Wed Oct  4 11:28:03 2000
//  Update count     : 17
//  CVS Info         : $Id: ApplicationWindow.java,v 1.2 2002/03/18 12:28:48 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  08.01.01    vigelius    Neues edit menu hinzugefuegt.
//  21.01.01    vigelius    fixed window bug at project loading/saving
//  27.02.01    vigelius    Enhanced documentation
//  06.03.01    vigelius    add addMenu / removeMenu
//  06.03.01    vigelius    realised the mainPane as a JSplitPane
//  07.03.01    vigelius    the left window remembers its divider position
//  07.03.01    vigelius    removed dynamic menubar bug
//  02.04.01    vigelius    added JHelpSystem
//  03.04.01    vigelius    the status line is now embedded in a scroll pane
//  27.06.01    vigelius    Adapted to the new Menu concept
//  20.11.01    vigelius    Added the ProjectFileFilter
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import java.io.File;

import java.awt.Rectangle;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Window;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JDesktopPane;
import javax.swing.DesktopManager;
import javax.swing.DefaultDesktopManager;
import javax.swing.JComponent;
import javax.swing.JLayeredPane;
import javax.swing.JInternalFrame;
import javax.swing.JSplitPane;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.Action;
import javax.swing.AbstractAction;
import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;
import javax.help.*;
import java.beans.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.filechooser.*;

import visalg.basics.Application;
import visalg.graphics.HistoryWindowContainer;
import visalg.graphics.ProjectWindowContainer;
import visalg.graphics.ModuleWindowContainer;
import visalg.graphics.MenuBarContainer;
import visalg.components.PreferencesDialog;


/**
 * ApplicationWindow is the top class of the GUI. It is responsible for the
 * HistoryWindow, the ProjectWindow, the menu and the ModuleWindows. It
 * furthermore acts as a Container for those and implements a DesktopManager,
 * that handles user actions for the menu windows, like dragging or resizing.
 * If the user changes one of them, the new window bounds are stored using
 * setWindowBounds(...) of the ModuleWindow.
 *
*/
public class ApplicationWindow extends JFrame
    implements HistoryWindowContainer,
	       ProjectWindowContainer,
	       ModuleWindowContainer,
	       MenuBarContainer {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to Application.
     */
    private transient Application m_application;

    /**
     * The Container for all windows.
     */
    private transient Container m_contentPane;

    /**
     * The HelpBroker object.
     */
    private transient HelpBroker m_helpBroker;

    // ----------------------- Menu ----------------------

    /**
     * The MenuBar.
     */
    private transient JMenuBar m_menuBar;

    /**
     * The Project menu.
     */
    private transient JMenu m_projectMenu;

    /**
     * The new project item (Project menu).
     */
    private transient JMenuItem m_newProjectMenuItem;

    /**
     * The open project item (Project menu).
     */
    private transient JMenuItem m_openProjectMenuItem;

    /**
     * The save project item (Project menu).
     */
    private transient JMenuItem m_saveProjectMenuItem;

    /**
     * The exit item (Project menu).
     */
    private transient JMenuItem m_exitMenuItem;

    /**
     * The edit menu.
     */
    private transient JMenu m_editMenu;

    /**
     * The edit preferences menu item (edit menu).
     */
    private transient JMenuItem m_editPreferencesMenuItem;

    /**
     * The help menu.
     */
    private transient JMenu m_helpMenu;

    /**
     * The info item (help menu).
     */
    private transient JMenuItem m_infoMenuItem;

    /**
     * The help item (help menu)
     */
    private transient JMenuItem m_manualMenuItem;

    // ------------ Projekt- und Historyfenster -----------

    /**
     * The splitPane for the ProjectWindow and the HistoryWindow.
     */
    private transient JSplitPane m_leftWindow;

    /**
     * The ProjectWindow.
     */
    private transient ProjectWindow m_projectWindow;

    /**
     * The HistoryWindow.
     */
    private transient HistoryWindow m_historyWindow;

    // ----------- Hauptfenster  ---------------------------

    /**
     * The desktopPane for the module windows.
     */
    private transient JDesktopPane m_desktopPane;

    /**
     * The SplitPane for the main window
     */
     private transient JSplitPane m_mainPane;

     /**
      * The location of the divider
      */

    private transient int m_dividerLocation;

    // --------- Status Zeile ------------------------------

    /**
     * The status line. No functionality yet.
     */
    private transient StatusLine m_statusLine;

    // -------------------- Befehle/Actions -------------------------

    /**
     * The action class for the "edit preferences" item. It shows a new
     * PreferencesDialog and waits until it has finished.
     *
     * @see visalg.components.PreferencesDialog
     */
    private transient Action editPreferences =
            new AbstractAction("Edit Preferences") {

            public void actionPerformed(ActionEvent evt) {
            PreferencesDialog dialog=
                    new PreferencesDialog(
                        (JFrame )m_application.getHistoryWindowContainer(),
                        m_application.getHistory());
        }
    };

    /**
     * The Action class for the "New Project" item. It closes all windows
     * using resetLeftWindow() and resetDesktop() and then creates a new
     * History using Application.newHistory().
     *
     * @see visalg.basics.History
     * @see visalg.basics.Application#newHistory()
     * @see ApplicationWindow#resetLeftWindow()
     * @see ApplicationWindow#resetDesktop()
     */
    private transient Action newHistory = new AbstractAction("New Project") {

	    public void actionPerformed(ActionEvent evt) {
		resetLeftWindow();
		resetDesktop();
		m_application.newHistory();
	    }
	};

    /**
     * The Action class for the "Open Project" item. It creates a new
     * JFileChooser dialog and then calls Application.loadHistory(File). If
     * this was successfull, all windows are closed using resetLeftWindow() and
     * resetDesktop() and then new windows are created by calling
     * Application.newWindows().
     *
     * @see visalg.basics.Application#loadHistory(File)
     * @see visalg.basics.Application#newWindows()
     * @see ApplicationWindow#resetLeftWindow()
     * @see ApplicationWindow#resetDesktop()
     */
    private transient Action loadHistory = new AbstractAction("Open Project...") {

	    public void actionPerformed(ActionEvent evt) {

		ProjectFileFilter ff = new ProjectFileFilter();

        JFileChooser d = new JFileChooser();
		d.setCurrentDirectory(new File("." + File.separator + "projects"));
        d.setAcceptAllFileFilterUsed(true);
        d.setFileFilter(ff);
		d.setMultiSelectionEnabled(false);

		int result = d.showOpenDialog(m_contentPane);
		if (result == JFileChooser.CANCEL_OPTION) {
		    return;
		}

		if (!(m_application.loadHistory(d.getSelectedFile()))) {
		    JOptionPane.showMessageDialog(null,
						  "Project could not be loaded.",
						  "Open Project",
						  JOptionPane.ERROR_MESSAGE);
		}
		else {
		    resetLeftWindow();
		    resetDesktop();
            m_application.afterDeserialisation();
		}
	    }
	};

    /**
     * The Action class for the "Save Project" item. It creates a new
     * JFileChooser dialog and then closes all open windows using
     * resetLeftWindow() and resetDesktop(). After that,
     * Application.saveHistory(File) is called in order to store the file on
     * disk. Then all windows are restored by calling Application.newWindows().
     *
     * @see ApplicationWindow#resetLeftWindow()
     * @see ApplicationWindow#resetDesktop()
     * @see visalg.basics.Application#saveHistory(File)
     * @see visalg.basics.Application#newWindows()
     */
    private transient Action saveHistory = new AbstractAction("Save Project...") {

	    public void actionPerformed(ActionEvent evt) {

        ProjectFileFilter ff = new ProjectFileFilter();

		JFileChooser d = new JFileChooser();
		d.setCurrentDirectory(new File("." + File.separator + "projects"));
        d.setAcceptAllFileFilterUsed(true);
        d.setFileFilter(ff);
		d.setMultiSelectionEnabled(false);

		int result = d.showSaveDialog(m_contentPane);
		if (result == JFileChooser.CANCEL_OPTION) {
		    return;
		}

        resetLeftWindow();
		resetDesktop();
		if (!(m_application.saveHistory(d.getSelectedFile()))) {
		    JOptionPane.showMessageDialog(null,
						  "Beim Speichern des Projekts trat ein Fehler auf.",
						  "Projekt speichern",
						  JOptionPane.ERROR_MESSAGE);
		}

		m_application.newWindows();
	    }
	};

    /**
     * The Action class for the "Exit" item. The application returns to the
     * system by calling Application.exit().
     *
     * @see visalg.basics.Application#exit()
     */
    private transient Action visAlgExit = new AbstractAction("Exit") {

	    public void actionPerformed(ActionEvent evt) {
		m_application.exit();
	    }
	};

    /**
     * The Action class for the "Info" item. Some usefull information is
     * printed on the standard output.
     */
    private transient Action visAlgInfo = new AbstractAction("Info") {

	    public void actionPerformed(ActionEvent evt) {
            String info="";
            info += "<HTML>VisAlg - Visualisation of Algorithms</HTML>\n\n";
            info += "<HTML><P><B>Developers :</B></HTML>\n\n";
            info += "<HTML>Marcus Oswald</HTML> \n";
            info += "<HTML>Dino Ahr</HTML> \n";
            info += "<HTML>Martin Spoden</HTML> \n";
            info += "<HTML>Matthias Vigelius</HTML> \n\n";
            info += "<HTML><br>mailto:mvigelius@gmx.de</HTML>";
            JOptionPane.showMessageDialog(null,info, "VisAlg V1.2",
                JOptionPane.PLAIN_MESSAGE);
	    }
	};

    // ----------------------- Action Listeners ------------------------------
    class MainPanePropertyChangeListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getPropertyName()==JSplitPane.DIVIDER_LOCATION_PROPERTY) {
                m_dividerLocation=((Integer)evt.getNewValue()).intValue();
            }
        }
    }

    class ModuleWindowInternalFrameListener extends InternalFrameAdapter {
        public void internalFrameActivated(InternalFrameEvent e) {
            ModuleWindow mw = (ModuleWindow)e.getInternalFrame().getContentPane();
            mw.onFocusGained();
        }

        public void internalFrameDeactivated(InternalFrameEvent e) {
            ModuleWindow mw = (ModuleWindow)e.getInternalFrame().getContentPane();
            mw.onFocusLost();
        }
    }

    /**
     * This class represents the FileFilter of VisAlg project files. The
     * extension for those files is ".vpr"
     */
    class ProjectFileFilter extends FileFilter {
        public String getExtension(File f) {
            String ext = null;

            String s = f.getName();
            int i = s.lastIndexOf('.');

            if (i>0 && i<s.length()-1) {
                ext = s.substring(i+1).toLowerCase();
            } else {
                ext = new String();
            }

            return ext;
        }

        public boolean accept(File f) {
            boolean rvalue = false;

            if (f.isDirectory()) {
                return true;
            } else if (getExtension(f).equals("vpr")) {
                return true;
            }

            return false;
        }

        public String getDescription() {
            return "VisAlg Project Files (.vpr)";
        }
    }//class ProjectFileFilter

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Sets up the main window. First, some internal references are stored. Then
     * the menus are initialised using loadMenu(). A new DesktopManager is
     * created, that reacts on user actions on the ModuleWindows, like dragging
     * or resizing. The new window bounds of the window is stored using
     * Module.setModuleWindowBounds(boolean, Rectangle). After that, a new
     * DesktopPane and a temporary ProjectPane as well as a temporary
     * HistoryPane are created. Additionally, a new WindowListener is set up,
     * that reacts on the close event of the main window and calls
     * Application.exit().
     *
     * @see ApplicationWindow#loadMenu()
     * @see visalg.basics.Module#setModuleWindowBounds(boolean, Rectangle)
     * @see visalg.basics.Application#exit()
     */
    public ApplicationWindow(Application a, Rectangle r) {

	super("VisAlg V1.2");

	m_application = a;

	// Create HelpBroker object

        m_helpBroker = m_application.getHelpSet().createHelpBroker();

        // Parameter for main window
	setLocation(r.x, r.y);
	setSize(r.width, r.height);

	m_contentPane = getContentPane();
	m_contentPane.setLayout(new BorderLayout());

	// Setup menu bar (private method)
	loadMenu();

	// Hauptfenster Setup
	//
	// Wenn die Frames verndert werden (Gre/Position),
	// schreibt der Desktop-Manager die Mae ins dazugehrige
	// Modul.
	DesktopManager myDesktopManager = new DefaultDesktopManager() {

	    public void closeFrame(JInternalFrame f) {
        ModuleWindow mw = (ModuleWindow)f.getContentPane();
        mw.onClose();
		storeBoundsInModule(f);
		super.closeFrame(f);
	    }

	    public void deactivateFrame(JInternalFrame f) {
		storeBoundsInModule(f);
		super.deactivateFrame(f);
	    }

	    public void endDraggingFrame(JInternalFrame f) {
		storeBoundsInModule(f);
		super.endDraggingFrame(f);
	    }

	    public void endResizingFrame(JInternalFrame f) {
		storeBoundsInModule(f);
		super.endResizingFrame(f);
	    }

	    public void setBoundsForFrame(JComponent f, int newX, int newY, int newWidth, int newHeight) {
		storeBoundsInModule((JInternalFrame)f);
		super.setBoundsForFrame(f, newX, newY, newWidth, newHeight);
	    }

	    public void setPreviousBounds(JInternalFrame f, Rectangle r) {
		storeBoundsInModule(f);
		super.setPreviousBounds(f, r);
	    }

	    private void storeBoundsInModule(JInternalFrame f) {
		ModuleWindow mw = (ModuleWindow)f.getContentPane();
		mw.getModule().setModuleWindowBounds(!f.isClosed(), f.getBounds());
	    }
	};

	// Create the whole pane as split pane
        m_mainPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        m_contentPane.add(m_mainPane, BorderLayout.CENTER);

        // Create desktop pane
	m_desktopPane = new JDesktopPane();
	m_desktopPane.setDesktopManager(myDesktopManager);
	//m_contentPane.add(m_desktopPane, BorderLayout.CENTER);
        m_mainPane.setRightComponent(m_desktopPane);
	resetDesktop();

	// Setup des linken Teilfensters (Project und History)
	m_leftWindow = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    m_leftWindow.setSize(r.height,r.width/2);
    m_dividerLocation = m_leftWindow.getDividerLocation();
    m_leftWindow.addPropertyChangeListener(new MainPanePropertyChangeListener());

	//m_contentPane.add(m_leftWindow, BorderLayout.WEST);
        m_mainPane.setLeftComponent(m_leftWindow);

	resetLeftWindow(); // Einsetzen von temporaeren Fenstern

	// Setup der Stausline
	m_statusLine = new StatusLine();
        JScrollPane scrollpane = new JScrollPane(m_statusLine);
	m_contentPane.add(scrollpane, BorderLayout.SOUTH);

	// Listener fuer Schliessen des Fensters
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		Window w = e.getWindow();
		w.setVisible(false);
		w.dispose();
		m_application.exit();
	    }
	});
    } // ApplicationWindow(Application a, Rectangle r)


    // --------- von Interface MenuBarContainer ----------------

    /**
     * Adds a new menu to the menu bar. Every window, that adds a menu item this
     * way, is responsible for also removing it, if the window closes.
     *
     * @param m The new menu
     *
     */
    public void addMenu(JMenu m) {
        // First removing the help menu
        m_menuBar.remove(m_helpMenu);

        // add the menu
        m_menuBar.add(m);

        // add the help menu
        m_menuBar.add(m_helpMenu);

        // repaint
        m_menuBar.revalidate();
    }//addMenu(JMenu)

    /**
     * Removes a menu from the menu bar.
     *
     * @param m The menu to be removed
     *
     */
    public void removeMenu(JMenu m) {
        m_menuBar.remove(m);
        m_menuBar.revalidate();
    }//removeMenu(JMenu)


    // ----------- von Interface ProjectWindowContainer ------------------

    /**
     * Sets a new ProjectWindow, that is shown on the top of the SplitPane.
     *
     * @param pw The ProjectWindow
     *
     * @see visalg.graphics.ProjectWindow
     */
    public void setProjectWindow(ProjectWindow pw) {
	if (pw != null)	{
	    m_leftWindow.setLeftComponent(pw);
	}
	// set divider
        m_leftWindow.setDividerLocation(m_dividerLocation);

 	m_leftWindow.repaint();
    }


    // -------- from Interface HistoryWindowContainer ------

    /**
     * Sets a new HistoryWindow, that is shown on the bottom side of the
     * SplitPane.
     *
     * @see visalg.graphics.HistoryWindow
     * @param hw The new HistoryWindow
     */
    public void setHistoryWindow(HistoryWindow hw) {
	if (hw != null)	{
	    m_leftWindow.setRightComponent(hw);
	}

	// set divider
        m_leftWindow.setDividerLocation(m_dividerLocation);

	m_leftWindow.repaint();
    }


    //  ----------- von Interface ModuleWindowContainer -------------------

    /**
     * Adds a new ModuleWindow to the desktop pane using an internal frame.
     * It is set to the top and
     * after that, Module.setShowing(true) is called.
     * @param mw Reference to the ModuleWindow
     * @param bounds The desired window bounds for this ModuleWindow
     *
     * @see visalg.graphics.ModuleWindow
     * @see visalg.basics.Module#setShowing(boolean)
     */
    public void addModuleWindow(ModuleWindow mw, Rectangle bounds) {

	if (mw == null) {
	    System.err.println("ApplicationWindow.addModuleWindow: Given ModuleWindow is null!");
	    return;
	}

	// Create Internal frame
	JInternalFrame myInternalFrame =
	    new JInternalFrame(mw.getName(),true,true,true,true);
	myInternalFrame.setBounds(bounds);
	myInternalFrame.setContentPane(mw);

    myInternalFrame.addInternalFrameListener(new ModuleWindowInternalFrameListener());

    // Put it on the desktop
	m_desktopPane.add(myInternalFrame, JLayeredPane.PALETTE_LAYER);
	m_desktopPane.moveToFront(myInternalFrame); // Bring it on the top
	myInternalFrame.setVisible(true);

        mw.getModule().setShowing(true);
    }//addModule(ModuleWindow, Rectangle)

    /**
     * Removes the JInternalFrame, that represents the specified ModuleWindow.
     * Module.setVisible(false) is called in order to tell the module, that
     * it does not have a window anymore.
     * @param mw The ModuleWindow to be closed
     * @see visalg.graphics.ModuleWindow
     * @see visalg.basics.Module#setShowing(boolean)
     */
    public void removeModuleWindow(ModuleWindow mw) {

	JInternalFrame[] frames = m_desktopPane.getAllFrames();
	for (int i = frames.length - 1; i >= 0; i--) {
	    ModuleWindow cmw = (ModuleWindow)frames[i].getContentPane();
	    if (cmw == mw) {
		m_desktopPane.remove(i);
	    }
	}

        mw.getModule().setShowing(false);

	m_desktopPane.invalidate();
	m_desktopPane.repaint();
    }//removeModuleWindow(ModuleWindow)

    /**
     * Closes all ModuleWindows. This done by calling resetDesktop().
     *
     * @see visalg.graphics.ModuleWindow
     * @see ApplicationWindow#resetDesktop()
     */
    public void discardModuleWindows() {
	resetDesktop();
    }//discardModuleWindows()


    // -----------------------------------------------------
    // ------- M e t h o d s ( p r i v a t e ) -------------
    // -----------------------------------------------------

    /**
     * Sets up temporary ProjectWindow and temporary HistoryWindow.
     */
    private void resetLeftWindow() {

	// Project
	JPanel tmpProjectPanel = new JPanel();
	tmpProjectPanel.add(new JLabel("Loading Project..."));
	m_leftWindow.setLeftComponent(tmpProjectPanel);
	m_projectWindow = null;

	// History
	JPanel tmpHistoryPanel = new JPanel();
	tmpHistoryPanel.add(new JLabel("Loading History Window..."));
	m_leftWindow.setRightComponent(tmpHistoryPanel);
	m_historyWindow = null;

	// set divider
        m_leftWindow.setDividerLocation(m_dividerLocation);

       // Anzeige erneuern
	m_leftWindow.revalidate();

    }//resetLeftWindow

    /**
     * Closes all ModuleWindows and repaints a clean desktop.
     *
     * @see visalg.graphics.ModuleWindow
     */
    private void resetDesktop() {
	m_desktopPane.removeAll();
	m_desktopPane.invalidate();
	m_desktopPane.repaint();
    }//resetDesktop()

    /**
     * Creates all menus. Because the menubar is then added to the main window,
     * m_contentPane must be set up already.
     */
    private void loadMenu() {

	// Project
	m_projectMenu = new JMenu("File");
	m_projectMenu.setMnemonic('F');
	m_newProjectMenuItem = m_projectMenu.add(newHistory);
	m_projectMenu.addSeparator();
	m_openProjectMenuItem = m_projectMenu.add(loadHistory);
	m_saveProjectMenuItem = m_projectMenu.add(saveHistory);
	m_projectMenu.addSeparator();
	m_exitMenuItem = m_projectMenu.add(visAlgExit);

        // Edit
        m_editMenu = new JMenu("Edit");
        m_editMenu.setMnemonic('E');
        m_editPreferencesMenuItem = m_editMenu.add(editPreferences);

	// Help
	m_helpMenu = new JMenu("Help");
	m_helpMenu.setMnemonic('H');
	m_infoMenuItem = m_helpMenu.add(visAlgInfo);
        m_manualMenuItem = new JMenuItem("Manual ...");
        m_manualMenuItem.addActionListener(new CSH.DisplayHelpFromSource(m_helpBroker));
        m_helpMenu.add(m_manualMenuItem);

	// Menu Bar
	m_menuBar = new JMenuBar();
	m_menuBar.add(m_projectMenu);
        m_menuBar.add(m_editMenu);
	m_menuBar.add(m_helpMenu);

	// Put Menu bar into main window
	m_contentPane.add(m_menuBar, BorderLayout.NORTH);

    } // loadMenu()

    /**
     * Deletes the old MenuBar and reloads it.
     */
    public void newMenu()
    {
        m_contentPane.remove(m_menuBar);
        loadMenu();
    }//newMenu()

    /**
     * Sets the divider location for the left window.
     */
    public void setLeftWindowDivider(int loc) {
        m_leftWindow.setDividerLocation(loc);
        m_dividerLocation = m_leftWindow.getDividerLocation();
    }
} // class ApplicationWindow