////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : DataModuleWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Wed Oct  4 11:28:23 2000
//  Update count     : 33
//  CVS Info         : $Id: DataModuleWindow.java,v 1.2 2002/03/18 12:28:48 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name	    Changes/Extensions
//  ----	-----       ------------------
//  27.02.01    vigelius    Enhanced documentation
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//  14.03.01    vigelius    module now writes the filename of the data to its
//                          string
//  18.06.01    vigelius    uses the main menu bar.
//  27.06.01    vigelius    Adapted to the new Menu concept
//  29.10.01    vigelius    Added reset menu button
//  14.11.01    vigelius    Added the possibility to create an empty data set
//  15.11.01    vigelius    Deleted disturbing File dialog
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import visalg.graphics.ModuleWindow;
import visalg.basics.DataModule;
import visalg.basics.Converter;
import visalg.basics.ObjectReader;
import visalg.types.ModuleChangeEvent;
import visalg.components.OpenViewerDialog;
import visalg.components.ClassTypeDialog;

import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.AbstractAction;
import javax.swing.event.ChangeEvent;

import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;


/**
 * The class for the DataModule window.It provides functionality for the user
 * to create new data out of different file formats, to save and load data
 * as whole classes and to print out the data to the standard window.
 *
 * @see visalg.basics.DataModule
 */
public class DataModuleWindow extends ModuleWindow implements ActionListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Label which indicates what type of class the data is.
     */
    private transient JLabel m_classLabel;

    /**
     * Reference to the corresponding dataModule.
     */
    private transient DataModule m_dataModule;

    /**
     * Icon for the "create data" item.
     * @serial
     */
    private static ImageIcon s_newIcon = new ImageIcon("images/new.gif");

    /**
     * Icon for the "open data" item
     * @serial
     */
    private static ImageIcon s_openIcon = new ImageIcon("images/open.gif");

    /**
     * Icon for the "save data" item
     * @serial
     */
    private static ImageIcon s_saveIcon = new ImageIcon("images/save.gif");

    /**
     * The menu.
     */
    private JMenu m_mnuData;

    // -------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) ---------
    // -------------------------------------------------

    /**
     * Creates a new window with a menu bar, the appropriate ActionListener
     * and sets a reference to the DataModule.
     *
     * @see visalg.basics.DataModule
     */
    public DataModuleWindow(String s,
			    DataModule dm,
			    Rectangle bounds,
			    ModuleWindowContainer mwc) {

	// Call constructor of ModuleWindow
	super(s, dm, bounds, mwc);

	m_dataModule = dm;

	JMenuBar mnuBar = new JMenuBar();


    //mnuBar.add(m_mnuData);

	// Setup panel
	m_contentPane.setLayout(new BorderLayout());
	m_contentPane.add(mnuBar, BorderLayout.NORTH);

	// Create class label and add it to the panel
	m_classLabel = new JLabel("(The module does not contain data.)");
	if (m_dataModule.getData() != null) {
	    m_classLabel.setText("Datatype: " + m_dataModule.getDataClass());
	}
	m_contentPane.add(m_classLabel, BorderLayout.CENTER);

	// Create button and add it to the panel
	// The class DataModuleWindow acts as ActionListener
	JButton systemOutButton = new JButton("System.out");
	systemOutButton.addActionListener(this);
	m_contentPane.add(systemOutButton, BorderLayout.SOUTH);

	// Repaint
	mnuBar.invalidate();
	revalidate();
	repaint();

    } // public DataModuleWindow(..)

    /**
     * Removes the menu.
     */
    public void onClose() {
    }

    public void onFocusGained() {

        // Instantiate Action classes
    	NewDataAction newDataAction = new NewDataAction();
        ImportDataAction  importDataAction  = new ImportDataAction();
	    OpenDataAction openDataAction = new OpenDataAction();
    	SaveDataAction saveDataAction = new SaveDataAction();
        OpenViewerAction openViewerAction = new OpenViewerAction();
        ResetDataAction resetDataAction = new ResetDataAction();

        m_mnuData = new JMenu("Data");
	    // Create menu bar
        m_mnuData.add(newDataAction);
        m_mnuData.addSeparator();
    	m_mnuData.add(importDataAction);
        m_mnuData.addSeparator();
	    m_mnuData.add(openDataAction);
    	m_mnuData.add(saveDataAction);
        m_mnuData.addSeparator();
        m_mnuData.add(openViewerAction);
        m_mnuData.addSeparator();
        m_mnuData.add(resetDataAction);

        getModule().getModuleManager().getMenuBarContainer().addMenu(m_mnuData);
    }

    public void onFocusLost() {
        getModule().getModuleManager().getMenuBarContainer().removeMenu(m_mnuData);
    }
    // -------- from interface ActionListener  -------------

    /**
     * This routine is triggered if the button "System.out" is pushed. The
     * content of the data will be printed to stdout.
     * @TODO Currently this
     * routine assumes the data to be an array of any type. This has to be
     * changed in future.
     */
    public void actionPerformed(ActionEvent evt) {
	m_dataModule.printData();
    }//actionPerformed

    // ----------- from class ModuleWindow (resp. Interface ModuleListener) ------------------

    /**
     * Will be triggered if the dataModule has changed and changes the text
     * that shows the label.
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
	if (m_dataModule.getData()!=null) {
	    m_classLabel.setText("Data type: " + m_dataModule.getDataClass());
	} else {
	    m_classLabel.setText("(The module contains no data.)");
	}
	revalidate();
	repaint();
    }//moduleChanged(ChangeEvent)

    // -----------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------
    // -----------------------------------------------


    // -------------------- class NewDataAction -------------------------
    /**
     * This class represents the "New Data" Menu item. By choosing "New Data",
     * an empty data set of a specified type will be created. The user has
     * to select the type out of a dialog.
     */
    class NewDataAction extends AbstractAction {
        public NewDataAction() {
            super("New Data ...", s_newIcon);
        }//NewDataAction()

        public void actionPerformed(ActionEvent evt) {
                ClassTypeDialog md = new ClassTypeDialog(m_dataModule);
        }//actionPerformed(ActionEvent)
    }//NewDataAction
    // -------------------- class ImportDataAction ----------------------

    /**
     * The internal class that represents the "Import data" menubar item.
     * If the user chooses this item, a dialog pops up allowing the user to
     * choose a file containing the data, either by directly entering the path
     * or by using a JFileChooser dialog. If the data file contains the
     * ObjectReader.converterMagicNumber, the appropriate converter is created,
     * which then tries to convert the file. If no converter is given in the
     * file, the user will be asked, to choose one manually. If the conversion
     * of the file succeeded, the new data object is send to the DataModule
     * using DataModule.setData(Object). Otherwise, an error message or an
     * error dialog will pop up.
     *
     * @see visalg.basics.ObjectReader#converterMagicNumber
     * @see visalg.basics.DataModule
     * @see visalg.basics.DataModule#setData(Object)
     * @see visalg.basics.Converter
     */
    class ImportDataAction extends AbstractAction {

	public ImportDataAction() {
	    super("Import data...");
	}

	// Will be triggered for the menu entry "Import data..."
	public void actionPerformed(ActionEvent evt) {

        // Create panel asking for the input file
	    final JPanel myPanel = new JPanel();

	    // Label
	    myPanel.add(new JLabel("Source file: "));

	    // TextField
	    final JTextField myFile = new JTextField(30);
	    myPanel.add(myFile);

	    // Button for using the fileChooser
	    JButton chooseFileButton = new JButton("Choose File...");

	    // Create ActionListener for file chooser button
	    chooseFileButton.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent e) {

		    JFileChooser d = new JFileChooser();
		    d.setCurrentDirectory(new File("." + File.separator + "import"));
		    d.setMultiSelectionEnabled(false);

		    // Show file chooser
		    int result = d.showOpenDialog(m_contentPane);

		    // User has pushed the Cancel button
		    if (result == JFileChooser.CANCEL_OPTION) {
			return;
		    }

            File f;
            if (d.getSelectedFile()!=null) {
    		    f = d.getSelectedFile();
            } else {
                return;
            }

		    // Put path of selected file into text field
		    myFile.setText(f.getAbsolutePath());
		    myFile.invalidate();
		    myPanel.revalidate();
		}
	    });

	    // Add button
	    myPanel.add(chooseFileButton);


        JFileChooser chooser = new JFileChooser();
        chooser.setCurrentDirectory(new File("." + File.separator + "import"));
        chooser.setMultiSelectionEnabled(false);

        File f = null;

        if (chooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION) {
            f = chooser.getSelectedFile();
        }

	    // Wenn keine Datei gewhlt, Abbruch mit Fehlermeldung
	    if ((f == null) || !(f.exists())) {
		JOptionPane.showMessageDialog(null,
					      "File does not exist!",
					      "New data...",
					      JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Datei vorhanden - interpretieren...
	    //... erste Zeile lesen
	    String line = null;
	    try {
		BufferedReader in = new BufferedReader(new FileReader(f));
		line = in.readLine();
		in.close();
	    }
	    catch (IOException e) {
		System.err.println(e);
		JOptionPane.showMessageDialog(null,
					      "Error while reading file " + f.getAbsolutePath() + ".",
					      "New data...",
					      JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Fall 1: line = null -> keine Daten zu lesen (leere Datei)
	    if (line == null) {
		JOptionPane.showMessageDialog(null,
					      "File is empty!",
					      "New data...",
					      JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Fall 2: Erstes Zeichen entspricht
	    // visalg.basics.ObjectReader.converterMagicNumber und
	    // nachfolgend ist eine gltige Konverterklassendatei angegeben
	    if (line.startsWith(ObjectReader.converterMagicNumber)) {

		line = line.substring(ObjectReader.converterMagicNumber.length());

		Converter converter = null;
		try {
		    Class c = ClassLoader.getSystemClassLoader().loadClass(line);

		    // Sicherheitslcke!!! (einfach instantiieren, um
		    // Interface zu prfen, aber...  ... wie sonst?)
		    Object o = c.newInstance();
		    if (o instanceof Converter) {
			converter = (Converter) o;
		    }
		}
		catch (Exception e) {
		    System.err.println(e);
		}

		// Falls Konverter gefunden und Konvertierung
		// erfolgreich data setzen, return.  Sonst manuelle
		// Auswahl des Konverters (Fall 3))
		if (converter != null) {

		    Object converterObject = converter.getData(f);

		    if (converterObject != null) {
                        m_dataModule.setString(f.getName());
        		m_dataModule.setNewData(converterObject);
 			return;
		    }
		    else {
			JOptionPane.showMessageDialog(null,
						      "Automatic conversion failed!",
						      "New data...",
						      JOptionPane.ERROR_MESSAGE);
		    }
		}
	    }

	    // Fall 3: kein Konverter (wird auch erreicht, wenn der
	    // spezifizierte Konverter nicht existiert)
	    JOptionPane.showMessageDialog(null,
					  "You have to select the class file of an apropiate converter!",
					  "New data...",
					  JOptionPane.PLAIN_MESSAGE);

	    myPanel.removeAll();
	    myPanel.add(new JLabel("Converter class file: "));
	    myFile.setText("");
	    myPanel.add(myFile);

	    chooseFileButton = new JButton("Choose file...");
	    chooseFileButton.addActionListener(new ActionListener() {

		public void actionPerformed(ActionEvent e) {

		    JFileChooser d = new JFileChooser();
		    d.setCurrentDirectory(new File("." + File.separator
						   + "visalg" + File.separator
						   + "converter"));
		    d.setMultiSelectionEnabled(false);

		    int result = d.showOpenDialog(m_contentPane);
		    if (result == JFileChooser.CANCEL_OPTION) {
			return;
		    }

		    File ff = d.getSelectedFile();
		    myFile.setText(ff.getAbsolutePath());
		    myFile.invalidate();
		    myPanel.revalidate();
		}
	    });

	    myPanel.add(chooseFileButton);

	    File converterFile = null;
	    if (JOptionPane.OK_OPTION ==
		JOptionPane.showConfirmDialog(null, myPanel, "New data...",
					      JOptionPane.OK_CANCEL_OPTION)) {
		converterFile = new File(myFile.getText());
	    }
	    else {
		return; //nicht OK-Button gedrckt...
	    }

	    //Wenn keine Datei gewhlt, Abbruch mit Fehlermeldung
	    if ( (converterFile == null) || !(converterFile.exists()) ) {
		JOptionPane.showMessageDialog(null,
					      "File does not exist!",
					      "New data...",
					      JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    // Datei vorhanden - interpretieren...  letztes visalg im Pfad soll
	    // zu visalg.* umgebaut werden... sonst: Versagen des Einlesens!
	    line = converterFile.getAbsolutePath();
	    if (line.endsWith(".class")) {
		line = line.substring(0, line.length() - 6);
	    }
	    while (line.indexOf("visalg") >= 0) {
		line = line.substring(line.indexOf("visalg")+6);
	    }
	    line = "visalg" + line;
	    line = line.replace(File.separatorChar, '.');

	    // line enthlt nun (hoffentlich) den Klassennamen...
	    // copy/paste von Fall 2 - bei Fehler endgltiger Abbruch...
	    Converter converter2 = null;
	    try {
		Class c2 = ClassLoader.getSystemClassLoader().loadClass(line);

		//Sicherheitslcke!!! (einfach instantiieren, um Interface zu prfen, aber...
		//... wie sonst?)
		Object o2 = c2.newInstance();
		if (o2 instanceof Converter) {
		    converter2 = (Converter) o2;
		}
	    }
	    catch (Exception e) {
		System.err.println("" + e);
	    }

	    // Falls Konverter gefunden und Konvertierung erfolgreich data
	    // setzen, return.  Sonst manuelle Auswahl des Konverters (Fall 3))
	    if (converter2 != null) {
		Object converterObject2 = converter2.getData(f);
		if (converterObject2 != null) {
                    m_dataModule.setString(f.getName());
		    m_dataModule.setNewData(converterObject2);
		    return;
		}
	    }
	    JOptionPane.showMessageDialog(null,
					  "Convertion failed!",
					  "New data...",
					  JOptionPane.ERROR_MESSAGE);
	} // actionPerformed
    } // class NewDataAction


    // ---------------- class OpenDataAction ------------------

    /**
     * The AbstractAction class representing the "Open data" menubar item.
     * A JFileChooser dialog pops up, asking the user to choose a file.
     * The class then calls DataModule.loadData(File). If the loading process
     * did not succeed, an error dialog pops up.
     *
     * @see visalg.basics.DataModule#loadData(File)
     */
    class OpenDataAction extends AbstractAction {

	public OpenDataAction() {
	    super("Open data...", s_openIcon);
	}

	public void actionPerformed(ActionEvent evt) {

	    JFileChooser d = new JFileChooser();
	    d.setCurrentDirectory(new File("." + File.separator + "data"));
	    d.setMultiSelectionEnabled(false);

	    int result = d.showOpenDialog(m_contentPane);
	    if (result == JFileChooser.CANCEL_OPTION) {
		return;
	    }

	    //wenn Datei ausgewhlt...
	    if (! m_dataModule.loadData(d.getSelectedFile())) {
		JOptionPane.showMessageDialog(null,
					      "File could not be opened!",
					      "Open data...",
					      JOptionPane.ERROR_MESSAGE);
	    }
	}
    } // class OpenDataAction

    // ------------------ class ResetDataAction ----------------------
    /**
     * This class represents the Reset Action. If the user chooses to
     * reset the data to its initial state, reset() of the DataModule
     * is called.
     */
    class ResetDataAction extends AbstractAction {
        public ResetDataAction() {
            super("Reset Data", s_openIcon);
        }

        public void actionPerformed(ActionEvent evt) {
            m_dataModule.reset();
        }
    }
    // ------------------ class SaveDataAction -----------------------

    /**
     * The AbstractAction class representing the "Save data as" item. A
     * JFileChooser dialog asks the user to choose a file. The class then tries,
     * to call DataModule.saveData(File) to save the data. If this did not
     * succeed, an error dialog pops up.
     *
     * @see visalg.basics.DataModule#saveData(File)
     */
    class SaveDataAction extends AbstractAction {

	public SaveDataAction() {
	    super("Save data as...", s_saveIcon);
	}

	public void actionPerformed(ActionEvent evt) {

	    if (m_dataModule.getData() == null) {
		JOptionPane.showMessageDialog(null,
					      "The module does not contain data to be saved!",
					      "Save data as...",
					      JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    JFileChooser d = new JFileChooser();
	    d.setCurrentDirectory(new File("." + File.separator + "data"));
	    d.setMultiSelectionEnabled(false);

	    int result = d.showSaveDialog(m_contentPane);
	    if (result == JFileChooser.CANCEL_OPTION) {
		return;
	    }

	    if (! m_dataModule.saveData(d.getSelectedFile())) {
		JOptionPane.showMessageDialog(null,
					      "Data could not be saved!",
					      "Save data as...",
					      JOptionPane.ERROR_MESSAGE);
	    }
	}
    } // class SaveDataAction

    class OpenViewerAction extends AbstractAction {

	public OpenViewerAction() {
	    super("Open Viewer Module ...");
	}

	public void actionPerformed(ActionEvent evt) {
        OpenViewerDialog ovd = new OpenViewerDialog((DataModule)getModule());
        ovd.show();
	}
    } // class OpenViewerAction

} // class DataModuleWindow
