////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : HistoryWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 22 15:26:32 2000
//  Update count     : 11
//  CVS Info         : $Id: HistoryWindow.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
// 27.02.01     vigelius        - added play/pause button and action
//                              - enhanced documentation
// 07.03.01     vigelius        added ToolTips
// 09.03.01     vigelius        - added popup menu and menubar menu
//                              - improved mouse handling
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import visalg.basics.History;
import visalg.basics.StepThread;
import visalg.graphics.HistoryWindowContainer;
import visalg.basics.Project;
import visalg.components.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.AbstractAction;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import javax.swing.event.*;

/**
 * The HistoryWindow is the GUI representation for all project and module
 * managing. It allows the user to save and reload certain project states as
 * well as to step the modules, either seperately or continously. Therefore,
 * it provides two toolbars, the CD panel for the module and project control,
 * and another panel for the managing of project states.
 *
 * @see History
 *
*/
public class HistoryWindow extends JPanel implements ActionListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the History itself.
     */
    private transient History m_history;

    /**
     * The table for visualizing the different Projects.
     */
    private transient JTable m_table;

    /**
     * A Slider control allowing the user to set the delay time for
     * continously stepping.
     *
     * @see visalg.basics.StepThread
     */
    private transient Slider m_slider;

    /**
     * Icons for the CD-Panel.
     */
    private static ImageIcon firstIcon = new ImageIcon("images/first.gif");
    private static ImageIcon previousIcon = new ImageIcon("images/previous.gif");
    private static ImageIcon undoIcon = new ImageIcon("images/undo.gif");
    private static ImageIcon stepIcon = new ImageIcon("images/step.gif");
    private static ImageIcon nextIcon = new ImageIcon("images/next.gif");
    private static ImageIcon lastIcon = new ImageIcon("images/last.gif");
    private static ImageIcon pauseIcon = new ImageIcon("images/pause.gif");

    /**
     * Icons for the project states panel.
     */
    private static ImageIcon loadIcon = new ImageIcon("images/open.gif");
    private static ImageIcon saveIcon = new ImageIcon("images/save.gif");
    private static ImageIcon deleteIcon = new ImageIcon("images/delete.gif");

    // ------------------------- menu : History -----------------------------

    /**
     * The History Menu.
     */
    private transient JMenu m_historyMenu;
    private transient JPopupMenu m_historyPopupMenu;

    private transient JMenuItem m_historyPreviousItem;
    private transient JMenuItem m_historyUndoItem;
    private transient JMenuItem m_historyStepItem;
    private transient JMenuItem m_historyNextItem;
    private transient JMenuItem m_historyLoadItem;
    private transient JMenuItem m_historySaveItem;
    private transient JMenuItem m_historyDeleteItem;

    private transient JMenuItem m_historyPreviousPopupItem;
    private transient JMenuItem m_historyUndoPopupItem;
    private transient JMenuItem m_historyStepPopupItem;
    private transient JMenuItem m_historyNextPopupItem;
    private transient JMenuItem m_historyLoadPopupItem;
    private transient JMenuItem m_historySavePopupItem;
    private transient JMenuItem m_historyDeletePopupItem;

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Sets up the panels including the appropriate Actions and the table, that
     * contains all saved project states. After that, the window will register
     * itself using HistoryWindowContainer.setHistoryWindow(HistoryWindow).
     *
     * @see visalg.graphics.HistoryWindowContainer#setHistoryWindow(HistoryWindow)
     */
    public HistoryWindow(History h, HistoryWindowContainer hwc) {

	super();
	m_history = h;

    // Create new slider

    m_slider = new Slider(0,1000,100, new SpeedChanged());

	// Define Layout
	setLayout(new BorderLayout());

	// Setup CD Panel (TOP side)
	JPanel cdPanel = new JPanel();
        IconButton prevActionButton = new IconButton(new previousAction());
        prevActionButton.setToolTipText("Load previous saved project");
	cdPanel.add(prevActionButton);
        IconButton undoActionButton = new IconButton(new undoAction());
        undoActionButton.setToolTipText("Undo last step");
	cdPanel.add(undoActionButton);
        IconButton stepActionButton = new IconButton(new stepAction());
        stepActionButton.setToolTipText("Step");
	cdPanel.add(stepActionButton);
        IconButton changePlayActionButton = new IconButton(new changePlayAction());
        changePlayActionButton.setToolTipText("Start stepping");
	cdPanel.add(changePlayActionButton);
        IconButton nextActionButton = new IconButton(new nextAction());
        nextActionButton.setToolTipText("Load next saved project");
	cdPanel.add(nextActionButton);
        cdPanel.add(m_slider);
	add(cdPanel, BorderLayout.NORTH);

	// Setup Table
	TableModel dataModel = new AbstractTableModel() {

		public int getColumnCount() {
		    return 2;
		}

		public int getRowCount() {
		    return m_history.getHistorySize();
		}

		public Object getValueAt(int row, int col) {

		    Project p = m_history.getProject(row);

		    // No project available
		    if (p == null) {
			return "";
		    }

		    // Column 0 contains time stamps
		    if (col == 0) {
			return new Integer(p.getTime());
		    }
		    else {
			return p.getComment();
		    }
		}

		public String getColumnName(int columnIndex) {
		    if (columnIndex == 0)
			return "Zeit";
		    if (columnIndex == 1)
			return "Projekt";
		    return "";
		}

		public boolean isCellEditable(int rowIndex, int columnIndex) {
		    if (columnIndex == 1)
			return true;
		    return false;
		}

		// Will be called when the Comment concerning the
		// project has been edited
		public void setValueAt(Object aValue, int rowIndex,
				       int columnIndex) {

		    if (columnIndex != 1)
			return;

		    if (aValue instanceof String) {
			Project p = m_history.getProject(rowIndex);
			if (p == null)
			    return;
			p.setComment((String) aValue);
		    }

		}

	    }; // end of AbstractTableModel() definition

	// Create table object
	m_table = new JTable(dataModel);
        m_table.setRowSelectionAllowed(true);
        m_table.setColumnSelectionAllowed(false);
        m_table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	// Customize first column
	TableColumn col = m_table.getColumn(m_table.getColumnName(0));
	col.setWidth(45);
	col.setPreferredWidth(45);
	col.setMaxWidth(45);

	// Put Table onto the panel (inside a SrollPane)
	JScrollPane scrollpane = new JScrollPane(m_table);
	add(scrollpane, BorderLayout.CENTER);
        scrollpane.addMouseListener(new HistoryMouseAdapter());
	// Setup CD Panel (BOTTOM side)
	cdPanel = new JPanel();
        IconButton loadActionButton = new IconButton(new loadAction());
        loadActionButton.setToolTipText("Load selected project");
	cdPanel.add(loadActionButton);
        IconButton saveActionButton = new IconButton(new saveAction());
        saveActionButton.setToolTipText("Save current project");
	cdPanel.add(saveActionButton);
        IconButton delActionButton = new IconButton(new deleteAction());
        delActionButton.setToolTipText("Delete selected project");
        cdPanel.add(delActionButton);
	add(cdPanel, BorderLayout.SOUTH);

	// HistoryWindow im HistoryWindowContainer anzeigen
	hwc.setHistoryWindow(this);

        loadMenu();
    }// HistoryWindow(History h, HistoryWindowContainer hwc)

    /**
     * Sets the new border values for the slider. This is used by the
     * PreferencesDialog.
     *
     * @param min The minimum value (in milliseconds)
     * @param max The maximum value (in milliseconds)
     *
     * @see visalg.components.PreferencesDialog
     */
    public void setDelayTimeProperties(int min, int max) {
        m_slider.setMinimum(min);
        m_slider.setMaximum(max);
    }//setDelayTimeProperties(int,int)

    /**
     * Returns the minimum value of the slider. This is used by the
     * PreferencesDialog.
     *
     * @return The minimum value (in milliseconds)
     *
     * @see visalg.components.PreferencesDialog
     */
    public int getDelayTimeMinimum() {
        return m_slider.getMinimum();
    }//getDelayTimeMinimum

    /**
     * Returns the maximum value of the slider. This is used by the
     * PreferencesDialog.-
     *
     * @return The maximum value (in milliseconds)
     *
     * @see visalg.components.PreferencesDialog
     */
    public int getDelayTimeMaximum() {
        return m_slider.getMaximum();
    }//getDelayTimeMaximum()

   // -------- from interface ActionListener --------------

    /**
     * Does nothing.
     *
     * @todo Maybe remove this function and the ActionListener interface.
     */
    public void actionPerformed(ActionEvent e) {
	// NOP
    }//actionPerformed(ActionEvent)


    // ------------------ private classes -------------------

    /**
     * Sets up the History menu as well as the associated Popup menu and adds
     * it to the MenuBarContainer.
     *
     * @see MenuBarContainer
     */
    private void loadMenu() {
        m_historyMenu = new JMenu("History");
        m_historyMenu.setMnemonic('H');

        m_historyLoadItem = m_historyMenu.add(new loadAction());
        m_historySaveItem = m_historyMenu.add(new saveAction());
        m_historyDeleteItem = m_historyMenu.add(new deleteAction());
        m_historyMenu.addSeparator();
        m_historyPreviousItem = m_historyMenu.add(new previousAction());
        m_historyNextItem = m_historyMenu.add(new nextAction());
        m_historyMenu.addSeparator();
        m_historyStepItem = m_historyMenu.add(new stepAction());
        m_historyUndoItem = m_historyMenu.add(new undoAction());

        m_historyMenu.getPopupMenu().addPopupMenuListener(new HistoryPopupMenuListener());
        m_history.getMenuBarContainer().addMenu(m_historyMenu);

        m_historyPopupMenu = new JPopupMenu("History");
        m_historyLoadPopupItem = m_historyPopupMenu.add(new loadAction());
        m_historySavePopupItem = m_historyPopupMenu.add(new saveAction());
        m_historyDeletePopupItem = m_historyPopupMenu.add(new deleteAction());
        m_historyPopupMenu.addSeparator();
        m_historyPreviousPopupItem = m_historyPopupMenu.add(new previousAction());
        m_historyNextPopupItem = m_historyPopupMenu.add(new nextAction());
        m_historyPopupMenu.addSeparator();
        m_historyStepPopupItem = m_historyPopupMenu.add(new stepAction());
        m_historyUndoPopupItem = m_historyPopupMenu.add(new undoAction());

        m_historyPopupMenu.addPopupMenuListener(new HistoryPopupMenuListener());

        m_table.add(m_historyPopupMenu);
        m_table.addMouseListener(new HistoryMouseAdapter());
    }

    // -----------------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------------
    // -----------------------------------------------------

    /**
     * The ChangeListener class for the Slider. If a change occurred, the
     * new value is read out and passed to the History using
     * History.setDelayTime(int)
     *
     * @see visalg.components.Slider
     * @see visalg.basics.History#setDelayTime(int)
     */
    class SpeedChanged implements ChangeListener {

        public void stateChanged(ChangeEvent evt) {
            int value = ((JSlider)evt.getSource()).getValue();
            m_history.setDelayTime(value);
        }//stateChanged
    }//class SpeedChanged

    /**
     * The MouseEvent listener for the table. Checks, if the right mouse button
     * has been pressed and shows the PopupMenu. It also selects
     * a project state, if the user right clicked on it.
     */
    class HistoryMouseAdapter extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (e.getModifiers()==InputEvent.BUTTON3_MASK) {
                if (e.getSource()==m_table) {
                    int selRow = m_table.rowAtPoint(e.getPoint());

                    if (selRow != -1) {
                        m_table.setRowSelectionInterval(selRow, selRow);
                    }
                } else {
                    m_table.clearSelection();
                }
                m_historyPopupMenu.show((JComponent) e.getSource(),e.getX(), e.getY());
            }
        }
    }//HistoryMouseAdapter

    /**
     * The Popup menu event listener, used to activate/inactivate the menu items
     * depending on if a row is selected or not.
     */
    class HistoryPopupMenuListener implements PopupMenuListener {
        public void popupMenuCanceled(PopupMenuEvent e) {}
        public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {}
        public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
            boolean active = false;
            if (m_table.getSelectedRowCount()==1) {
                active = true;
            }

            m_historyDeleteItem.setEnabled(active);
            m_historyDeletePopupItem.setEnabled(active);
            m_historyLoadItem.setEnabled(active);
            m_historyLoadPopupItem.setEnabled(active);
            m_historyNextItem.setEnabled(active);
            m_historyNextPopupItem.setEnabled(active);
            m_historyPreviousItem.setEnabled(active);
            m_historyPreviousPopupItem.setEnabled(active);
        }
    }
    // ------------- Action classes for the Buttons -------------------------

    /*
      not used in the moment !
    class firstAction extends AbstractAction {

	public firstAction() {
	    super("erstes Projekt", firstIcon);
	}

	public void actionPerformed(ActionEvent evt) {
	    m_history.loadProject(0);
	    m_table.setRowSelectionInterval(0, 0);
	}

    }*/

    /**
     * The AbstractAction class for the "previous project" item. Reads out
     * the position of the selected item and loads the previous project using
     * History.LoadProject(int).
     *
     * @see visalg.basics.History#loadProject(int)
     */
    class previousAction extends AbstractAction {

	public previousAction() {
	    super("Load previous project", firstIcon);
	}

	public void actionPerformed(ActionEvent evt) {
	    int i = m_table.getSelectedRow();
	    m_history.loadProject(i-1);
	    m_table.setRowSelectionInterval(i-1, i-1);
	}

    }//class previous Action

    /**
     * The AbstractAction class representing the "undo" item. The call is simply
     * passed to the History by calling History.undo().
     *
     * @see visalg.basics.History#undo()
     */
    class undoAction extends AbstractAction {

	public undoAction() {
	    super("undo", undoIcon);
	}

	public void actionPerformed(ActionEvent evt) {
	    m_history.undo();
	}
    }//class undoAction

    /**
     * The AbstractAction class representing the "step" item. The event is
     * simply passed to the History by calling History.step().
     *
     * @see visalg.basics.History#step()
     */
    class stepAction extends AbstractAction {

	public stepAction() {
	    super("Step project", stepIcon);
	}

	public void actionPerformed(ActionEvent evt) {
	    m_history.step();
	}
    }//class stepAction

    /**
     * This class implements the action, which is performed, if the play
     * button is pressed by the user. It stores a reference to the play icon
     * which is read out by the IconButton class and implements its
     * actionPerformed method. If the latter one is repeatedly pressed,
     * it will cause the thread to activate resp. inactivate itself. Futhermore,
     * the current state of the thread is stored in the history. This history
     * member variable is used for activity detection of the thread, not the
     * internal thread field member. Thus, the current activity state of the
     * history can be read out by using the method isStepping().
     * @see History#isStepping()
     * @see visalg.basics.StepThread
     */
    class changePlayAction extends AbstractAction {

        /**
         * Calls the superclass constructor and passes a string, describing
         * the button function, and the initial icon reference to the play
         * icon.
         */
    	public changePlayAction() {
	        super("Durchlauf start/stop", nextIcon);
	    }

        /**
        * This method starts a new stepping thread which repeatedly invokes
        * a global step event until it is pressed again. Furthermore, the ico
        * will be changed to start/stop icon depending on the current state
        * of the history.
        */

	    public void actionPerformed(ActionEvent evt) {

                if (m_history.isStepping()==true) {
                    m_history.setStepping(false);
                    // sets Icon of the event causing button
                    ((JButton)evt.getSource()).setIcon(nextIcon);
                    ((JButton)evt.getSource()).setToolTipText("Start stepping");
                } else {
                    // sets Icon of the event causing button
                    ((JButton)evt.getSource()).setIcon(pauseIcon);
                    ((JButton)evt.getSource()).setToolTipText("Stop stepping");
                    m_history.setStepping(true);
                }
	    }//actionPerformed
    }//class nextAction

    /**
     * The AbstractAction class representing the "next Project" item. It will
     * read out the currently selected item and load the next one, using
     * History.loadProject(int).
     *
     * @see visalg.basics.History#loadProject(int)
     */
    class nextAction extends AbstractAction {

    	public nextAction() {
	        super("Load next project state", lastIcon);
	    }

	    public void actionPerformed(ActionEvent evt) {
            int i = m_table.getSelectedRow();
            i += 1;
            m_history.loadProject(i);
    	    m_table.setRowSelectionInterval(i, i);
	    }//actionPerformed
    }//class nextAction

   /*
    currently not used !
   class lastAction extends AbstractAction {

	public lastAction() {
	    super("letztes Projekt", lastIcon);
	}

	public void actionPerformed(ActionEvent evt) {
	    int i = m_history.getHistorySize() - 1;
	    m_history.loadProject(i);
	    m_table.setRowSelectionInterval(i, i);
	}

    }*/

    /**
     * The AbstractAction representing the "load project" item. It reads out
     * the currently selected project and tries to load it using
     * History.loadProject(int). If this was not successfull, an error dialog
     * is shown.
     *
     * @see visalg.basics.History#loadProject(int)
     */
    class loadAction extends AbstractAction {

	public loadAction() {
	    super("Load Project state", loadIcon);
	}//loadAction

	public void actionPerformed(ActionEvent evt) {
	    if (!(m_history.loadProject(m_table.getSelectedRow())))
		JOptionPane.showMessageDialog(null,
					      "Fehler beim Wiederherstellen des Projekts!",
					      "Projekt laden...",
					      JOptionPane.ERROR_MESSAGE);
	}//actionPerformed(ActionEvent)
    }//class loadAction

    /**
     * The AbstractAction class representing the "save project" item. It tries
     * to save the current project using History.saveProject() and shows an
     * error dialog box on failure.
     *
     * @see visalg.basics.History#saveProject()
     */
    class saveAction extends AbstractAction {

	public saveAction() {
	    super("Save Project state", saveIcon);
	}//saveAction()

	public void actionPerformed(ActionEvent evt) {
	    if (!(m_history.saveProject()))
		JOptionPane.showMessageDialog(null,
					      "Fehler beim Speichern des Projekts!",
					      "Projekt speichern...",
					      JOptionPane.ERROR_MESSAGE);
	    m_table.invalidate();
	    revalidate();
	}//actionPerformed(ActionEvent)
    }//class saveAction

    /**
     * The AbstractAction class representing the "delete project" item. It reads
     * out the currently selected item, and calls History.deleteProject(int) in
     * order to remove this project from the table.
     *
     * @see visalg.basics.History#deleteProject(int)
     */
    class deleteAction extends AbstractAction {

	public deleteAction() {
	    super("Delete selected project", deleteIcon);
	}//deleteAction()

	public void actionPerformed(ActionEvent evt) {
	    m_history.deleteProject(m_table.getSelectedRow());
	    m_table.invalidate();
	    revalidate();
	}//actionPerformed(ActionEvent)
    }//class deleteAction
} // class HistoryWindow

