////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ModuleWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Thu Sep 28 12:15:11 2000
//  Update count     : 10
//  CVS Info         : $Id: ModuleWindow.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  26.02.01    vigelius        enhanced documentation
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//  08.03.01    vigelius        declared abstract in order to force ModuleWindows
//                              to overwrite ModuleChanged()
//  04.04.01    vigelius    added method onClose()
//  27.06.01    vigelius    adapted to the new Menu Concept
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import visalg.basics.Module;
import visalg.basics.ModuleListener;
import visalg.types.ModuleChangeEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;
import java.util.*;

/**
 * The base class, where every ModuleWindow has to be derived from. It provides
 * a constructor, that registers the window at the ModuleWindowContainer and
 * a reference to the associated Module as well as a String, containing the
 * Modules name.
 *
 * @see visalg.basics.Module
 * @see visalg.graphics.ModuleWindowContainer
 */
abstract public class ModuleWindow extends JPanel implements ModuleListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * The ContentPane for this window. This will be initialised with the
     * ModuleWindow itself and is just for comfort.
     */
    protected transient Container m_contentPane;

    /**
     * Reference to the associated module.
     */
    private transient Module m_module;

    /**
     * Name of the ModuleWindow. This is the window title, as well.
     */
    private transient String m_name;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Initialises the window. All internal references are stored, and the
     * LayoutManager is set to BorderLayout.
     * @param s Name of the ModuleWindow (appears as title)
     * @param m The associated Module
     * @param bounds Initial window bounds
     * @param mwc The associated ModuleWindowContainer
    */
    public ModuleWindow(String s,
			Module m,
			Rectangle bounds,
			ModuleWindowContainer mwc) {
	super(new BorderLayout());
	m_contentPane = this;
	m_name = s;
	m_module = m;

    // Modulfenster an den ModuleWindowContainer, der fr die Anzeige sorgt...
	mwc.addModuleWindow(this, bounds);
   }//ModuleWindow(String, Module, Rectangle, ModuleWindowContainer)

    /**
     * Returns the associated module.
     * @return module The module
     */
    public Module getModule() {
	return m_module;
    }//getModule()

    /**
     * Activates the internal frame, this module window belongs to. This method
     * assumes its parent component to be a JInternalFrame object. If this is
     * not the case for some reason, this method needs to be overwritten.
     */
    public void activate() {
        JInternalFrame internalFrame = (JInternalFrame) getParent();
        try {
            internalFrame.setSelected(true);
        }
        catch (PropertyVetoException ex) {

        }
    }
    /**
     * Returns the name of the ModuleWindow.
     * @return name The name of the window
     */
    public String getName() {
	return m_name;
    }//getName()

    /**
     * Will be called by the ModuleWindowContainer, if the window is about
     * to be closed.
     */
    public void onClose() {
    }

    /**
     * Will be called by the ModuleWindowContainer, if the window gets the focus.
     */
    public void onFocusGained() {
    }

    /**
     * Will be called by the ModuleWindowContainer, if the window looses the
     * focus.
     */
    public void onFocusLost() {
    }

    // ----------- from Interface ModuleListener ------------------


    /**
     * This method will be called by the Module, if a change occurred. It should
     * be overwritten in order to react on changes. If it changes any window
     * components (for example by painting), revalidate() should be called at
     * the end to notify the window manager.
     * @param cEvt contains no information
     */
    abstract public void moduleChanged(ModuleChangeEvent cEvt);
} // class ModuleWindow

