////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ProjectWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 11:55:27 2000
//  Update count     : 14
//  CVS Info         : $Id: ProjectWindow.java,v 1.1.1.2 2002/03/18 12:42:41 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  27.02.01    vigelius        enhanced documentation
//  07.03.01    vigelius        made deletemodule button available via menu and
//                              added a confirmation
//  07.03.01    vigelius        added ToolTips
//  08.03.01    vigelius        added popupmenu and improved the mouse handling
//  27.06.01    vigelius        module windows are closed on module deletion
//
///////////////////////////////////////////////////////////////////////////////


package visalg.graphics;


import visalg.basics.ProjectListener;
import visalg.basics.*;
import visalg.graphics.*;
import visalg.components.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.JPopupMenu;

/**
 * This class represents the user-side part of the Project. It provides items
 * to remove,swap and activate or deactivat modules as well a tree,
 * containing all loaded modules.
 *
 * @see visalg.basics.Module
 * @see visalg.basics.Project
 * @todo Provide a button for the user to reset an algorithm module
 * @todo Every button should have an associated menu in the menubar
 */
public class ProjectWindow extends JPanel implements ProjectListener {

    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the Project (Model).
     */
    private transient Project m_project;

    /**
     * The icons for the toolbar.
     */
    private static ImageIcon s_deleteIcon = new ImageIcon("images/delete.gif");
    private static ImageIcon s_onIcon = new ImageIcon("images/on.gif");
    private static ImageIcon s_offIcon = new ImageIcon("images/off.gif");
    private static ImageIcon s_deactivatedIcon = new ImageIcon("images/deactivated.gif");
    private static ImageIcon s_upIcon = new ImageIcon("images/up.gif");
    private static ImageIcon s_downIcon = new ImageIcon("images/down.gif");

    /**
     * A scollPane for the module tree.
     */
    private transient JScrollPane m_scrollPane;

    /**
     * The Module tree.
     */
    private transient JTree m_tree;

    /**
     * The TreeModel for the Tree.
     */
    private transient TreeModel m_treeModel;

    /**
     * The TreeCellRenderer for the Tree.
     */
    private transient TreeCellRenderer m_renderer;

    /**
     * The project Tree PopUp Menu.
     */
    private transient JPopupMenu m_projectTreePopupMenu;
    private transient JMenuItem m_pTreeModuleAddItem;
    private transient JMenuItem m_pTreeModuleDelItem;
    private transient JMenuItem m_pTreeModuleToggleItem;
    private transient JMenuItem m_pTreeModuleUpItem;
    private transient JMenuItem m_pTreeModuleDownItem;

    /**
     * The module menu.
     */
    private transient JMenu m_moduleMenu=null;

     /**
      * The add module menu item (module menu)
      */
    private transient JMenuItem m_moduleAddModuleMenuItem;

    /**
     * The remove module menu item (module menu)
     */
    private transient JMenuItem m_moduleRemoveModuleMenuItem;

    /**
     * The toggle module menu item (module menu)
     */
    private transient JMenuItem m_moduleToggleModuleMenuItem;

    /**
     * The "Move module up" menu item (module menu)
     */
    private transient JMenuItem m_moduleModuleUpMenuItem;

    /**
     * The "Move module down" menu item (module menu)
     */
    private transient JMenuItem m_moduleModuleDownMenuItem;

    /**
     * The action class for the "add module" item.
     */
     private transient Action m_addModuleAction =
        new AbstractAction("Add module ...") {

            public void actionPerformed(ActionEvent evt) {
                AddModuleDialog md = new AddModuleDialog(m_project);
            }
        };

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * The constructor sets up the ModuleTree, including the TreeModel, a
     * TreeCellRenderer and an ActionListener as well as the buttons. The Tree
     * shows all loaded modules using the Project.getModules() and the
     * Module.getSubModules() methods.
     * If the user doubleclicks on an item of the Tree, the associated
     * window will be opened using Module.newModuleWindow().
     *
     * @param p The associated Project
     * @param pwc The associated ProjectWindowContainer
     *
     * @see visalg.basics.Project#getModules()
     * @see visalg.basics.Module#getSubModules()
     * @see visalg.basics.Module#newModuleWindow()
     *
     * @todo Check, if a ModuleWindow has already been opened
     */
    public ProjectWindow(Project p, ProjectWindowContainer pwc) {

   // We take the BorderLayout
	super(new BorderLayout());

 	m_project = p;

	// Create Tree Model (by implementing the interface)
	m_treeModel = new TreeModel() {

		public Object getChild(Object parent, int index) {

		    Vector v = null;

		    // the parent is the root node, i.e. the project itself
		    if (parent == m_project) {
			v = m_project.getModules();
		    }

		    // the parent is a module
		    if (parent instanceof Module) {
			v = ((Module)parent).getSubModules();
		    }

		    //Wenn keine Werte oder fehlerhafter Zugriff: null
		    if ( (v == null) || (index < 0) ||(index >= v.size()) ) {
			return null;
		    }

		    return v.get(index);
		}

		public int getChildCount(Object parent) {

		    Vector v = null;

		    if (parent == m_project) {
			v = m_project.getModules();
		    }

		    if (parent instanceof Module) {
			v = ((Module)parent).getSubModules();
		    }

		    if (v == null) {
			return 0;
		    }

		    return v.size();
		}

		public int getIndexOfChild(Object parent, Object child) {

		    Vector v = null;

		    if (parent == m_project) {
			v = m_project.getModules();
		    }

		    if (parent instanceof Module) {
			v = ((Module)parent).getSubModules();
		    }

		    if (v == null) {
			return -1;
		    }

		    return v.indexOf(child);
		}

		public Object getRoot() {
		    return m_project;
		}

		public boolean isLeaf(Object node) {

		    if (node == m_project) {
			return false;
		    }

		    if ( (node instanceof Module) &&
			 (((Module)node).getSubModules()!=null)) {
		        return false;
		    }

		    return true;
		}

		public void valueForPathChanged(TreePath path, Object newValue) {

		    // Traverse all registered listeners
		    for (Enumeration e = m_listeners.elements() ; e.hasMoreElements() ;) {

			// Get array of objects which specify the path
			Object[] changePath;
			changePath = path.getPath();

			// ??? Warum wird fuer einen laengeren Path die letzte
			// Komponente abgeschnitten?
			if (changePath.length > 1) {
			    changePath = path.getParentPath().getPath();
			}

			TreeModelListener tml = (TreeModelListener)(e.nextElement());
			tml.treeStructureChanged(new TreeModelEvent(this, changePath));
		    }

		}

		private Vector m_listeners;

		public void addTreeModelListener(TreeModelListener l) {

		    if (m_listeners == null) {
			m_listeners = new Vector();
		    }

		    m_listeners.add(l);

		}

		public void removeTreeModelListener(TreeModelListener l) {
		    if (m_listeners == null) {
			return;
		    }

		    m_listeners.remove(l);
		}

	    }; // END OF Create Tree Model (by implementing the interface)


	// Create a DefaultTreeCellRenderer (by implicit derivation of the class)
	m_renderer = new DefaultTreeCellRenderer() {

		class IconWrapper implements Icon {
		    private Icon icon1;
		    private Icon icon2;

		    public IconWrapper(Icon i1, Icon i2) {
			icon1 = i1;
			icon2 = i2;
		    }

		    public int getIconHeight() {
			//Returns the icon's height.
			try {
			    return Math.max(icon1.getIconHeight(), icon2.getIconHeight());
			} catch (Exception e) {
			    return 0;
			}
		    }

		    public int getIconWidth() {
			//Returns the icon's width.
			try {
			    return icon1.getIconWidth() + icon2.getIconWidth() + 10;
			} catch (Exception e) {
			    return 0;
			}
		    }

		    public void paintIcon(Component c, Graphics g, int x, int y) {
			//Draw the icon at the specified location.
			icon1.paintIcon(c, g, x, y);
			icon2.paintIcon(c, g, x + icon2.getIconWidth(), y);
		    }

		} // class IconWrapper

		public Component getTreeCellRendererComponent(JTree tree,
							      Object value,
							      boolean selected,
							      boolean expanded,
							      boolean leaf,
							      int row,
							      boolean hasFocus) {

		    // Value is a Module
		    if ( !(value instanceof Project) && (value instanceof Module) ) {

			Module m = (Module)value;

			//Icons suchen... i1: vom Module; i2: Glhbirne je nach Zustand
			Icon i1 = m.getNodeIcon();
			Icon i2;

			if (m.isStepping()) {
			    i2 = s_onIcon;
			}
			else {
			    i2 = s_offIcon;
			    if ( (m instanceof Algorithm) && ( !((Algorithm)m).isDone())) {
				i2 = s_deactivatedIcon;
			    }
			}

			//Icons zusammensetzen (sofern vorhanden)
			Icon icon = null;
			if (i1 != null) {
			    if (i2 != null) {
				icon = new IconWrapper(i1, i2);
			    }
			    else {
				icon = i1;
			    }
			}
			else {
			    if (i2 != null) {
				icon = i2;
			    }
			}

			//...und bergeben...
			if (icon != null) {
			    setLeafIcon(icon);
			    setClosedIcon(icon);
			    setOpenIcon(icon);
			}

		    } // Value is a Module

		    else {
			// hier sollten noch eigene Icons eingefgt werden
			// (Ordner auf/zu) fr Projekt bsp.sweise
			setLeafIcon(getDefaultLeafIcon());
			setClosedIcon(getDefaultClosedIcon());
			setOpenIcon(getDefaultOpenIcon());
		    }

		    return super.getTreeCellRendererComponent(tree, value, selected,
							      expanded, leaf, row, hasFocus);
		}

	    }; // END OF Create a DefaultTreeCellRenderer (by implicit
	       // derivation of the class)

	// Baum zusammensetzen.
	m_tree = new JTree(m_treeModel);
        ((DefaultTreeCellRenderer)m_renderer).setPreferredSize(new Dimension(200,15));
	m_tree.setCellRenderer(m_renderer);

        // Popup menu for project tree
        m_projectTreePopupMenu = new JPopupMenu("Project");
        m_pTreeModuleAddItem = m_projectTreePopupMenu.add(m_addModuleAction);
        m_pTreeModuleDelItem = m_projectTreePopupMenu.add(new deleteAction());
        m_projectTreePopupMenu.addSeparator();
        m_pTreeModuleToggleItem = m_projectTreePopupMenu.add(new onOffAction());
        m_projectTreePopupMenu.addSeparator();
        m_pTreeModuleUpItem = m_projectTreePopupMenu.add(new upAction());
        m_pTreeModuleDownItem = m_projectTreePopupMenu.add(new downAction());
        m_projectTreePopupMenu.addPopupMenuListener(new projectPopupMenuListener());
        m_tree.add(m_projectTreePopupMenu);

	// react on mouse events
	m_tree.addMouseListener(new MouseAdapter() {

		public void mouseClicked(MouseEvent e) {

		    // Get information where the click took place
		    int selRow = m_tree.getRowForLocation(e.getX(), e.getY());
		    TreePath selPath = m_tree.getPathForLocation(e.getX(), e.getY());

		    // handling left clicks
                    if (e.getModifiers()==InputEvent.BUTTON1_MASK) {
                        if (selRow != -1) {

                            if (e.getClickCount() == 1) {
                            }
			    else {
			        if (e.getClickCount() == 2) {
                                    //myDoubleClick(selRow, selPath);
    				    Object o = selPath.getLastPathComponent();

				    // Open new module window
				    if (o instanceof Module) {
                        // Check if window has already been opened
                        if (!(((Module)o).getModuleWindow() instanceof ModuleWindow)) {
                            ((Module)o).newModuleWindow();
                        }
				    }
			        }//doubleclick
			    }//singleclick
                        }//selRow!=-1
		    }//Button1
                    else if (e.getModifiers()==InputEvent.BUTTON3_MASK)
                    // handling right clicks
                    {
                        // when nothing or just one row
                        // is currently selected, select the row
                        if (m_tree.getSelectionCount()<2)
                        {
                            m_tree.setSelectionRow(selRow);
                        }
                        m_projectTreePopupMenu.show(m_tree, e.getX(), e.getY());
                    }
		}
	    } // Creation of MouseAdapter
        );

	// und schlielich die Scrollleiste.
	m_scrollPane = new JScrollPane(m_tree);
	add(m_scrollPane, BorderLayout.CENTER);

	// NORTH: (Buttons...)
	JButton deleteButton = new IconButton(new deleteAction());
        deleteButton.setToolTipText("Delete selected module(s)");
	JButton onButton = new IconButton(new onOffAction());
        onButton.setToolTipText("Toogle module active/inactive");
	JButton upButton = new IconButton(new upAction());
        upButton.setToolTipText("Move selected module up");
	JButton downButton = new IconButton(new downAction());
        downButton.setToolTipText("Move selected module down");

	//(... in ein Panel...)
	JPanel North = new JPanel();
	North.add(deleteButton);
	North.add(onButton);
	North.add(upButton);
	North.add(downButton);

	//(Schaltflchenpanel nach oben)
	add(North, BorderLayout.NORTH);

	//Projektfenster anzeigen...
	pwc.setProjectWindow(this);

        // add module menu to the application menu if none exists yet

        m_moduleMenu = new JMenu("Project");
        m_moduleMenu.setMnemonic('P');
        m_moduleAddModuleMenuItem = m_moduleMenu.add(m_addModuleAction);
        m_moduleRemoveModuleMenuItem = m_moduleMenu.add(new deleteAction());
        m_moduleMenu.addSeparator();
        m_moduleToggleModuleMenuItem = m_moduleMenu.add(new onOffAction());
        m_moduleMenu.addSeparator();
        m_moduleModuleUpMenuItem = m_moduleMenu.add(new upAction());
        m_moduleModuleDownMenuItem = m_moduleMenu.add(new downAction());
        m_moduleMenu.getPopupMenu().addPopupMenuListener(new projectPopupMenuListener());
        m_project.getMenuBarContainer().addMenu(m_moduleMenu);
    }// ProjectWindow(Project p, ProjectWindowContainer pwc)


    // -------- from interface ProjectListener -------------

    /**
     * Called, if the Project has changed. The project tree will be repainted.
     *
     * @param cEvt Contains no information
     */
    public void projectChanged(ChangeEvent cEvt) {
	Object[] o = new Object[1];
	o[0] = m_project;
	m_treeModel.valueForPathChanged(new TreePath(o), null);
	this.repaint();
    }//projectChanged(ChangeEvent)

    // ----------------- actions for items/menus -------------------

    /**
     * Looks for the currently selected module and deletes it from the list.
     */
    public void deleteSelectedModule()
    {
        TreePath[] modulesToBeDeleted = m_tree.getSelectionPaths();
        if (modulesToBeDeleted == null) {
            return;
        }

        if (new JOptionPane().
            showConfirmDialog(null,
                              "Really remove this module ?",
                              "Remove Module ...",
                              JOptionPane.YES_NO_OPTION)==
                              JOptionPane.NO_OPTION)
        {
            return;
        }

        for (int i = modulesToBeDeleted.length - 1; i >= 0; i--) { //von hinten

            Object o = modulesToBeDeleted[i].getLastPathComponent();

            //root schtzen!
            if (o instanceof Project) {
                return;
            }

            // Get parent (??? Geht das nicht unkomplizierter?)
            Object oParent =
                modulesToBeDeleted[i].getPathComponent(modulesToBeDeleted[i].getPathCount() - 2);

            // Fenster lschen
            ((Module)o).discardWindows();

            try {
                ((ModuleManager)oParent).removeModule((Module)o);
                m_treeModel.valueForPathChanged(modulesToBeDeleted[i], null);
            } catch (Exception e) {
                System.err.println(e);
                System.err.print("ProjectWindow.java: Fehler beim Lschen von Modulen: ");
                System.err.println("Hierarchie von ModuleManagern und Modules stimmt nicht!");
            }

        } // for
    }//deleteSelectedModule()

    /**
     * Toggles the state of the currently selected module on/off.
     */
    private void toggleSelectedModuleOnOff() {
        TreePath[] modulesToStep = m_tree.getSelectionPaths();

        if (modulesToStep == null) {
            return;
        }

        for (int i = modulesToStep.length - 1; i >= 0; i--) { //von hinten

            Object o = modulesToStep[i].getLastPathComponent();

            //root schtzen!
            if (o instanceof Project) {
                return;
            }

            try {

            // Negate  stepping value in order to switch on/off
            boolean newSteppingValue = !(((Module)o).isStepping());

            ((Module)o).setStepping(newSteppingValue);
            m_treeModel.valueForPathChanged(modulesToStep[i], o);

            } catch (Exception e) {
                System.err.println(e);
                System.err.print("ProjectWindow.java: Fehler bei Modul an/aus:");
                System.err.println("Hierarchie von Modules stimmt nicht!");
            }
        } // for
     }//toggleSelectedModuleOnOff

    /**
     * Moves the selected module one position up.
     */
     private void moduleUp()
     {
        TreePath[] modulesToOrderUpwards = m_tree.getSelectionPaths();

        if (modulesToOrderUpwards==null) {
            return;
        }

        for (int i = 0; i < modulesToOrderUpwards.length; i++) {

            Object o = modulesToOrderUpwards[i].getLastPathComponent();

            //root schtzen!
            if (!(o instanceof Project)) {

                try {
                    ModuleManager mm = ((Module)o).getModuleManager();
                    Vector v = mm.getModules();
                    int index = 0;
                    for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
                        if (o == e.nextElement()) break;
                        index++;
                    }
                    mm.swapModules(index-1, index);
                    m_treeModel.valueForPathChanged(modulesToOrderUpwards[i].getParentPath(), mm);
                } catch (Exception e) {
                    System.err.println(e);
                    System.err.print("Fehler bei Modul aufwrts: ");
                    System.err.println("Hierarchie von Modules stimmt nicht!");
                }
            }
        } // for
     }//moduleUp()

    /**
     * Swaps the selected Module with the one below.
     */
    private void moduleDown() {

        TreePath[] modulesToOrderDownwards = m_tree.getSelectionPaths();

        if (modulesToOrderDownwards==null) {
            return;
        }

        for (int i = 0; i < modulesToOrderDownwards.length; i++) {

            Object o = modulesToOrderDownwards[i].getLastPathComponent();

            //root schtzen!
            if ( !(o instanceof Project)) {
                try {
                    ModuleManager mm = ((Module)o).getModuleManager();
                    Vector v = mm.getModules();
                    int index = 0;
                    for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
                        if (o == e.nextElement()) break;
                        index++;
                    }
                    mm.swapModules(index, index+1);
                    m_treeModel.valueForPathChanged(modulesToOrderDownwards[i].getParentPath(), mm);
                } catch (Exception e) {
                    System.err.println(e);
                    System.err.print("Fehler bei Modul aufwrts: ");
                    System.err.println("Hierarchie von Modules stimmt nicht!");
                }
            }
        } // for
    }//moduleDown

    /**
     * Unloads the menu.
     */
    public void unloadMenu() {
        if (m_moduleMenu!=null) {
            m_project.getMenuBarContainer().removeMenu(m_moduleMenu);
        }
    }//unloadMenu()
    // -----------------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------------
    // -----------------------------------------------------

    /**
     * The AbstractAction class representing the "delete module" button.
     * It deletes all selected modules and the associated submodules by
     * calling ModuleManager.removeModule(Module).
     *
     * @see ModuleManager#removeModule(Module)
     */
    class deleteAction extends AbstractAction {

	public deleteAction() {
	    super("Delete module ...", s_deleteIcon);
	}

	public void actionPerformed(ActionEvent evt) {
            deleteSelectedModule();
	}
    } //class deleteAction

    /**
     * The AbstractAction representing the "Module on/off" button. The selected
     * Module will toggle its stepping state.
     *
     * @see visalg.basics.Module#isStepping()
     * @see visalg.basics.Module#setStepping(boolean)
     */
    class onOffAction extends AbstractAction {

	public onOffAction() {
	    super("Toggle module on/off", s_onIcon);
	}

	public void actionPerformed(ActionEvent evt) {
            toggleSelectedModuleOnOff();
	}
    } //class onOffAction

    /**
     * The AbstractAction class representing the "Module up" button. The
     * selected Module swaps its position with the one above.
     */
    class upAction extends AbstractAction {

	public upAction() {
	    super("Module up", s_upIcon);
	}

	public void actionPerformed(ActionEvent evt) {
            moduleUp();
	}
    } // class upAction

    /**
     * The AbstractAction class representing the "Module down" button. The
     * selected Module swaps its position with the one below.
     */
    class downAction extends AbstractAction {

	public downAction() {
	    super("Module down", s_downIcon);
	}

	public void actionPerformed(ActionEvent evt) {
            moduleDown();
	}
    } // class downAction

    /**
     * This class implements the listener for the project window popup menu.
     * It checks, if at least one module is selected and then sets the
     * menuitems to enabled or disabled. This listener will be called also, if
     * the "Project" menu of the main menubar pops up.
    */
     class projectPopupMenuListener implements PopupMenuListener {

        public void popupMenuCanceled(PopupMenuEvent e) {}
        public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {}

        public void popupMenuWillBecomeVisible(PopupMenuEvent e) {

        boolean active=false;

            if (m_tree.getSelectionCount()>0) {
                active = true;  // set enabled, if at least one module is sel.
            }

            // set menu items

            m_moduleModuleDownMenuItem.setEnabled(active);
            m_moduleModuleUpMenuItem.setEnabled(active);
            m_moduleRemoveModuleMenuItem.setEnabled(active);
            m_moduleToggleModuleMenuItem.setEnabled(active);

            m_pTreeModuleDelItem.setEnabled(active);
            m_pTreeModuleDownItem.setEnabled(active);
            m_pTreeModuleUpItem.setEnabled(active);
            m_pTreeModuleToggleItem.setEnabled(active);
        }
     }
} // class ProjectWindow