////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : ArrayViewer.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Wed May 16 15:10:00 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
//  29.10.01    vigelius    included new DataType VisAlgIntArray
///////////////////////////////////////////////////////////////////////////////

package visalg.modules;

import visalg.graphics.*;

import visalg.types.*;

import visalg.basics.DataModule;

import java.awt.*;
import javax.swing.event.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * This class is the frontend to the ArrayViewer module. It visualises data,
 * that is hold in an array including additional indicator bars, that are
 * represented by the property IndicatorBarProperty.
 *
 * @see visalg.modules.ArrayViewer
 * @see visalg.types.IndicatorBarProperty
 */
public class ArrayViewerWindow extends ModuleWindow {

    /**
     * Reference to the ArrayViewer.
     */
    private ArrayViewer m_arrayViewer;

    /**
     * The dimension of the window.
     */
    private transient Dimension m_dimension;

    /**
     * @serial Offset for scaling
     */
	private static int s_scaleOffset = 30;       //links

    /**
     * @serial Offset for borde
     */
	private static int s_borderOffset = 10;      //oben und unten

    /**
     * @serial Window font
     */
	private static Font s_font = new Font("SansSerif", Font.PLAIN, 10);

    /**
     * Metrics of the window font
     */
	private transient FontMetrics m_fontMetrics;

    /**
     * Standard window color
     */
	private transient Color m_standardColor;

    /**
     * @serial Color for value bar
     */
	private static Color s_barColor = new Color(0, 128, 0);       //dunkelgrn

    /**
     * Height of the window panel
     */
	private transient int m_panelHeight;

    /**
     * Width of the window panel
     */
	private transient int m_panelWidth;

    /**
     * Maximum data value
     */
	private transient int m_maxOfData;

    /**
     * Initialisation.
     * Calls the superconstructor and sets the viewer reference and the
     * panel size.
     * @param name Name of the window
     * @param av Reference to the array viewer module
     * @param bounds Initial bounds of the window
     * @param mwc Reference to the window container
     */
    public ArrayViewerWindow(String name, ArrayViewer av, Rectangle bounds,
            ModuleWindowContainer mwc) {

        super(name, av, bounds, mwc);
        m_arrayViewer = av;
        m_dimension = getSize(new Dimension());
    }//ArrayViewerWindow(String, ArrayViewer, Rectangle, ModuleWindowContainer)

    /**
     * Paint method.
     * This method is called by the GUI if a repainting of the window is
     * necessary. It will read out the properties of the algorithm and do the
     * representation.
     * @param g Graphics object to paint in
     */
    public void paintComponent(Graphics g) {
		super.paintComponent(g);
		m_standardColor = g.getColor();
		g.setFont(s_font);
		m_fontMetrics = g.getFontMetrics(s_font);
		m_dimension = getSize(m_dimension);

		m_panelHeight = m_dimension.height;
		m_panelWidth = m_dimension.width;

        Object o = m_arrayViewer.getDataModule().getData();

        // create an array of VisAlgInt
        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();

        int totalNumberOfData = data.length;

		m_maxOfData = 0;

		for (int i=0; i < data.length; i++) {
			if ( (data[i]!=null)
                  && (data[i].compareToZero() > m_maxOfData)) {
                m_maxOfData = data[i].compareToZero();
            }
		}

		//Achse beschriften...
		for (int i=0; i < data.length; i++) {
			data[i].paintComponent(g, 5, getY(i,totalNumberOfData)+5);
		}
		//Flche nach rechts lschen und die Achse zeichnen...
		g.clearRect(s_scaleOffset, 0, m_panelWidth - s_scaleOffset,
                    m_panelHeight);
		g.drawLine(s_scaleOffset, 0, s_scaleOffset, m_panelHeight);
		//endlich: 3D-Balken zeichnen... (erhht)
		g.setColor(s_barColor);
		for (int i=0; i < data.length; i++) {
			g.fill3DRect(s_scaleOffset, getY(i,totalNumberOfData),
                         getX(data[i].compareToZero()), 3, true);
		}

        // Paint Indicators

        DataModule dm = ((ArrayViewer)getModule()).getDataModule();

        Vector ind =
            dm.queryPropertyObjects(IndicatorBarProperty.INDICATOR_BAR_PROPERTY);

        int currPos = 0;
        int relPos = 0;

        int y;

        for (Enumeration e = ind.elements(); e.hasMoreElements();) {
            IndicatorBarProperty prop = (IndicatorBarProperty) e.nextElement();
            currPos = prop.getIndicatorPosition();
            relPos = prop.getRelativePosition();

            if (currPos != -1) {
                g.setColor(prop.getIndicatorColor());
                y = getY(currPos, totalNumberOfData) + (relPos*3);
		        g.fill3DRect(s_scaleOffset, y, m_panelWidth - s_scaleOffset, 3, true);
		        g.drawString(prop.getIndicatorName(), s_scaleOffset + 5, y + 10);
            }
        }
	}//paintComponent

    /**
     * Gives the width of a bar depending on its values
     * @param value Value to be representated.
     */
	private int getX(int value) {
		float f = (float)(m_panelWidth - s_scaleOffset) * value;
		//return s_scaleOffset + Math.round(f/m_maxOfData);
		return Math.round(f/m_maxOfData);
	}//getX

    /**
     * Calculates the y position.
     * The y position depends on the total number of data and the relative
     * position of the data to be drawn.
     * @param numberOfData Relative position of this value
     * @param totalNumberOfData Total amount of data
     */
	private int getY(int numberOfData, int totalNumberOfData) {
		float f = (float)(m_panelHeight
                - s_borderOffset - s_borderOffset) * numberOfData;
		return s_borderOffset + Math.round(f/totalNumberOfData);
	}//getY

    /**
     * Module has changed.
     * This method is called by the module manager if the data module has bee
     * changed. It just repaints itself then.
     */
	public void moduleChanged(ModuleChangeEvent cEvt) {
		repaint();
	}//moduleChanged
}//class ArrayViewerWindow