////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PrimitiveObserverWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 16:23:16 2000
//  Update count     : 2
//  CVS Info         : $Id: PrimitiveObserverWindow.java,v 1.1.1.2 2002/03/18 12:42:42 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;
import visalg.modules.PrimitiveObserver;


import visalg.graphics.ModuleWindow;
import visalg.graphics.ModuleWindowContainer;

import visalg.basics.Module;
import visalg.basics.ModuleListener;
import visalg.basics.Algorithm;
import visalg.types.ModuleChangeEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


/**
* This class represents the frontend to the PrimitiveObserver module. It just
* prints a list of all currently loaded modules with additional information.
* This list will be repainted, each time a step event occurred.
*
* @see PrimitiveObserver
*/
public class PrimitiveObserverWindow extends ModuleWindow {
    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Label, that contains the current project time.
     */
    private transient JLabel m_stepTesterLabel;

    /**
     * A list of all currently loaded modules.
     */
    private transient Vector m_modules;

    /**
     * Reference to the PrimitiveObserver module.
     */
    private transient PrimitiveObserver m_primitiveObserver;

    /**
     * The ScrollPane, that provides scroll functionality to the window.
     */
    private transient JScrollPane m_scrollPane;

    /**
     * A list item, containing and printing a list of all modules including
     * additional information.
     */
    private transient JList m_moduleList;

    /**
     * An enhanced list model, used for this class.
     */
    private transient MyListModel m_projectModules;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Sets up the main window.
     *
     * @param s The name of this ModuleWindow
     * @param po Reference to the PrimitiveObserver module
     * @param bounds The initial window bounds
     * @param mwc The ModuleWindowContainer
     */
    public PrimitiveObserverWindow(String s, PrimitiveObserver po,
				   Rectangle bounds, ModuleWindowContainer mwc) {
	super(s, po, bounds, mwc);

	//Initialisierung der (lokalen) Variablen
	m_primitiveObserver = po;
	m_stepTesterLabel = new JLabel("Time : " + m_primitiveObserver.getTime());
	m_contentPane.add(m_stepTesterLabel, BorderLayout.NORTH);

	//Liste
	m_projectModules = new MyListModel();
	m_moduleList = new JList(m_projectModules);
	m_moduleList.setCellRenderer(new MyCellRenderer());
	m_moduleList.setVisibleRowCount(5);

	//... und schlielich die Scrollleiste.
	m_scrollPane = new JScrollPane(m_moduleList);
	m_contentPane.add(m_scrollPane, BorderLayout.CENTER);

	revalidate();
    }//PrimitiveObserver(String, PrimitiveObserver, Rectangle, ModuleWindowContainer)


    // ----------- from Interface ModuleListener  ------------------

    /**
     * Just passes the change event to the ListModel by calling
     * MyListModel.fireListChanged(). The module time label will be updated.
     *
     * @see MyListModel#fireListChanged()
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
	m_stepTesterLabel.setText("Zeit: " + m_primitiveObserver.getTime());
	m_projectModules.fireListChanged();
	repaint();
    }//moduleChanged(ChangeEvent)


    // -----------------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------------
    // -----------------------------------------------------


    /**
     * The cell renderer. Each cell has three columns. The first one contains
     * an index, the second one the name of the module and the last one the
     * name of the last performed command (if the module is an algorithm) or
     * "no algorithm" else. The module name is requested using Module.toString()
     * and the last command by Algorithm.getLastCommand().
     *
     * @see Module#toString()
     * @see visalg.basics.Algorithm#getLastCommand()
     */
    private class MyCellRenderer extends JPanel implements ListCellRenderer {
	private JLabel indexLabel;
	private JLabel valueToStringLabel;
	private JLabel lastOperationLabel;

	public MyCellRenderer() {
	    super();
	    indexLabel = new JLabel("");
	    valueToStringLabel = new JLabel("");
	    lastOperationLabel = new JLabel("");
	    add(indexLabel);
	    add(valueToStringLabel);
	    add(lastOperationLabel);
	}

	public Component getListCellRendererComponent(JList list, Object value, int index,
						      boolean isSelected, boolean cellHasFocus) {
	    indexLabel.setText("" + index);
	    if (value instanceof Module) { //sollte immer so sein!
		Module module = (Module) value;
		valueToStringLabel.setText(value.toString());
		if (value instanceof Algorithm) {
		    Algorithm algo = (Algorithm) value;
		    lastOperationLabel.setText("last : " + algo.getLastCommand());
		} else {
		    lastOperationLabel.setText("(no algorithm)");
		}
	    } else {                       //darf nach Konstruktion nie auftauchen!
		valueToStringLabel.setText("(no module)");
		lastOperationLabel.setText(" ");
	    }
	    setBackground(isSelected ? Color.gray : Color.lightGray);
	    setForeground(Color.black);
	    invalidate();
	    return this;
	}
    }//class MyCellRenderer

    /**
     * The list model. Just provides simple swing functionality.
     */
    class MyListModel extends AbstractListModel {
	public int getSize() { return m_primitiveObserver.getModules().size(); }
	public Object getElementAt(int index) { return m_primitiveObserver.getModules().get(index); }
	public void fireListChanged() { fireContentsChanged(this, 0, getSize() - 1); }
    }//class MyListModel
} // class PrimitiveObserverWindow
