////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PrimitiveStatistics.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 15:06:02 2000
//  Update count     : 3
//  CVS Info         : $Id: PrimitiveStatistics.java,v 1.1.1.2 2002/03/18 12:42:42 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  02.03.01    vigelius        revised documentation
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//  15.03.01    vigelius    the user is now able to choose its algorithm
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.ModuleChangeEvent;

/**
 * PrimitiveStatistics looks for the first algorithm in the list of loaded
 * modules and visualises the number of performed commands of one type. In order
 * to get all available command classes for this algorithm, a getCommands() call
 * is performed. The frequency of command calls is represented by rectangles.
 *
 * @see visalg.basics.Algorithm#getCommands()
*/
public class PrimitiveStatistics extends AbstractModule implements ModuleListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the currently observed algorithm.
     * @serial
     */
    private Algorithm m_algorithm;

    /**
     * Array of all observed commands.
     * @serial
     */
    private Command[] m_commands;

    /**
     * Number of all performed commands.
     * @serial
     */
    private int[] m_commandCounter;

    /**
     * The maximum value of the frequencies.
     * @serial
     */
    private int m_maxValue;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Looks for the algorithm, creates a new array for the command call
     * frequencies and registers itself as a listener to the found module.
     * To look for the algorithm, the method findAlgorithm() is called.
     *
     * @see visalg.basics.Algorithm
     * @see PrimitiveStatistics#findAlgorithm()
     * @param mm The ModuleManager
     *
     */
    public PrimitiveStatistics(ModuleManager mm) {
	super("Primitive Statistics module", mm);

    m_algorithm = null;

    }//PrimitiveStatistics(ModuleManager)

    /**
     * Returns the name of the currently observed algorithm.
     *
     * @return The name of the observed algorithm
     */
    public String getAlgorithmName() {
	return m_algorithm.toString();
    }//getAlgorithmName

    /**
     * Returns the number of available commands for the observed algorithm.
     *
     * @return Number of commands
     */
    public int getNumberOfCommands() {
	return m_commands.length;
    }//getNumberOfCommands()

    /**
     * Returns a reference to the specified command.
     *
     * @param index The index of the command, that is requested
     * @return Reference to the command
     */
    public Command getCommand(int index) {
	if ( (index >= 0) && (index < m_commands.length) ) {
	    return m_commands[index];
	}

	return null;
    }

    /**
     * Returns the number of command calls of a specified command.
     *
     * @param index The index of the requested commands
     * @return The frequency of this command
     */
    public int getCommandCounter(int index) {
	if ( (index >= 0) && (index < m_commandCounter.length) ) {
	    return m_commandCounter[index];
	}

	return 0;
    }//getCommandCounter(int)

    /**
     * Returns the maximum frequency of all commands.
     *
     * @return The maximum frequency
     */
    public int getMax() {
	return m_maxValue;
    }//getMax()


    // ----------- from Interface Module  ------------------

    /**
     * Creates a new PrimitiveStatisticsWindow object.
     *
     * @see PrimitiveStatisticsWindow
     */
    public void newModuleWindow() {

	    try {
		m_moduleWindow =
		    new PrimitiveStatisticsWindow("Primitive Statistics module",
						  this,
						  m_moduleWindowBounds,
						  m_moduleManager.getModuleWindowContainer());
	    }
	    catch (Exception e) {
		System.err.println(e);
	    }
    }//newModuleWindow()

    /**
     * Does nothing.
     */
    public void step() {
    }//step()


    // ----------- from Interface ModuleListener  ------------------

    /**
     * This method will be called, if the observed module has been changed.
     * It reads out the last performed command using Algorithm.getLastCommand()
     * and increases the frequency counter for this command. At the end,
     * fireEvent() is called in order to notify the associated window object of
     * the change.
     *
     * @see visalg.basics.Algorithm#getLastCommand()
     * @see visalg.basics.AbstractModule#fireEvent()
     * @see PrimitiveStatisticsWindow
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {

	Command c = m_algorithm.getLastCommand();

	if (c == null) {
	    return;
	}

	for (int i = 0; i < m_commands.length; i++) {

	    if (m_commands[i].getClass().isInstance(c)) {

		m_commandCounter[i] += 1;

		if (m_maxValue < m_commandCounter[i]) {
		    m_maxValue = m_commandCounter[i];
		}

		fireEvent();
	    }
	}
    }//moduleChanged(ChangeEvent)

    public void setAlgorithm(Algorithm alg) {

        // remove old listeners
        if (m_algorithm!=null) {
            m_algorithm.removeModuleListener(this);
        }

        m_algorithm = alg;

        m_algorithm.addModuleListener(this);

	    //Initialisieren...
	    m_commands = m_algorithm.getCommands();
	    m_commandCounter = new int[m_commands.length];
	    for (int i = 0; i < m_commandCounter.length; i++) {
		m_commandCounter[i] = 0;
	    }
	    m_maxValue = 0;

        //fireEvent();
    }
    // -----------------------------------------------------
    // ------- M e t h o d s ( p r i v a t e ) -------------
    // -----------------------------------------------------

    /**
     * Looks for the first algorithm in the list of loaded modules of this
     * ModuleManager and stores it.
     */
    private void findAlgorithm() {

	m_algorithm = null;

	for (Enumeration e = m_moduleManager.getModules().elements() ; e.hasMoreElements() ;) {

	    Object o = e.nextElement();
	    if (o instanceof Algorithm) {
		m_algorithm = (Algorithm)o;
	    }

	    if (o == this) {
		return;
	    }

	}
    }//findAlgorithm
} // class PrimitiveStatistics
