////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PrimitiveStatisticsWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 16:07:23 2000
//  Update count     : 3
//  CVS Info         : $Id: PrimitiveStatisticsWindow.java,v 1.1.1.2 2002/03/18 12:42:42 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  02.03.01    vigelius        Revised documentation
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//  15.03.01    vigelius    the user is now able to choose its algorithm
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import visalg.basics.Module;
import visalg.basics.ModuleListener;
import visalg.basics.*;
import visalg.graphics.ModuleWindow;
import visalg.graphics.ModuleWindowContainer;
import visalg.types.ModuleChangeEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


/**
* This class is the frontend to the PrimitiveStatistics module. It paints the
* command call frequencies using a bar chart.
*
* @see PrimitiveStatistics
*/
public class PrimitiveStatisticsWindow extends ModuleWindow implements ActionListener{


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the PrimitiveStatistics module.
     */
    private transient PrimitiveStatistics m_primitiveStatistics;

    /**
     * Reference to the Charts object, which does the painting.
     */
    private transient Charts m_charts;

    /**
     * A JComboBox, which allows the user to choose out of the available
     * modules.
     */
    private transient JComboBox m_moduleChooser;

    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Sets up the main window and adds the Charts panel to it.
     *
     * @param s The name of the window
     * @param ps Reference to the PrimitiveStatistics module
     * @param bounds The initial window bounds
     * @param mwc Reference to the ModuleWindowContainer
     */
    public PrimitiveStatisticsWindow(String s, PrimitiveStatistics ps,
				     Rectangle bounds, ModuleWindowContainer mwc) {
	super(s, ps, bounds, mwc);
	m_primitiveStatistics = ps;
    m_charts = null;

    // Add the module chooser

    // get all available modules
    Vector modules = m_primitiveStatistics.getModuleManager().getModules();

    // construct a new vector containing the algorithm modules
    Vector algorithms = new Vector();

    for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
        Object o = e.nextElement();
        if (o instanceof Algorithm) {
            algorithms.add(o);
        }
    }

    m_moduleChooser = new JComboBox(algorithms);
    m_moduleChooser.addActionListener(this);

    m_contentPane.add(m_moduleChooser, BorderLayout.NORTH);

	revalidate();
	repaint();
    }//PrimitiveStatisticsWindow(String, PrimitiveStatistics, Rectangle, ModuleWindowContainer)


    // ----------- from Interface ModuleListener  ------------------

    /**
     * Just passes the repaint() call to the Chart object.
     *
     * @param cEvt Contains no information
     * @see Charts#repaint()
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
	    if (m_charts!=null) {
            m_charts.repaint();
        }
    }//moduleChanged(ChangeEvent)


    // -----------------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------------
    // -----------------------------------------------------

    /**
     * This class contains all functionality of the PrimitiveStatisticsWindow.
     * It paints the command call frequencies as a bar chart, using the
     * PrimitiveStatistics module to find out the frequencies.
     *
     * @see PrimitiveStatistics
     */
    private class Charts extends JPanel {

        /**
         * The font that is used for the description
         */
	private Font f = new Font("SansSerif", Font.PLAIN, 10);

        /**
         * The FontMetrics.
         */
	private FontMetrics fm;

        /**
         * The standard color.
         */
	private Color standardColor;

        /**
         * The bar color.
         */
	private Color barColor = new Color(0, 128, 0);       //grn

        /**
         * The bar text color.
         */
	private Color barTextColor = new Color(255,255,255); //wei

        /**
         * The dimension of the panel.
         */
	private Dimension dimension;

        /**
         * The height of the panel.
         */
	private int panelHeight;

        /**
         * The width of the panel.
         */
	private int panelWidth;

        /**
         * The scale offset.
         */
	private int scaleOffset = 30;       //grer oder gleich 20

        /**
         * The description offset.
         */
	private int descriptionOffset = 20; //grer oder gleich 15

        /**
         * Size of the gap between the bars.
         */
	private int gap = 5;

        /**
         * The maximum command call frequency. Is always between 0 and 10.
         */
	private int max;

        /**
         * The dimension of the frequency. actual number = max * base10
         */
	private int base10;

        /**
         * Passes the call to the superclass constructor and creates a new empty
         * dimension object.
         */
	public Charts() {
	    super();
	    dimension = getSize(new Dimension());
	}//Charts

        /**
         * Paints the bar chart dependent on the window bounds and the command
         * call frequencies given by the PrimitiveStatistics module.
         *
         * @param g The graphics context
         *
         * @see PrimitiveStatistics
         */
	public void paintComponent(Graphics g) {
	    super.paintComponent(g);
	    //zunchst Mae bestimmen...
	    init(g);
	    //Titel
	    g.drawString(m_primitiveStatistics.getAlgorithmName(),panelWidth/2,descriptionOffset/2);
	    drawScale(g);
	    drawBars(g);
	}//paintComponent(Graphics)

        /**
         * Prepares the painting process. It reads out the current window bounds
         * and the maximum frequency and thus calculates the scale.
         *
         * @param g The graphics context
         */
	private void init(Graphics g) {
	    standardColor = g.getColor();
	    g.setFont(f);
	    fm = g.getFontMetrics(f);
	    dimension = getSize(dimension);
	    panelHeight = dimension.height;
	    panelWidth = dimension.width;

	    int myMax = m_primitiveStatistics.getMax();
	    //Mastab berechnen...
	    //verschiebe auf die nchste runde Zahl, z.B. 534 auf 600, 12 auf 20 oder 8 auf 10
	    int i=1;
	    while (myMax/10 > 0) {
		myMax /= 10; //Wenn max grer als 10 war, Komma nach links verschieben, abrunden
		i++;         //Zhle gekrzte Zehnerpotenzen
	    }
	    int myBase10 = 1;
	    for (int j = 1; j < i; j++) myBase10 *= 10;
	    base10 = myBase10;
	    max = (myMax + 1) * myBase10;
	}//init(Graphics)

        /**
         * Draws the scale that has been calculated using init().
         *
         * @param g The graphics context
         *
         * @see PrimitiveStatisticsWindow.Charts#init(Graphics)
         */
	private void drawScale(Graphics g) {
	    //Spitze zeichnen
	    Polygon s = new Polygon();
	    s.addPoint(scaleOffset - 3,8);
	    s.addPoint(scaleOffset - 6,20);
	    s.addPoint(scaleOffset,20);
	    g.fillPolygon(s);
	    //Skala zeichnen
	    g.drawLine(scaleOffset - 3, descriptionOffset - 4, scaleOffset - 3,
		       panelHeight - descriptionOffset);
	    int y;
	    int x;
	    for(int i = max; i >= 0; i-=base10) {
		y = getY(i);
		g.drawLine(scaleOffset - 6, y, scaleOffset, y);
		x = scaleOffset - 6 - fm.stringWidth("" + i); //rechtsbndig
		if (x < 0) x = 0; //aber noch auf die Seite...
		g.drawString(""+i,x,y);
	    }
	}//drawScale(Graphics)

        /**
         * Draws the bars dependent on the scale and the frequencies.
         *
         * @param g The graphics context
         */
	private void drawBars(Graphics g) {
	    int n = m_primitiveStatistics.getNumberOfCommands();
	    int barWidth = (panelWidth - scaleOffset - gap) / n;
	    int x = scaleOffset + gap;
	    for (int i=0; i < n; i++) {
		int y = getY(m_primitiveStatistics.getCommandCounter(i));
		g.drawString(m_primitiveStatistics.getCommand(i).toString(), x,
			     panelHeight - descriptionOffset + fm.getAscent() + 2);
		g.setColor(barColor);
		g.fillRect(x, y, barWidth - gap, panelHeight - descriptionOffset - y);
		g.draw3DRect(x, y, barWidth - gap, panelHeight - descriptionOffset - y, true);
		g.setColor(barTextColor);
		y += fm.getAscent();
		if (y > panelHeight - descriptionOffset) y = panelHeight - descriptionOffset;
		g.drawString(""+m_primitiveStatistics.getCommandCounter(i), x+gap, y + fm.getAscent());
		g.setColor(standardColor);
		x += barWidth;
            }
	}//drawBars(Graphics)

        /**
         * Calculates the height of a bar depending on the specified value and
         * the scale.
         *
         * @param counterValue The frequency
         * @return The height in pixel
         */
	private int getY(int counterValue) {
	    return (int) (
			  (double)descriptionOffset +
			  (((double)max - (double)counterValue)/(double)max) *
			  ((double)panelHeight - (double)(2*descriptionOffset)));
	}//getY(counterValue)
    } // class Charts

    // --------------------- from Interface ActionListener -------------------
    public void actionPerformed(ActionEvent evt) {
        Algorithm alg = (Algorithm) m_moduleChooser.getSelectedItem();
        m_primitiveStatistics.setAlgorithm(alg);
        if (m_charts!=null) {
            m_contentPane.remove(m_charts);
        }
        m_charts = new Charts();
	    m_contentPane.add(m_charts, BorderLayout.CENTER);
        revalidate();
        repaint();
    }
} // class PrimitiveStatisticsWindow
