////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PrimitiveTrigger.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 15:56:40 2000
//  Update count     : 4
//  CVS Info         : $Id: PrimitiveTrigger.java,v 1.1.1.2 2002/03/18 12:42:42 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  03.03.01    vigelius        revised documentation
//  08.03.01    vigelius        changed ChangeEvent to ModuleChangeEvent
//  15.03.01    vigelius    user is now able to change the algorithm
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.ModuleChangeEvent;

/**
* This class reprensents a simple trigger module, through which the user gains
* the ability, to let an algorithm perform continously, until a specified
* command (like pointer comparison etc.) is reached.
*
*/
public class PrimitiveTrigger extends AbstractModule implements ModuleListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // ----------------------------------------------------

    /**
     * Reference to the triggered algorithm.
     * @serial
     */
    private Algorithm m_algorithm;

    /**
     * Array of the possible commands of this algorithm.
     * @serial
     */
    private Command[] m_commands;

    /**
     * Last performed command of this algorithm.
     * @serial
     */
    private Command m_lastCommand;

    /**
     * The command, that should be triggered on.
     * @serial
     */
    private Command m_triggerCommand;

    /**
     * Indicates the state of the trigger module (True, if it is triggering at
     * the moment).
     * @serial
     */
    private boolean m_triggering;

    /**
     * True, if it should be triggered on the next step.
     * @serial
     */
    private boolean m_myHistoryStep;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Looks for the algorithm module and reads out its command list.a It
     * registers itself as a ModuleListener to the Algorithm as well.
     *
     * @param mm The associated ModuleManager
     *
     * @see visalg.basics.Algorithm
     * @see visalg.basics.Command
     * @see visalg.basics.ModuleListener
     */
    public PrimitiveTrigger(ModuleManager mm) {
	super("Primitive Trigger module", mm);
	//Algorithmus suchen, linken
	/*findAlgorithm();
	if (m_algorithm!=null) {//Sicherstellen, da ein Algorithmus gefunden wird...
	}*/
        m_algorithm = null;
        m_commands = null;
        m_lastCommand = null;
        m_triggerCommand = null;
        m_triggering = false;
        m_myHistoryStep = false;
    }//PrimitiveTrigger(ModuleManager)

    /**
     * Returns the name of the algorithm, that is triggered on.
     *
     * @return The name of the algorithm
     */
    public String getAlgorithmName() {
	return m_algorithm.toString();
    }

    /**
     * Returns the command list of the triggered algorithm.
     *
     * @return The command list
     */
    public Command[] getCommands() {
	return m_commands;
    }//getCommands()

    /**
     * Returns the state of the Trigger module.
     *
     * @return True, if it is currently triggering
     */
    public boolean isTriggering() {
	return m_triggering;
    }//isTriggering()

    /**
     * Invokes a ModuleManager.historyStep() until the algorithm has reached a
     * specified command or the algorithm has finished. The associated window
     * will be notified, by calling fireEvent().
     *
     * @param op The command, that is triggered on
     *
     * @see visalg.basics.Algorithm
     * @see visalg.basics.AbstractModule#fireEvent()
     * @see visalg.modules.PrimitiveTriggerWindow
     */
    public void triggerOnOperation(Command op) {
	if (op!=null) {
	    m_triggering = true;
	    m_triggerCommand = op;
	    fireEvent();
	    m_moduleManager.historyStep();
	}
    }//triggerOnOperation(Command)

    /**
     * Stops the triggering process and notifies the PrimitiveTriggerWindow.
     *
     * @see PrimitiveTriggerWindow
     *
     * @todo I dont know, what this stop trigger is for - it does not stop the
     * triggering process.
     */
    public void stopTriggering() {
	m_triggering = false;
	fireEvent();
    }//stopTriggering()

    /**
     * Sets a new algorithm, that should be triggered. The command list of this
     * algorithm is loaded, and the module registers itself as a listener for it.
     * If a former algorithm is loaded, it deregisters itself.
     *
     * @param alg The new algorithm
     */
    public void setAlgorithm(Algorithm alg) {
        if (m_algorithm!=null) {
            m_algorithm.removeModuleListener(this);
        }
        m_algorithm = alg;

        m_algorithm.addModuleListener(this);
        //Initialisieren...
        m_commands = m_algorithm.getCommands();
        m_lastCommand = m_algorithm.getLastCommand();
        m_triggerCommand = null;
        m_triggering = false;
        m_myHistoryStep = false;
    }
    // ----------- from Interface Module  ------------------

    /**
     * Creates a new PrimitiveTriggerWindow object.
     *
     * @see PrimitiveTriggerWindow
     */
    public void newModuleWindow() {
        try {
        	m_moduleWindow =
	        new PrimitiveTriggerWindow("Primitives Triggermodul",
	        			       this,
					       m_moduleWindowBounds,
					       m_moduleManager.getModuleWindowContainer());
	    }
	    catch (Exception e) {
		System.err.println(e);
	    }

    }//newModuleWindow()

    /**
     * @todo add some useful information here
     */
    public void step() {
	super.step();
	if (m_myHistoryStep) {
	    m_myHistoryStep = false;
	    m_moduleManager.historyStep();
	}
    }


    // ----------- from Interface ModuleListener  ------------------

    /**
     * @todo add some useful information here
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
	if (m_triggering) {
	    m_lastCommand = m_algorithm.getLastCommand();
	    if ((m_lastCommand==null) || (m_triggerCommand==null)) {
		m_triggering = false;
		fireEvent();
		return;
	    }

	    if (m_triggerCommand.getClass().isInstance(m_lastCommand)) m_triggering = false;
	    if (m_algorithm.isDone()) m_triggering = false;
	    fireEvent();
	    if (m_triggering) m_myHistoryStep = true;
	}
    }//moduleChanged(ChangeEvent)

    // -----------------------------------------------------
    // ------- M e t h o d s ( p r i v a t e ) -------------
    // -----------------------------------------------------

} // class PrimitiveTrigger
