////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : PrimitiveTriggerWindow.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Mon Oct  9 16:06:44 2000
//  Update count     : 2
//  CVS Info         : $Id: PrimitiveTriggerWindow.java,v 1.1.1.2 2002/03/18 12:42:42 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//  03.03.01    vigelius        revised documentation
//  08.03.01    vigelius    changed ChangeEvent to ModuleChangeEvent
//  15.03.01    vigelius    user is now able to change the algorithm
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import visalg.modules.PrimitiveTrigger;

import visalg.basics.Module;
import visalg.basics.ModuleListener;
import visalg.basics.*;
import visalg.graphics.ModuleWindow;
import visalg.graphics.ModuleWindowContainer;
import visalg.types.ModuleChangeEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


/**
* This class represents the front end of the PrimitiveTrigger module. The user
* can choose between all available commands and start or stop the triggering
* process.
*
* @see PrimitiveTrigger
*
* @todo I do not know, what this start/stop button is for
*/
public class PrimitiveTriggerWindow extends ModuleWindow implements ActionListener {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the PrimitiveTrigger module.
     */
    private transient PrimitiveTrigger m_primitiveTrigger;

    /**
     * JList item, containing all available commands of the triggered algorithm.
     */
    private transient JList m_commandList;

    /**
     * Button for the "start/stop Trigger" item.
     */
    private transient JButton m_triggerButton;

    /**
     * Combobox for the list of available algorithms
     */
    private transient JComboBox m_algorithmBox;
    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Sets up the main window including the list of available commands, that
     * is received by calling PrimitiveTrigger#getCommands().
     *
     * @param s The name of the window
     * @param pt Reference to the PrimitiveTrigger module
     * @param bounds The initial window bounds
     * @param mwc Reference to the associated ModuleWindowContainer
     *
     * @see PrimitiveTrigger
     * @see PrimitiveTrigger#getCommands()
     */
    public PrimitiveTriggerWindow(String s, PrimitiveTrigger pt,
				  Rectangle bounds, ModuleWindowContainer mwc) {
	super(s, pt, bounds, mwc);
	m_primitiveTrigger = pt;

	//m_contentPane.add(new JLabel(m_primitiveTrigger.getAlgorithmName()), BorderLayout.NORTH);

	ListModel commandsModel = new AbstractListModel() {

		public int getSize() {
		    if (m_primitiveTrigger.getCommands()==null) {
                        return 0;
                    } else {
                        return m_primitiveTrigger.getCommands().length;
                    }
		}

		public Object getElementAt(int index) {
                    if (m_primitiveTrigger.getCommands()==null) {
                        return "";
                    } else {
		        return "" + (m_primitiveTrigger.getCommands())[index];
                    }
		}
	    };

	m_commandList = new JList(commandsModel);
	m_commandList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	m_contentPane.add(new JScrollPane(m_commandList), BorderLayout.CENTER);

	m_triggerButton = new JButton("Trigger");
	m_triggerButton.addActionListener(this);
        m_triggerButton.setEnabled(false);
	m_contentPane.add(m_triggerButton, BorderLayout.SOUTH);

        // get all available modules
        Vector modules = m_primitiveTrigger.getModuleManager().getModules();

        // construct a new vector containing the algorithm modules
        Vector algorithms = new Vector();

        for (Enumeration e = modules.elements(); e.hasMoreElements(); ) {
            Object o = e.nextElement();
            if (o instanceof Algorithm) {
                algorithms.add(o);
            }
        }

        m_algorithmBox = new JComboBox(algorithms);
        m_algorithmBox.addActionListener(this);

        m_contentPane.add(m_algorithmBox, BorderLayout.NORTH);
	revalidate();
	repaint();
    }//PrimitiveTriggerWindow(String,PrimitiveTrigger,Rectangle,ModuleWindowContainer)

    /**
     * Creates a new data model for the ListBox. This is necessary, after the
     * user has changed the algorithm, that has to be triggered.
     */
    private void setModel() {
	ListModel commandsModel = new AbstractListModel() {

		public int getSize() {
		    if (m_primitiveTrigger.getCommands()==null) {
                        return 0;
                    } else {
                        return m_primitiveTrigger.getCommands().length;
                    }
		}

		public Object getElementAt(int index) {
                    if (m_primitiveTrigger.getCommands()==null) {
                        return "";
                    } else {
		        return "" + (m_primitiveTrigger.getCommands())[index];
                    }
		}
	    };
        m_commandList.setModel(commandsModel);
    }//setModel
    // ----------- from Interface ModuleListener  ------------------

    /**
     * Sets the button label depending on PrimitiveTrigger.isTriggering().
     *
     * @param cEvt Contains no information
     *
     * @see PrimitiveTrigger#isTriggering()
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
        if(m_primitiveTrigger.isTriggering()) {
            m_triggerButton.setText("Stop triggering");
            m_triggerButton.setActionCommand("Stop triggering");
            m_triggerButton.invalidate();
        }
        else {
            m_triggerButton.setText("Trigger");
            m_triggerButton.setActionCommand("Trigger");
            m_triggerButton.invalidate();
        }
        m_triggerButton.revalidate();
        repaint();
    }//moduleChanged(ChangeEvent)


    // ----------- from Interface ActionListener  ------------------

    /**
     * Will be called, if the user presses the "start/stop Trigger" button.
     * It looks for the current state of the trigger. If it is not currently
     * triggering, it will start triggering on the selected command by calling
     * PrimitiveTrigger.triggerOnOperation(Command). Otherwise, it will stop
     * triggering.
     *
     * @param evt The passed ActionEvent
     *
     * @see PrimitiveTrigger#triggerOnOperation(Command)
     */
    public void actionPerformed(ActionEvent evt) {

        if (evt.getSource() == m_triggerButton) {
            if (m_triggerButton.getText().equals("Trigger")) {
                int i = m_commandList.getSelectedIndex();
                if (i >= 0) {
                    m_primitiveTrigger.triggerOnOperation((m_primitiveTrigger.getCommands())[i]);
                }
                m_triggerButton.setText("Stop triggering");
                m_triggerButton.setActionCommand("Stop triggering");
            }
            else {
                m_primitiveTrigger.stopTriggering();
                m_triggerButton.setText("Trigger");
                m_triggerButton.setActionCommand("Trigger");
            }
            revalidate();
            repaint();
        } else if (evt.getSource()==m_algorithmBox) {
            Algorithm alg = (Algorithm) m_algorithmBox.getSelectedItem();

            if (alg!=null) {
                m_primitiveTrigger.setAlgorithm(alg);
                m_triggerButton.setEnabled(true);
                setModel();
                revalidate();
                repaint();
            }
        }

    }//actionPerformed(ActionEvent)
} // class PrimitiveTriggerWindow
