////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : QuickSort.java
//  Description      :
//  Author           : Babsi & Jochen
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Thu Sep 28 11:23:29 2000
//  Update count     : 1
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//  21.02.01    vigelius        new module window uses correct window bound
//  06.03.01    vigelius        algorithm does work now (nearly)
//  06.03.01    vigelius        revised documentation
//  06.06.01    vigelius        QuickSort now uses properties
//  06.11.01    vigelius        Does now use VisAlgVector
//  07.11.01    vigelius        Does now support module changes
//
///////////////////////////////////////////////////////////////////////////////

package visalg.modules;

import java.util.*;
import java.awt.event.*;
import java.io.File;
import java.awt.Color;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.*;

import visalg.converter.*;

/**
 * This class implements the QuickSort algorithm. QuickSort is one of the
 * fastest sorting algorithms. The runtime in the worst case is O(N^2), in the
 * best case O(N log N). The average runtime is hardly worse than the best one.
 *
 * The set is divided in two subsets. This is done by choosing one Pivot element
 * and copying all elements, that are smaller or equal this Pivot element into
 * the left subset. After that, there are in the right subset just elements,
 * that are bigger than the Pivot element.
 *
 * The Pivot element can be chosen randomly at the beginning. If one chooses
 * the element at the right border, then one can let run two pointers from
 * left to right respective from right to left. If the left pointer has found
 * an element, that is bigger than the Pivot element and the right pointer has
 * found one, that is smaller, these elements are swapped.
 *
 * The two pointers will meet at an element, that is equal or bigger than the
 * Pivot element. This element is swapped with the Pivot element, and is chosen
 * as the new Pivot element.
 *
 * QuickSort is then called for each, the right subset and the left subset.
 *
 * QuickSort is a recursive algorithm. This implementation, however, is not
 * recursive. A non-recursive algorithm must use a stack in order to remember
 * the subsets, that are to be sorted. This implementation does not use a stack
 * either, and is by that not a real QuickSort algorithm ! It does treat the
 * left subsets correctly, but will set the right border of right subsets to the
 * end of the data set and by that perform more steps than necessary. Additionally,
 * the end condition is not correctly. It can, however, demonstrate the partition
 * algorithm.
 *
 * Because the implementation of the algorithm is quite "spagetti-code" like,
 * a basic-like description can be found in the file "QuickSort.basic".
 *
 * @todo Add a real recursive QuickSort algorithm
*/

public class QuickSort extends Algorithm {

        /**
         * Current data module.
         * @serial
         */
        private DataModule m_dataModule=null;

        /**
         * Current pivot number.
         * @serial
         */
        private int m_pivot;

        private IndicatorBarProperty m_pivotProperty =
            new IndicatorBarProperty("Pivot", new Color(255,255,255), -1, -1);

        /**
         * Left border of the current subset.
         * @serial
         */
        private int m_left1;

        /**
         * Recursion level.
         * @serial
         */
        private int r;

        /**
         * Right border of the current subset.
         * @serial
         */
        private int m_right1;

        /**
         * Left pointer.
         * @serial
         */
        private int m_left;

        private IndicatorBarProperty m_leftProperty =
            new IndicatorBarProperty("Left", new Color(0,0,255), -1, 1);

        /**
         *  Right pointer.
         *  @serial
         */
        private int m_right;

        private IndicatorBarProperty m_rightProperty =
            new IndicatorBarProperty("Right", new Color(128,0,0), -1, 0);

        /**
         * Pointer to the last sorted element.
         * @serial
         */
        private int m_sorted;

        private IndicatorBarProperty m_sortedProperty =
            new IndicatorBarProperty("Sorted", new Color(128,100,0), -1, 2);

        /**
         * Result of the last comparison.
         * @serial
         */
        private int m_comparation;

        /**
         * True, if left1 is the only element.
         * @serial
         */
        private boolean m_left1TheOnlyElement;

        /**
         * True, if left1 equals sorted-1.
         * @serial
         */
        private boolean m_left1AtSortedMinusOne;

        /**
         * True, if left equals right.
         * @serial
         */
        private boolean m_leftAtRight;

        /**
         * True, if right equals left.
         * @serial
         */
        private boolean m_rightAtLeft;

        /**
         * True, if left equals right1.
         * @serial
         */
        private boolean m_leftAtRight1;


        /**
        * Initialisation of command list, pointers, states
        * @param p Current Project
        */
        public QuickSort(Project p) {
                super("QuickSort", p);
                m_done = false;
                m_commands = new Command[] {
                        new PointerComparison(),
                        new PointerSetting(),
                        new ValueComparison(),
                        new SwopValues(),
                        new PointerName()
                };
                // Pointer Initialisation
                m_pivot = 0;
                m_left1 = 0;
                m_right1 = getDataSize()-1;
                r = 0;
                m_sorted = getDataSize();
                // Initialisation of state variables
                m_comparation = 0;
                m_left1TheOnlyElement = false;
                // Initial State of Command List
                setNextCommand(new IsLinksTheOnlyElement());
        } // QuickSort

        /**
         * Returns current Data Module
         * @return m_dataModule
         */
        public DataModule getDataModule() {
                return m_dataModule;
        } // getDataModule

        /**
         * Sets current Data Module
         * @param DataModule Module to be set
         */
        public void setDataModule(DataModule dm) {
                super.setDataModule(dm);
                if (dm==null) return;
                Object o = dm.getData();
                if (o==null) return;
                if (!(o instanceof VisAlgVector)
                    ||!((VisAlgVector) o).isComparable()) return;

                VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector) o).toArray();
                m_dataModule = dm;
                reset();
                registerProperties();
               fireEvent();
        } // setDataModule

        /**
         * Returns size of current data
         * @return length of data
         */
        public  int getDataSize() {
                if (m_dataModule==null) return 0;
                Object o = m_dataModule.getData();
                if (!(o instanceof VisAlgVector)
                    ||!((VisAlgVector)o).isComparable()) return 0;
                VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();
                return data.length;
        } // getDataSize

        /**
         * Gets a specific data element
         * @param index element index
         * @return element
         */
        public VisAlgComparableData getDataElementAt(int index) {
                if (m_dataModule==null) return null;
                Object o = m_dataModule.getData();
                if (!(o instanceof VisAlgVector)
                    ||!((VisAlgVector) o).isComparable()) return null;

                VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();
                if ((index < 0) || (index >= data.length)) return null;
                return data[index];
        } // getDataElementAt

        /**
         * returns member variable r
         * @return r
         */
        public int getR() {
                return r;
        } // getR

        /**
         * returns member variable m_pivot
         * @return m_pivot
         */
        public int getPivot() {
                return m_pivot;
        } // getPivot

        /**
         * returns member variable m_left1
         * @return m_left1
         */
       	public int getLeft1() {
        	return m_left1;
    	} // getLeft1

        /**
         * Returns member variable m_right1.
         * @return m_right1
         */
    	public int getRight1() {
        	return m_right1;
    	}//getRight1

        /**
         * Returns member variable m_left.
         * @return m_left
         */
        public int getLeft() {
                return m_left;
        }//getLeft

        /**
         * Returns member variable m_right.
         * @return m_right
         */
        public int getRight() {
                return m_right;
        }//getRight

        /**
         * Returns member variable m_sorted.
         * @return m_sorted
         */
        public int getSorted() {
                return m_sorted;
        }//getSorted

        /**
         * Returns member variable m_comparation.
         * @return m_comparation
         */
        protected int getComparation() {
                return m_comparation;
        }//getComparation

        /**
         * Returns member variable m_left1AtSortedMinusOne.
         * @return m_left1AtSortedMinusOne.
         */
        protected boolean getLinksAtSortedMinusOne() {
                return m_left1AtSortedMinusOne;
        }//getLinksAtSortedMinusOne

        /**
         * Returns member variable m_left1TheOnlyElement.
         * @return m_left1TheOnlyElement
         */
	protected boolean getLinksTheOnlyElement() {
                return m_left1TheOnlyElement;
        }//getLinksTheOnlyElement

        /**
         * Returns member variable m_leftAtRight.
         * @return m_leftAtRight
         */
        protected boolean getLeftAtRight() {
                return m_leftAtRight;
        }//getLeftAtRight

        /**
         * Returns member variable m_rightAtLeft.
         * @return m_rightAtLeft
         */
        protected boolean getRightAtLeft() {
                return m_rightAtLeft;
        }//getRightAtLeft

        /**
         * Returns member variable m_leftAtRight1.
         * @return m_leftAtRight1
         */
       	protected boolean getLeftAtRechts() {
                return m_leftAtRight1;
        }//getLeftAtRechts

        /**
         * Resets all member states and stops the algorithm.
         */
        protected void reset() {
                //Zeiger
                m_pivot = 0;
                m_left1 = 0;
                m_right1 = getDataSize();
                r = 0;
                m_sorted = getDataSize();
                //Zwischenergebnisse
                m_comparation = 0;
                m_left1TheOnlyElement = false;
                //Anfangszustand
                setNextCommand(new IsLinksTheOnlyElement());

                m_done = false;

                m_time  = 0;

                fireEvent();
        }//reset

        /**
         * Sets the next executed command.
         * @param ac The command, that will be executed next.
         */
        protected void setNextCommand(Command ac) { super.setNextCommand(ac); }

        /**
         * Sets the algorithm state to "done" or "not done".
         * @param b True, if the algorithm has finished
         */
        protected void setDone(boolean b) { super.setDone(b); }

        //OPERATIONEN
        /**
         * Superclass for all pointer comparing commands.
         */
        private class PointerComparison extends AbstractCommand {
                public String toString() {
                        return "Zeigervergleich";
                }
        }//class PointerComparison

        /**
         * Superclass for all pointer setting commands.
         */
        private class PointerSetting extends AbstractCommand {
		public String toString() {
			return "Set Pointer";
		}
	}//class PointerSetting

        /**
         * Superclass for all value comparing commands.
         */
        private class ValueComparison extends AbstractCommand {
                public String toString() {
                        return "Compare Values";
                }
        }//class ValueComparison

        /**
         * Superclass for all value swapping commands.
         */
        private class SwopValues extends AbstractCommand {
                public String toString() {
                        return "Swap Values";
                }
        }//class SwopValues

        /**
         * Superclass for all pointer naming commands.
         */
        private class PointerName extends AbstractCommand {
		public String toString() {
			return "Name Pointer";
		}
	}//class PointerName

        // Implementation classes

        private class IsLinksTheOnlyElement extends PointerComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();


                        if (getLeft1() < data.length-1) {
                                m_left1TheOnlyElement = false;
                                setNextCommand(new ZeigerName());
                        } else {
                                m_left1TheOnlyElement = true;
                                setNextCommand(null);
                                setDone(true);
                        }
                }
                public String toString() {
                        return "Compare left1 - only element";
                }
        }
        private class ZeigerName extends PointerName {
                public void doIt() {
                     // Abbruchbedingung
                    if (m_sorted==getDataSize()-1&&m_left1==m_sorted&&m_right1==m_sorted)
                        setDone(true);
                        //String m_sorted = "sorted";
			//m_sorted = m_sorted+r; // +"r"
                        setNextCommand(new IsLinksAtSortedMinusOne());
                }
                public String toString() {
                        return "Name Pointer";
                }
        }
	private class IsLinksAtSortedMinusOne extends PointerComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

                        if (getLeft1() < getSorted()-1) {
                                m_left1AtSortedMinusOne = false;
                                setNextCommand(new IncreaseR());
                        } else {
                                m_left1AtSortedMinusOne = true;
                                setNextCommand(new SetRightSidePointer());
                        }
                }
   	}
        private class IncreaseR extends PointerName {
                public void doIt() {
                        r = r+1;
			setNextCommand(new SetLeftSidePointer());
                }
                public String toString() {
                        return "Name Pointer";
                }
        }
	private class SetLeftSidePointer extends PointerSetting {
                public void doIt() {
                        m_left = m_left1;
                        m_pivot = m_left1;
                        m_right = m_sorted-1;
                        setNextCommand(new IsLeftAtRight());
                }
                public String toString() {
                        return "Set Pointer m_sorted";
                }
        }
        private class IsLeftAtRight extends PointerComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

                        if (getLeft() < getRight()) {
                                m_leftAtRight = false;
                                setNextCommand(new IncreaseLeft());
                        } else {
                                m_leftAtRight = true;
                                setNextCommand(new SwopPivotAndLeft());
                        }
                }
                public String toString() {
                        return "Compare Pointer m_left - m_right";
                }
        }
        private class IncreaseLeft extends PointerSetting {
                public void doIt() {
                        m_left += 1;
                        setNextCommand(new CompareLeftWithPivot());
                }
                public String toString() {
                        return "Set Pointer m_left";
                }
        }
        private class CompareLeftWithPivot extends ValueComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }

                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

                        m_comparation = data[getLeft()].compareTo(data[getPivot()]);
                        if (m_comparation < 0) {
                                setNextCommand(new IsRightAtLeft()); // pivot<left
                        } else {
                                setNextCommand(new IsLeftAtRight()); //pivot>left
                        }
                }
                public String toString() {
                        return "Compare values m_left - m_pivot";
                }
        }
        private class IsRightAtLeft extends PointerComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

                        if (getLeft() < getRight()) {
                                m_rightAtLeft = false;
                                setNextCommand(new CompareRightWithPivot());
                        } else {
                                m_rightAtLeft = true;
                                setNextCommand(new SwopPivotAndLeftMinusOne());
                        }
                }
                public String toString() {
                        return "Compare Values m_left - m_right";
                }
        }
        private class CompareRightWithPivot extends ValueComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

                        m_comparation = data[getRight()].compareTo(data[getPivot()]);
                        if (m_comparation > 0) {
                                setNextCommand(new SwopLeftAndRight()); // pivot>right
                        } else {
                                setNextCommand(new DecreaseRight()); // pivot<right
                        }
                }
                public String toString() {
                        return "Compare Values m_right - m_pivot";
                }
        }
        private class SwopLeftAndRight extends SwopValues {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }

                        VisAlgVector v = (VisAlgVector) o;
                        v.swapValues(getLeft(), getRight());

                        setNextCommand(new IsLeftAtRight());
                }
                public String toString() {
                        return "Swap Values m_left - m_right";
                }
        }
        private class DecreaseRight extends PointerSetting {
                public void doIt() {
                        m_right -= 1;
                        setNextCommand(new IsRightAtLeft());
                }
                public String toString() {
                        return "Set Pointer m_right";
                }
        }
        private class SwopPivotAndLeftMinusOne extends SwopValues {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }

                        VisAlgVector v = (VisAlgVector) o;
                        v.swapValues(getPivot(), getLeft()-1);

                        setNextCommand(new SetSortedToLeftMinusOne());
                }
                public String toString() {
                        return "Swap Values m_pivot - m_left-1";
                }
        }
        private class SetSortedToLeftMinusOne extends PointerSetting {
                public void doIt() {
                        m_sorted = m_left-1;
                        setNextCommand(new SetRechtsToSorted());
                }
                public String toString() {
                        return "Set Pointer m_sorted";
                }
        }
        private class SetRechtsToSorted extends PointerSetting {
                public void doIt() {
                        m_right1 = m_sorted;
			setNextCommand(new ZeigerName());
                }
                public String toString() {
                        return "Set Pointer m_right1 to m_sorted";
                }
        }
	private class SwopPivotAndLeft extends SwopValues {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }

                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }

                        VisAlgVector v = (VisAlgVector) o;
                        v.swapValues(getPivot(), getLeft());

                        setNextCommand(new SetSortedToLeft());
                }
                public String toString() {
                        return "Swap Values m_pivot - m_left";
                }
        }
        private class SetSortedToLeft extends PointerSetting {
                public void doIt() {
                        m_sorted = m_left;
                        setNextCommand(new SetRechtsToSorted());
                }
                public String toString() {
                        return "Set Pointer m_sorted";
                }
        }
        private class SetRightSidePointer extends PointerSetting {
                public void doIt() {
			m_left1 = m_sorted+1;
                        m_left = m_sorted+1;
                        m_pivot = m_sorted+1;
  			setNextCommand(new DecreaseR());
                }
                public String toString() {
                        return "Set Right Side Pointers";
                }
        }
        private class DecreaseR extends PointerName {
                public void doIt() {
                        r = r-1;
                /*	String sorted = "sorted";*/
			//m_sorted = m_sorted +r; // "r"
			setNextCommand(new SetRightAndRechts());
                }
                public String toString() {
                        return "Name Pointer";
                }
        }
	private class SetRightAndRechts extends PointerSetting {
                public void doIt() {
                        m_right1=getDataSize()-1;
                        m_right=getDataSize()-1;
                        /*m_right1 = m_sorted;
			m_right = m_sorted;*/
                        setNextCommand(new IsLeftAtRechts());
                }
                public String toString() {
                        return "Set Right Border Pointers";
                }
        }
	private class IsLeftAtRechts extends PointerComparison {
                public void doIt() {
                        //Fehlerhafte Konstellation abfangen...
                        if (m_dataModule==null) {
                                reset();
                                return;
                        }
                        Object o = m_dataModule.getData();
                        if (!(o instanceof VisAlgVector)
                            ||!((VisAlgVector)o).isComparable()) {
                                reset();
                                return;
                        }
                        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

                        if (getLeft() < getRight1()) {
                                m_leftAtRight1 = false;
                                setNextCommand(new IncreaseLeft());
                        } else {
                                m_leftAtRight1 = true;
                                setNextCommand(new SwopPivotAndLeft());
                        }
                }
                public String toString() {
                        return "Compare Pointer m_left - m_right1";
                }
        }

        /**
         * Checks, if the data is valid, and resets the algorithm if not.
        */
        public void step() {
                if ((m_dataModule==null) ||
                    (m_dataModule.getData()==null) ||
                    !(m_dataModule.getData() instanceof VisAlgVector)
                    ||!((VisAlgVector)m_dataModule.getData()).isComparable())
                        reset();

                super.step();
        }//step()

        /**
         * Creates a new QuickSortWindow object.
         *
         * @see QuickSortWindow
         */
        public void newModuleWindow() {
                try {
                        m_moduleWindow = new QuickSortWindow(m_name,
                                                            this,
                                                            m_moduleWindowBounds,
                                                            m_moduleManager.getModuleWindowContainer());
                } catch (Exception e) {}
        }//newModuleWindow

        public void registerProperties() {
               m_dataModule.addProperty(m_pivotProperty);
                m_dataModule.addProperty(m_leftProperty);
                m_dataModule.addProperty(m_rightProperty);
                m_dataModule.addProperty(m_sortedProperty);
        }

        public void setProperties() {
            m_pivotProperty.setIndicatorPosition(m_pivot);
            m_leftProperty.setIndicatorPosition(m_left);
            m_rightProperty.setIndicatorPosition(m_right);
            m_sortedProperty.setIndicatorPosition(m_sorted);
            if (m_dataModule!=null) {
                m_dataModule.propertyChanged();
            }
        }

    public void moduleChanged(ModuleChangeEvent evt) {
        reset();
        registerProperties();
        setProperties();
    }//moduleChanged(ModuleChangeEvent)
}//class QuickSort
