////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : SelectionSort.java
//  Description      :
//  Author           : Martin Spoden
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Tue Oct 12 10:32:28 1999
//  Last modified by : ahr
//  Last modified on : Fri Sep 29 15:53:20 2000
//  Update count     : 23
//  CVS Info         : $Id: SelectionSort.java,v 1.2 2002/03/18 12:28:50 vigelius Exp $
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		    Changes/Extensions
//  ----	----		    ------------------
//  21.02.01    vigelius    new module window uses correct window bound
//  03.03.01    vigelius    revised documentation
//  06.06.01    vigelius    now uses properties
//  06.11.01    vigelius    does now use VisAlgVector
//  07.11.01    vigelius    Does now support module changes
//
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import java.util.*;
import java.awt.event.*;
import java.awt.Color;

import java.io.File;

import visalg.basics.*;
import visalg.graphics.*;
import visalg.types.*;
import visalg.converter.*;


/**
 * This class implements the SelectionSortAlgorithm. The SelectionSort algorithm
 * suitable for the sorting of big data sets, in that the swapping of values
 * takes a lot of time, but not the comparing.
 *
 * Selection Sort looks for the smallest element in the set and swaps it with
 * the first one. After that, the smallest element of the not yet sorted set is
 * swapped with the second one. It is necessary for that, to use two interlocked
 * loops. The outer one provides the position of the element, that is to be
 * swapped next, the inner one looks for the smallest element of the unsorted
 * data set.
 *
 * Due to the high number of comparisons, the runtime of this algorithm is
 * $O(N^2)$. The swapping commands, however, take $O(N)$ memory, so that this
 * algorithm is suitable for complex data sets.
 */
public class SelectionSort extends Algorithm {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
     * Reference to the DataModule.
     * @serial
     */
    private DataModule m_dataModule;

    /**
     * Position, of the last sorted element.
     * @serial
     */
    private int m_sorted;
    private IndicatorBarProperty m_sortedProperty =
        new IndicatorBarProperty("Sorted", new Color(0,0,255), -1, 1);

    /**
     * Position of the currently smalles element of the unsorted set.
     * @serial
     */
    private int m_smallest;
    private IndicatorBarProperty m_smallestProperty =
        new IndicatorBarProperty("Smallest", new Color(255,255,255), -1,-1);

    /**
     * Position of the current element.
     * @serial
     */
    private int m_current;
    private IndicatorBarProperty m_currentProperty =
        new IndicatorBarProperty("Current", new Color(128,0,0), -1, 0);

    /**
     * Result of the last comparison.
     * @serial
     */
    private int m_comparison;

    /**
     * Indicates, if the current element is the last element of the array.
     * @serial
     */
    private boolean m_currentAtEnd;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Creates a list of all available commands and initialises internal
     * states. The first command executed is IsCurrentAtEnd().
     *
     * @param p Reference to the associated project.
     */
    public SelectionSort(Project p) {

	super("SelectionSort", p);

	// Initialize list of commands (from Algorithm)
	m_commands = new Command[] {
	    new PointerComparison(),
	    new PointerSetting(),
	    new ValueComparison(),
	    new SwopValues(),
	};

	// Initialize variables
	m_sorted = -1;
	m_smallest = 0;
	m_current = 0;
	m_comparison = 0;
	m_currentAtEnd = false;
	m_done = false;

	// Set first command
	setNextCommand(new IsCurrentAtEnd());
    }//SelectionSort(Project)

    /**
     * Returns the data element at position index.
     * @return The data element.
     */
    public VisAlgComparableData getDataElementAt(int index) {

	if (m_dataModule == null) {
	    System.err.println("SelectionSort.getDataElementAt(): Data Module is null!");
	    return null;
	}

	Object o = m_dataModule.getData();
    if (!(o instanceof VisAlgVector)
        ||!((VisAlgVector)o).isComparable()) {
            reset();
            return null;
        }
    VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

	if ( (index < 0) || (index >= data.length) ) {
	    System.err.println("SelectionSort.getDataElementAt(): Index out of bounds!");
	    return null;
	}

	return data[index];
    }//getDataElementAt(int)

    /**
     * Returns the dataModule the algorithm works on.
     *
     * @return Reference to the DataModule
     */
    public DataModule getDataModule() {
	return m_dataModule;
    }//getDataModule()

    /**
     * Gives the length of the data array.
     * @return Length of data
     */
    public int getDataSize() {

	if (m_dataModule == null) {
	    System.err.println("SelectionSort.getDataSize(): Data Module is null!");
	    return 0;
	}

	Object o = m_dataModule.getData();
    if (!(o instanceof VisAlgVector)
        ||!((VisAlgVector)o).isComparable()) {
            reset();
            return 0;
        }
    VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();
	return data.length;
    }//getDataSize()

    /**
     * Returns the position of the current element.
     *
     * @return Position of the current element
     */
    public int getCurrent() {
	return m_current;
    }//getCurrent

    /**
     * Returns the position of the currently smallest element.
     *
     * @return Position of the smallest element
     */
    public int getSmallest() {
	return m_smallest;
    }//getSmallest

    /**
     * Returns the position, up to that the array is currently sorted.
     *
     * @return Position of the last sorted element
     */
    public int getSorted() {
	return m_sorted;
    }//getSorted()

    /**
     * Sets the dataModule the algorithm should work on.
     *
     * @param Reference to the DataModule
     */
    public void setDataModule(DataModule dm) {

    super.setDataModule(dm);

	// Input null
	if (dm == null){
	    System.err.println("SelectionSort.setDataModule(): Data Module is null!");
	    return;
	}

	// DataModule has no data
	Object o = dm.getData();
	if (o == null) {
	    System.err.println("SelectionSort.setDataModule(): Data Module has no data!");
	    return;
	}

	// Data must be of type VisAlgComparableData; otherwise return
	if ( !(o instanceof VisAlgVector)
        ||!((VisAlgVector)o).isComparable() ) {
	    System.err.println("SelectionSort.setDataModule(): Data is not of type VisAlgComparableData!");
	    return;
	}

	reset();

	m_dataModule = dm;
	m_done = false;

    registerProperties();

	fireEvent();
    } // setDataModule(DataModule)


    // ------- from class AbstractModule (resp. interface Module) ------

    /**
     * Checks, if the DataModule is null and calls reset() in this case.
     * Otherwise, the superclass method is called.
     */
    public void step() {

	// ??? Warum wird in diesem Fall reset() aufgerufen?
	if ( (m_dataModule == null) ) {
	    System.err.println("SelectionSort.step(): Data Module is null!");
	    reset();
	}

	super.step();
    }//step()


    // ----------- from Interface Module  ------------------

    /**
     * Creates a new SelectionSortWindow.
     *
     * @see SelectionSortWindow
     */
    public void newModuleWindow() {

	// Create a SelectionSortWindow where the user can select a data module.
	try {
	    m_moduleWindow =
		new SelectionSortWindow(m_name,
					this,
					m_moduleWindowBounds,
					m_moduleManager.getModuleWindowContainer());
	}
	catch (Exception e) {
	    System.err.println(e);
	}
    }//newModuleWindow()


    // -----------------------------------------------------
    // ------- M e t h o d s ( p r o t e c t e d ) ---------
    // -----------------------------------------------------

    /**
     * Checks, if the current element is the last one.
     *
     * @return True, if it is
     */
    protected boolean getCurrentAtEnd() {
	return m_currentAtEnd;
    }//getCurrentAtEnd()

    /**
     * Returns the result of the last comparison.
     *
     * @return The result
     */
    protected int getComparison() {
	return m_comparison;
    }//getComparison()

    /**
     * Sets the algorithm to "finished/not finished".
     *
     * @param b True, if the algorithm has finished
     */
    protected void setDone(boolean b) {
	super.setDone(b);
    }//setDone(b)

    /**
     * Sets the next command, that will be executed.
     *
     * @param ac The next command
     */
    protected void setNextCommand(Command ac) {
	super.setNextCommand(ac);
    }//setNextCommand(Command)


    // -----------------------------------------------------
    // ------- M e t h o d s ( p r i v a t e ) -------------
    // -----------------------------------------------------

    /**
     * Puts all variables into the initial state.
     */
    protected void reset() {

	m_sorted = -1;
	m_smallest = 0;
	m_current = 0;
	m_comparison = 0;
	m_currentAtEnd = false;
	setNextCommand(new IsCurrentAtEnd());
	m_done = false; // Warum done auf true???
    m_time = 0;
    fireEvent();
	// getDataSize();???
    }//reset()


    // -----------------------------------------------
    // ------- I n t e r n a l   c l a s s e s -------
    // -----------------------------------------------

    // Some general classes

    /**
     * Superclass for all "Pointer Comparison" commands, that should be derived
     * from this one. Sets toString() method to return "Pointer Comparison".
     */
    private class PointerComparison extends AbstractCommand {
	public String toString() {
	    return "Pointer Comparison";
	}
    }//class PointerComparison

    /**
     * Superclass for all "Pointer Setting" commands, that should be derived
     * from this one. Sets toString() method to return "SetPointer".
     */
    private class PointerSetting extends AbstractCommand {
	public String toString() {
	    return "Set Pointer";
	}
    }//class PointerSetting

    /**
     * Superclass for all "Value Comparison" commands, that should be derived
     * from this one. Sets toString() method to return "Value Comparison".
     */
    private class ValueComparison extends AbstractCommand {
	public String toString() {
	    return "Value Comparison";
	}
    }//class ValueComparison

    /**
     * Superclass for all "Value swapping" commands, that should be derived
     * from this one. Sets toString() method to return "Exchange of Values".
     *
     * @todo I think, its spelled "swapped" ;-))
     */
    private class SwopValues extends AbstractCommand {
	public String toString() {
	    return "Exchange of Values";
	}
    }//class SwopValues

    // Specialiced classes for the algorithm execution

    private class IsCurrentAtEnd extends PointerComparison {

	public void doIt() {

	    if (m_dataModule == null) {
		System.err.println("IsCurrentAtEnd.doIt(): Data Module is null!");
		reset();
		return;
	    }

	    Object o = m_dataModule.getData();
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector)o).isComparable()) {
                reset();
                return;
            }
        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

	    // Compare current position with maximal allowed
	    if (getCurrent() < data.length - 1) {
		m_currentAtEnd = false;
		setNextCommand(new IncreaseCurrent());
	    }
	    else {
		m_currentAtEnd = true;
		setNextCommand(new SwopSmallestAndSortedPlusOne());
	    }
	}

	public String toString() {
	    return "Pointer comparison: Current position and lenght of array.)";
	}

    } // class IsCurrentAtEnd

    private class IncreaseCurrent extends PointerSetting {

	public void doIt() {

	    m_current += 1;
	    setNextCommand(new CompareSmallestWithCurrent());
	}

	public String toString() {
	    return "Set pointer: Increment current.";
	}
    }

    private class CompareSmallestWithCurrent extends ValueComparison {

	public void doIt() {

	    if (m_dataModule == null) {
		System.err.println("CompareSmallestWithCurrent.doIt(): Data Module is null!");
		reset();
		return;
	    }

	    Object o = m_dataModule.getData();
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector)o).isComparable()) {
                reset();
                return;
            }
        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

	    m_comparison = data[getSmallest()].compareTo(data[getCurrent()]);

	    if (m_comparison < 0) {
		setNextCommand(new SetSmallestToCurrent());
	    }
	    else {
		setNextCommand(new IsCurrentAtEnd());
	    }
	}

	public String toString() {
	    return "Value Comparison: current lower than smallest.)";
	}

    }

    private class SetSmallestToCurrent extends PointerSetting {

	public void doIt() {

	    m_smallest = m_current;
	    setNextCommand(new IsCurrentAtEnd());
	}

	public String toString() {
	    return "Set pointer: Smallest to current.";
	}
    }

    private class SwopSmallestAndSortedPlusOne extends SwopValues {

	public void doIt() {

	    if (m_dataModule == null) {
		System.err.println("SwopSmallestAndSortedPlusOne.doIt(): Data Module is null!");
		reset();
		return;
	    }

	    Object o = m_dataModule.getData();
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector)o).isComparable()) {
                reset();
                return;
            }

        VisAlgVector v = (VisAlgVector) o;
        v.swapValues(getSorted()+1, getSmallest());

	    setNextCommand(new IncreaseSorted());

	}

	public String toString() {
	    return "Exchange of Values:  sorted+1 with smallest.";
	}

    }

    private class IncreaseSorted extends PointerSetting {

	public void doIt() {
	    m_sorted++;
	    setNextCommand(new IsSortedAtEnd());
	}

	public String toString() {
	    return "Set Pointer: Increment sorted";
	}

    }

    private class IsSortedAtEnd extends PointerComparison {

	public void doIt() {

	    if (m_dataModule == null) {
		System.err.println("IsSortedAtEnd.doIt(): Data Module is null!");
		reset();
		return;
	    }

	    Object o = m_dataModule.getData();
        if (!(o instanceof VisAlgVector)
            ||!((VisAlgVector)o).isComparable()) {
                reset();
                return;
            }
        VisAlgComparableData[] data = (VisAlgComparableData[]) ((VisAlgVector)o).toArray();

	    if (m_sorted == data.length - 1) {
		setNextCommand(null);
		setDone(true);
	    }
	    else {
		setNextCommand(new SetCurrentToSortedPlusOne());
	    }
	}

	public String toString() {
	    return "Pointer comparison: sorted with end of array.";
	}
    }

    private class SetCurrentToSortedPlusOne extends PointerSetting {

	public void doIt() {
	    m_current = m_sorted + 1;
	    setNextCommand(new SetSmallestToCurrent());
	}

	public String toString() {
	    return "Set pointer: current to sorted + 1.";
	}
    }

    public void registerProperties() {
        m_dataModule.addProperty(m_currentProperty);
        m_dataModule.addProperty(m_smallestProperty);
        m_dataModule.addProperty(m_sortedProperty);
    }

    public void setProperties() {
        m_sortedProperty.setIndicatorPosition(m_sorted);
        m_currentProperty.setIndicatorPosition(m_current);
        m_smallestProperty.setIndicatorPosition(m_smallest);
        if (m_dataModule!=null) {
            m_dataModule.propertyChanged();
        }
    }
    public void moduleChanged(ModuleChangeEvent evt) {
        reset();
        registerProperties();
        setProperties();
    }
} // class SelectionSort