////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : TuringMachine.java
//  Description      : Simulation of a Turing Machine
//  Author           : Heiner Violet
//  Email            : hviolet@ix.urz.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : 30.09.01
//  Last modified by :
//  Last modified on :
//  Update count     : V 1.0
//
///////////////////////////////////////////////////////////////////////////////
//
// Date          Name               Change/Extensions
// ----          ----               -----------------
//
////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import visalg.basics.Algorithm;
import visalg.basics.ModuleManager;
import visalg.basics.AbstractCommand;
import visalg.basics.Command;
import visalg.basics.DataModule;

import visalg.modules.TuringMachineWindow;

import visalg.types.VisAlgTuringMachineData;

import java.io.*;
import java.util.*;

/**
 * Main module for the simulation of a Turing Machine
 */
public class TuringMachine extends Algorithm {

    // -----------------------------------------------------
    // ------- V A R I A B L E S ---------------------------
    // -----------------------------------------------------

    /**
     * @serial state of the Turing Machine
     */
    private int m_state;

    /**
     * Reference to the DataModule.
     * @serial
     */
    private DataModule m_dataModule;

    /**
     * Object for a TuringMachine
     */
    private VisAlgTuringMachineData m_turingMachineData;

    /**
     * ArrayList for coded TM program
     */
    private ArrayList m_turingProgramCode;

    /**
     * Type of syntax of TM-Program
     */
    private int m_turingProgramSyntaxType;

    /**
     * Array for tape alphabet of the Turing machine
     */
    private char[] m_tapeAlphabet;

    /**
     * Length of the tape alphabet
     */
    private int m_tapeAlphabetLength;

    /**
     * Current state of the TM-program. This state controls the TM-program flow
     */
    private int m_turingProgramCurrentState;

    // -----------------------------------------------------
    // ------- M E T H O D S   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public TuringMachine(String s, ModuleManager mm) {

	super(s, mm);
	m_done = false;
        m_turingMachineData = null;
	m_commands = new AbstractCommand[] { new FindInitialState(),
                                                    new SingleMachineStep() };
	m_state = 1;

    } //Constructor

    /**
     * Creating a new Module Window
     */
    public void newModuleWindow() {
        try {
	    m_moduleWindow =
		new TuringMachineWindow("" + m_name,
					 this,
					 m_moduleWindowBounds,
					 m_moduleManager.getModuleWindowContainer());
	} catch (Exception e) {}
    } // newModulWindow


    /**
     * Setting the Data Module
     */
    public void setDataModule(DataModule dm) {
        int err, errType;
        String commentText;

        if (dm == null){
            System.err.println("Data Module is null!");
            return;
        }

        Object o = dm.getData();
        if (o == null) {
            System.err.println("Data Module has no data!");
            return;
        }

        if ( !(o instanceof VisAlgTuringMachineData) ) {
            System.err.println("Data is not of type VisAlgTuringMachineData!");
            return;
        }

        super.setDataModule(dm);

        m_dataModule = dm;
        m_turingMachineData = (VisAlgTuringMachineData) o;
        m_turingMachineData.resetMachine();

        m_turingProgramCode = m_turingMachineData.m_turingProgramCode;
        m_turingProgramSyntaxType = m_turingMachineData.m_turingProgramSyntaxType;
        m_tapeAlphabet = m_turingMachineData.m_tapeAlphabet;
        m_tapeAlphabetLength = m_turingMachineData.m_alphabetLength;

        setNextCommand(new FindInitialState());
        setDone(false);
        System.out.println(" OK");

        fireEvent();

    } // setDataModule

    /**
     * Returning a Data Module
     */
    public DataModule getDataModule() {
	return m_dataModule;
    } //getDataModule

  // ------- from class AbstractModule (resp. interface Module) ------

    /**
     * If the DataModule isn't null, then the method super.step() is called.
     */
    public void step() {
        if ( (m_dataModule == null) ) {
            System.err.println("Turingmachine.step(): Data Module is null!");
        }
        super.step();
    }//step()

    /*
     * Calls propertyChanged() in the data module.
     */
    public void setProperties() {
        if (m_dataModule!=null) {
          m_dataModule.propertyChanged();
        }
    }  // method setProperties()


    // -------------------------------------------------------
    // ------- M E T H O D S ( p r o t e c t e d ) -----------
    // -------------------------------------------------------

    /**
     *
     */
    protected void setNextCommand(Command ac) {
	super.setNextCommand(ac);
    }

    /**
     *
     */
    protected void setDone(boolean b) {
	super.setDone(b);
    }


    // -------------------------------------------------------
    // ------- M E T H O D S   ( p r i v a t e ) -------------
    // -------------------------------------------------------

    /**
     * Checks if the parameter 'state' is a terminating one
     */
    private boolean finalStateReached(int state, char character) {
        int[] command = new int[5];

        for(int i = 0; i < m_turingProgramCode.size(); ++i) {
            command = (int[])m_turingProgramCode.get(i);
            if (m_turingProgramSyntaxType == 1) {
                if (command[0] == state) {
                    return false;
                }
            }
            else if (m_turingProgramSyntaxType == 2) {
                if ((command[0] == state) && (command[1] == character)) {
                    return false;
                }
            }
        } // for loop: runs through the program code lines
        return true;

    } // finalStateReached

    /**
     * Executes one single step of the TM-program
     */
    private int nextTuringMachineStep() {

        int[] command = new int[5];

        char actualCharacter = m_turingMachineData.readFromTape();

        if (finalStateReached(m_turingProgramCurrentState, actualCharacter)) {
            return -1;
        }
        for(int i = 0; i < m_turingProgramCode.size(); ++i) {
            command = (int[])m_turingProgramCode.get(i);
            if((m_turingProgramSyntaxType == 1) &&
                                 (command[0] == m_turingProgramCurrentState)) {
                break; // current command found
            }
            else if ((m_turingProgramSyntaxType == 2) &&
                          (command[0] == m_turingProgramCurrentState) &&
                                            (actualCharacter == command[1]) ) {
                break; // current command found
            }
        }

    // evaluating the current command:
// m_turingProgramSyntaxType 1
        if (m_turingProgramSyntaxType == 1) {
            int nextPos = 4;   // position of the number telling the follow state
            if (command[1] == 1) {           // Move command
                if (command[2] == 'L') {       // one cell to the left
                    m_turingMachineData.moveHead(-1);
                }
                else if (command[2] == 'R') {  // one cell to the right
                    m_turingMachineData.moveHead(1);
                }
                else if(command[2] == 'S') {   // stand
                    m_turingMachineData.moveHead(0);
                }
            }
            else if(command[1] == 2) {        // Print command
                m_turingMachineData.writeToTape((char)command[2]);
            }
            else if(command[1] == 3) {           // Test
                if (actualCharacter == command[2]) {
                    nextPos = 4;
                }
                else {
                    nextPos = 3;
                }
            }
            m_turingProgramCurrentState = command[nextPos];
            m_turingMachineData.m_turingProgramCurrentState =
                                                  m_turingProgramCurrentState;
        }
// m_turingProgramSyntaxType 2
        else if(m_turingProgramSyntaxType == 2) {

            m_turingMachineData.writeToTape((char)command[2]);

            if (command[3] == 'L') {       // move one cell to the left
                m_turingMachineData.moveHead(-1);
            }
            else if (command[3] == 'R') {  // move one cell to the right
                m_turingMachineData.moveHead(1);
            }
            else if(command[3] == 'S') {   // stand
                m_turingMachineData.moveHead(0);
            }
            m_turingProgramCurrentState = command[4];
            m_turingMachineData.m_turingProgramCurrentState =
                                                   m_turingProgramCurrentState;
        }
        m_turingMachineData.m_turingProgramCurrentState =
                                                   m_turingProgramCurrentState;
        return 0;

    } // nextTuringMachineStep

    // -----------------------------------------------------
    // ------- I N T E R N A L   C L A S S E S -------------
    // -----------------------------------------------------

    /**
     * FindInitialState
     */
    private class FindInitialState extends AbstractCommand {

        public void doIt() {
            int[] command= new int[5];

            command = (int[])m_turingProgramCode.get(0);
            m_turingProgramCurrentState = command[0];
            for(int i = 0; i < m_turingProgramCode.size(); ++i) {
                command = (int[])m_turingProgramCode.get(i);
                if (command[0] < m_turingProgramCurrentState) {
                    m_turingProgramCurrentState = command[0];
                }
            }

            m_turingMachineData.m_turingProgramCurrentState =
                                                  m_turingProgramCurrentState;
            m_turingMachineData.m_turingProgramInitialError = 0;
            m_turingMachineData.m_turingProgramRunningError = 0;
            setNextCommand(new SingleMachineStep());
        }

        public String toString() {
          return "FindInitialState";
        }
    } // Inner class FindInitialState

    /**
     * Executes a single step of the virtual Turing Machine
     */
    private class SingleMachineStep extends AbstractCommand {

        public void doIt() {
            int err = nextTuringMachineStep();
            m_turingMachineData.m_turingProgramRunningError = err;
            if(err == -1) {
                System.out.println(" program terminated after " +
                         m_turingMachineData.m_numberOfTuringProgramSteps +
                                                                " steps, "+
                               m_turingMachineData.numberOfVisitedCells() +
                                      " cells visited." +  "\n\tOutput: " +
                                       m_turingMachineData.getOutput() + ".");
                setDone(true);
            }
            else {
                m_turingMachineData.increaseNumberOfSteps();
            }
            setNextCommand(new SingleMachineStep());
        }

        public String toString() {
            return "single Turing Machine step";
        }
    } // Inner class SingleMachineStep

    /**
     * Sets the module time to zero and calls setDataModule() to reset the
     * machine.
     */
    public void reset() {
        m_time = 0;

        m_turingMachineData.resetMachine();

        m_turingProgramCode = m_turingMachineData.m_turingProgramCode;
        m_turingProgramSyntaxType = m_turingMachineData.m_turingProgramSyntaxType;
        m_tapeAlphabet = m_turingMachineData.m_tapeAlphabet;
        m_tapeAlphabetLength = m_turingMachineData.m_alphabetLength;

        setNextCommand(new FindInitialState());
        setDone(false);
        System.out.println(" OK");

        fireEvent();
    }//reset()
} // class TuringMachine
