////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : TuringMachineViewerWindow.java
//  Description      : Viewer window for module TuringMachineViewer
//  Author           : Heiner Violet
//  Email            : hviolet@ix.urz.uni-heidelberg.de
//  Copyright        : Uni Heidelberg
//  Created on       : 30.09.01
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		Changes/Extensions
//  ----	    ----		------------------
///////////////////////////////////////////////////////////////////////////////


package visalg.modules;


import visalg.graphics.*;

import visalg.types.VisAlgComparableData;
import visalg.types.ModuleChangeEvent;
import visalg.types.IndicatorBarProperty;
import visalg.types.VisAlgTuringMachineData;
import visalg.basics.DataModule;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;

/**
 * This class is the front end of the TuringMachineViewer module.
 * @see visalg.modules.TuringMachineViewer.
 */
public class TuringMachineViewerWindow extends ModuleWindow {

    // -----------------------------------------------------------------------
    // ---------------- V A R I A B L E S ------------------------------------
    // -----------------------------------------------------------------------

    /**
     * Reference to the TuringMachineViewer
     */
    private TuringMachineViewer m_turingMachineViewer;

    /**
     * Reference to the Turing Machine Data
     */
    private VisAlgTuringMachineData m_turingMachineData;

    /**
     * The dimension of the window.
     */
    private transient Dimension m_dimension;

    /**
     * @serial Offset for border
     */
    private static int s_borderOffset = 10;      //oben und unten

    /**
     * @serial Window font
     */
    private static Font s_font = new Font("SansSerif", Font.PLAIN, 10);

    /**
     * Metrics of the window font
     */
    private transient FontMetrics m_fontMetrics;

    /**
     * Standard window color
     */
    private transient Color m_standardColor;

    /**
     * Height of the window panel
     */
    private transient int m_panelHeight;

    /**
     * Width of the window panel
     */
    private transient int m_panelWidth;

 // informations about and object for the viewer-object
     /**
      * Number of cells of the visible part of the tape
      */
     private int m_numberOfVisibleCells = 15;

     /**
      * Width of the tape in pixels
      */
     private int m_tapeWidth = 15;

     /**
      * Position of the most left visible cell
      */
     private int m_minTapeIndex;

     /**
      * The text containing the text of the Turing program
      */
     private String[] m_turingProgramText;

     /**
      * The Object representing the graphics of the tape of the TM
      */
     private TuringMachineTapeGraphic m_tapeGraphic;

     /**
      * Main Panel of the GUI
      */
     private JPanel m_GroundPanel;

     /**
      * Panel cantainig the text of the Turing program
      */
     private JPanel m_programTextPanel;

     /**
      * Panel containig the interactive elements of the GUI, i.e.
      * input fields, textFields, buttons
      */
     private JPanel m_controlPanel;

     /**
      * Panel containing the m_controlPanel
      */
     private JPanel m_controlGroundPanel;

     /**
      * Panel containing the input field (input for the Turing machine)
      */
     private JPanel m_turingMachineInputPanel;

     /**
      * Panel containing the output field (output of the Turing machine)
      */
     private JPanel m_turingMachineOutputPanel;

     /**
      * Panel containing the input field for the number of visible cells
      */
     private JPanel m_numberOfVisibleCellsPanel;

     /**
      * Panel containing the text field for the number steps of the TM
      */
     private JPanel m_numberOfStepsPanel;

     /**
      * Panel containing the text field for the number of visited cells
      */
     private JPanel m_numberOfVisitedCellsPanel;

     /**
      * Panel containing the "load input"- and "save output"-buttons
      */
     private JPanel m_buttonPanel1;

     /**
      * TextField for writing down the input for the Turing machine
      */
     private JTextField m_turingMachineInputField;

     /**
      * TextField where stands the output of the Turing machine
      */
     private JTextField m_turingMachineOutputField;

     /**
      * TextField for writing down the number of visible cells
      */
     private JTextField m_numberOfVisibleCellsField;

     /**
      * TextField where stands the number of steps
      */
     private JTextField m_numberOfStepsField;

     /**
      * TextField where stands the number of visited cells
      */
     private JTextField m_numberOfVisitedCellsField;

     /**
      * A JScrollPane that provides the area for the program text
      */
     private JScrollPane m_programTextPane;

     /**
      * JList containig the Turing program text
      */
     private JList m_programTextArea;

     /**
      * TextField where stands the comments about the Turing program
      */
     private JTextField m_turingProgramCommentTextArea;

     /**
      * Button calling "load input"
      */
     private JButton m_loadInputButton;

     /**
      * Button calling "save output"
      */
     private JButton m_saveOutputButton;

     /**
      * Button calling "save tape"
      */
     private JButton m_saveTapeButton;

     /**
      * Object for selecting files
      */
     private JFileChooser m_fileChooserDialog;

     /**
      * String for storing the last opened path
      */
     private String m_lastOpenedPath;

  // -----------------------------------------------------------------------
  // -------- M E T H O D S  ( p u b l i c ) -------------------------------
  // -----------------------------------------------------------------------

    /**
     * Initialisation
     * Builds up a UI for managing a Turing Machine.
     */
    public TuringMachineViewerWindow(String name, TuringMachineViewer av,
              Rectangle bounds,
              ModuleWindowContainer mwc) {
        super(name, av, bounds, mwc);
        m_turingMachineViewer = av;
        m_dimension = getSize(new Dimension());
        m_panelWidth = (int)m_dimension.getWidth();
        m_panelHeight = (int)m_dimension.getHeight();

        m_turingMachineData = getData();

        int nol = m_turingMachineData.m_turingProgramText.size();
        m_turingProgramText = new String[nol];
        for(int i = 0; i < nol; ++i) {
            m_turingProgramText[i] = (String)m_turingMachineData.getLine(i);
        }

        m_programTextPanel = new JPanel(new BorderLayout());
        m_programTextArea = new JList();
        m_programTextArea.setFont(new Font("Monospaced", Font.PLAIN,14));
        m_programTextArea.setListData(m_turingProgramText);

        m_programTextPane = new JScrollPane();
        m_programTextPane.getViewport().setView(m_programTextArea);

        m_turingProgramCommentTextArea = new JTextField(30);  // ehemals 30
        m_turingProgramCommentTextArea.setEditable(false);
        m_programTextPanel.add(m_programTextPane, BorderLayout.CENTER);
        m_programTextPanel.add(m_turingProgramCommentTextArea, BorderLayout.SOUTH);

        m_turingMachineInputPanel = new JPanel(new FlowLayout(2,0,0));
        m_turingMachineInputPanel.setBackground(Color.lightGray);
        m_turingMachineInputPanel.add(new JLabel("Input ", JLabel.CENTER));
        m_turingMachineInputPanel.add(m_turingMachineInputField = new JTextField(18));
        m_turingMachineInputField.setFont(new Font("Monospaced", Font.PLAIN,14));
        m_turingMachineInputField.addActionListener(new InputListener());

        m_turingMachineOutputPanel = new JPanel(new FlowLayout(2,0,0));
        m_turingMachineOutputPanel.setBackground(Color.lightGray);
        m_turingMachineOutputPanel.add(new JLabel("Output ", JLabel.CENTER));
        m_turingMachineOutputPanel.add(m_turingMachineOutputField = new JTextField(18));
        m_turingMachineOutputField.setFont(new Font("Monospaced", Font.PLAIN,14));
        m_turingMachineOutputField.setEditable(false);

        m_numberOfVisibleCellsPanel = new JPanel(new FlowLayout(2,0,0));
        m_numberOfVisibleCellsPanel.setBackground(Color.lightGray);
        m_numberOfVisibleCellsField = new JTextField(6);
        m_numberOfVisibleCellsField.setFont(new Font("Monospaced", Font.PLAIN,14));
        m_numberOfVisibleCellsField.setText("" + m_numberOfVisibleCells);
        m_numberOfVisibleCellsField.addActionListener(new NumberOfVisibleCellsListener());
        m_numberOfVisibleCellsPanel.add(new JLabel("Number of visible cells ", JLabel.CENTER));
        m_numberOfVisibleCellsPanel.add(m_numberOfVisibleCellsField);

        m_numberOfStepsPanel = new JPanel(new FlowLayout(2,0,0));
        m_numberOfStepsPanel.setBackground(Color.lightGray);
        m_numberOfStepsField = new JTextField(6);
        m_numberOfStepsField.setFont(new Font("Monospaced", Font.PLAIN,14));
        m_numberOfStepsField.setEditable(false);
        m_numberOfStepsPanel.add(new JLabel("Number of steps ", JLabel.CENTER));
        m_numberOfStepsPanel.add(m_numberOfStepsField);

        m_numberOfVisitedCellsPanel = new JPanel(new FlowLayout(2,0,0));
        m_numberOfVisitedCellsPanel.setBackground(Color.lightGray);
        m_numberOfVisitedCellsField = new JTextField(6);
        m_numberOfVisitedCellsField.setFont(new Font("Monospaced", Font.PLAIN,14));
        m_numberOfVisitedCellsField.setEditable(false);
        m_numberOfVisitedCellsPanel.add(new JLabel("Number of visited cells ", JLabel.CENTER));
        m_numberOfVisitedCellsPanel.add(m_numberOfVisitedCellsField);

        m_buttonPanel1 = new JPanel(new GridLayout(1,2));
        m_buttonPanel1.setFont(new Font("SansSerif", Font.PLAIN, 12));
        m_buttonPanel1.add(m_loadInputButton = new JButton("Load Input"));
        m_buttonPanel1.add(m_saveOutputButton = new JButton("Save Output"));
        m_loadInputButton.addActionListener(new ButtonListener("load Inp"));
        m_saveOutputButton.addActionListener(new ButtonListener("save Outp"));

        m_controlPanel = new JPanel(new GridLayout(0,1));
        m_controlPanel.setBackground(Color.lightGray);
        m_controlPanel.add(m_turingMachineInputPanel);
        m_controlPanel.add(m_turingMachineOutputPanel);
        m_controlPanel.add(m_numberOfVisibleCellsPanel);
        m_controlPanel.add(m_numberOfStepsPanel);
        m_controlPanel.add(m_numberOfVisitedCellsPanel);
        m_controlPanel.add(m_buttonPanel1);
        m_controlPanel.add(m_saveTapeButton = new JButton("Save Tape"));
        m_saveTapeButton.addActionListener(new ButtonListener("save Tape"));

        m_controlGroundPanel = new JPanel(new FlowLayout(1,0,0));
        m_controlGroundPanel.setBackground(Color.lightGray);
        m_controlGroundPanel.add(m_controlPanel);

        m_GroundPanel = new JPanel(new BorderLayout());
        m_GroundPanel.setBackground(Color.white);
        m_GroundPanel.add(m_tapeGraphic = new TuringMachineTapeGraphic(), BorderLayout.NORTH);
        m_GroundPanel.add(m_programTextPanel, BorderLayout.CENTER);
        m_GroundPanel.add(m_controlGroundPanel, BorderLayout.EAST);

        setBackground(Color.lightGray);

        setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        add(m_GroundPanel, BorderLayout.CENTER);

    }//TuringMachineViewerWindow

    /**
     * Paint method.
     * This method is called by the GUI if a repainting of the window is
     * necessary. It will read out the properties of the algorithm and do the
     * representation.
     * @param g Graphics object to paint in
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        m_dimension = getSize(m_dimension);
        m_panelHeight = m_dimension.height;
        m_panelWidth = m_dimension.width;

        if(m_turingMachineData.m_turingProgramInitialError == -1) {
            return;
        }
        int err = m_turingMachineData.m_turingProgramRunningError;
        programRunningErrorCheck(err);
        markActualLine();
        m_tapeGraphic.repaint();

    } //paintComponent

    /**
     * Module has changed.
     * This method is called by the module manager if the data module has bee
     * changed. It just repaints itself then.
     */
    public void moduleChanged(ModuleChangeEvent cEvt) {
        repaint();
    } //moduleChanged

    // -----------------------------------------------------------------------
    // -------- M E T H O D S  ( p r i v a t e ) -----------------------------
    // -----------------------------------------------------------------------

    /*
     * Checks if the Turing program has done an eror (while running)
     */
    private void programRunningErrorCheck(int err) {
        String commentText = "";

        if(m_turingMachineData.m_numberOfTuringProgramSteps == 0) {
            String inputString = m_turingMachineInputField.getText();
            m_turingMachineData.m_turingMachineInput = inputString;
            m_turingMachineData.inputToTape();
            m_turingMachineOutputField.setText("");
            m_turingProgramCommentTextArea.setText(" OK");
        }
        if(err == -1) {
            commentText =" program terminated after " +
               m_turingMachineData.m_numberOfTuringProgramSteps + " steps, " +
                     m_turingMachineData.numberOfVisitedCells() +
                                                            " cells visited.";
            m_turingProgramCommentTextArea.setText(commentText);
            m_turingMachineOutputField.setText(m_turingMachineData.getOutput());
        }
        m_numberOfStepsField.setText("" +
                            m_turingMachineData.m_numberOfTuringProgramSteps);
        m_numberOfVisitedCellsField.setText("" +
                                  m_turingMachineData.numberOfVisitedCells());
    } // method programRunningErrorCheck()

    /**
     * Marks the actual line in the TM program text with a row
     */
    private void markActualLine() {

        int state = m_turingMachineData.m_turingProgramCurrentState;
        int index = m_turingMachineData.numberOfLine(state);
        m_programTextArea.setSelectedIndex(index);
        m_programTextArea.ensureIndexIsVisible(index);

    } // markActualLine

    /**
     * Gets the actual Data from the Data Module
     */
    private VisAlgTuringMachineData getData() {
        Object o = m_turingMachineViewer.getDataModule().getData();
        return (VisAlgTuringMachineData) o;
    } // getData

    /**
     * Loads an input from a specified file
     */
    private void loadInput() {

        String inputFile = getFileString(1);
        String line = "";
        String inputString = "";

        if(inputFile != null) {
            try {
                FileReader fr = new FileReader(inputFile);
                BufferedReader br = new BufferedReader(fr);
                while((line = br.readLine()) != null) {  // While-loop:
                    inputString = inputString + line;    // reads the data from
                }                                        // the Inputfile
            }
            catch(Exception e) {
                System.out.println("Error while reading input");
            }
            m_turingMachineData.m_turingMachineInput = inputString;
            m_turingMachineData.inputToTape();
            m_tapeGraphic.repaint();
            m_turingMachineInputField.setText(inputString);
       }

    } // loadInput

    /**
     * Saves the output to a specified file
     */
    private void saveOutput() {

        String outputFile;                   // String for Outputfile
        String s;                            // String for TM-output
        PrintWriter pw = null;

        outputFile = getFileString(2);
        if(outputFile == null) {
            return;
        }
        s = m_turingMachineData.getOutput();
        try {
            pw = new PrintWriter(new FileWriter(outputFile));
            pw.println(s);
        }
        catch(Exception e) {
            m_turingProgramCommentTextArea.setText(" error while saving output");
        }
        finally {
            if(pw != null) {
                pw.close();
            }
        }
    } // saveOutput

    /**
     * Saves the whole tape to a specified file
     */
    private void saveTape() {

        String outputFile;                   // String for Outputfile
        String s;                            // String for TM-output
        PrintWriter pw = null;

        outputFile = getFileString(2);
        if(outputFile == null) {
            return;
        }
        s = m_turingMachineData.getTape();
        try {
            pw = new PrintWriter(new FileWriter(outputFile));
            pw.println(s);
        }
        catch(Exception e) {
            m_turingProgramCommentTextArea.setText(" Error while saving tape");
        }
        finally {
            if(pw != null) {
                pw.close();
            }
        }
    } // saveTape

    /**
     * Creates a FileDialog-Object that one can choose a file
     */
    private String getFileString(int ind) {

        String[] title = {"Load Program","Load Input","Save Output"};
        String fileName;
        int returnValue = 0;

        m_fileChooserDialog = new JFileChooser(m_lastOpenedPath);
        if(ind != 2) {
            returnValue = m_fileChooserDialog.showDialog(null, null);
        }
        else {
            returnValue = m_fileChooserDialog.showSaveDialog(null);
        }
        if(returnValue == 0) {
            fileName = m_fileChooserDialog.getSelectedFile().getAbsolutePath();
            m_lastOpenedPath = m_fileChooserDialog.getSelectedFile().getParent();
            return fileName;
        }
        else {
            return null;
        }

    } // getFileString

    // ---------------------------------------------------------------------
    // ----- I N T E R N A L  C L A S S E S --------------------------------
    // ---------------------------------------------------------------------


    /**
     * This class implements the Graphics for the tape of the Turing machine
     */
    class TuringMachineTapeGraphic extends JComponent {

        double cellWidth;  // Width of one cell of the tape
        double cellHeight;
        double x;          // x-coordinate of a cell
        double textX;      // Integers textX and textY for text-coordinates
        double textY;      //
        int ind;
        int thickness;  // thickness of the tape-Head in pixels
        int r;          // These three Integers are used for
        int signInd;    // the adjustation of the cell-position
        int d;          // specifiers under the tape
        char s;         // char for one symbol in a tape-cell
        int textSize = 20;
        int textWidth = (2*textSize)/3;

        public Dimension getPreferredSize() {
          return new Dimension(610,100);
        }

        public void paint(Graphics g) {

            // drawing the tape:

            int diffX = 5;

            m_panelWidth = m_dimension.width;
            m_tapeWidth = m_panelWidth - 40;
            cellWidth = (double)m_tapeWidth/(double)(m_numberOfVisibleCells+1);
            cellHeight = 40;
            if(cellWidth < 40) {
                cellHeight = cellWidth;
                textSize = (int)(cellWidth*2)/3;
                textWidth = (2*textSize)/3;
            }

            g.setColor(Color.black);
            g.drawLine(10,25,(int)(m_tapeWidth+10),25);   // drawing the tape
            g.drawLine(10,(int)(25+cellHeight),(int)(m_tapeWidth+10),(int)(25+cellHeight));

            // filling the tape:

            m_minTapeIndex = m_turingMachineData.m_headPosition - m_numberOfVisibleCells/2;
            g.setFont(new Font("SansSerif", Font.PLAIN, textSize));
            textY = 25 + cellHeight - cellHeight/4;
            for(int i = 0; i <= m_numberOfVisibleCells; ++i) {
                x = 10 + cellWidth/2 + i*cellWidth;
                textX = 11 + cellWidth - textWidth/2 + i*cellWidth;
                ind = m_minTapeIndex + i;
                s = m_turingMachineData.readFromTape(ind);
                g.drawLine((int)x,25,(int)x,(int)(25+cellHeight));  // draw vert. line
                if(s != '#') {                     // use different colors
                    g.setColor(new Color(60,10,255));  // for non-'#'s
                }
                if(i<m_numberOfVisibleCells) {
                    g.drawString(s+"",(int)textX,(int)textY); // draw cell-content
                }
                g.setColor(Color.black);
            }

            // drawing the Head of the TM:

            thickness = textSize/5;
            g.setColor(Color.red);
            x = (int)(10 + cellWidth/2 + cellWidth* (m_numberOfVisibleCells/2));
            for(int i = 0; i < thickness; ++i) {
                g.drawRoundRect((int)(x-i),25-i,(int)(cellWidth+2*i),(int)(cellHeight+2*i),i,i);
            }
            g.setColor(Color.black);
            g.setFont(new Font("SansSerif", Font.PLAIN, (textSize*2)/3));
            ind = m_turingMachineData.m_turingProgramCurrentState;
            d = (int)(cellWidth/2)-textSize/3;
            g.drawString(ind+"", (int)(x+d), 20);


            // drawing the numbers of the cells under the tape:

            for(int i = 0; i < m_numberOfVisibleCells; ++i) {
                ind = i + m_minTapeIndex;
                r = 1;
                if(ind != 0) {
                    r = ind/Math.abs(ind);  // r=1: ind>=0, r=-1: ind<0
                }
                signInd = (r - Math.abs(r))/2;
                d = (int)(textWidth/3 - signInd*textWidth/3);
                textX = 10 + cellWidth - d + i*cellWidth;
                g.drawString(ind+"",(int)textX,(int)(23+cellHeight)+textSize);
            }
        }

    } // inner class TuringMachineTapeGraphic

    /**
     * Implements an ActionListener interface for managing
     * the button events
     */
    class ButtonListener implements ActionListener {

        private String s;

        ButtonListener(String s) {
            this.s = s;
        }
        public void actionPerformed(ActionEvent e) {
            if(s.equals("load Inp")) {
                loadInput();
            }
            if(s.equals("save Outp")) {
                saveOutput();
            }
            if(s.equals("save Tape")) {
                saveTape();
            }

        }

    } // inner class ButtonListener

    /**
     * Implements an ActionListener interface for getting the
     * input for the Turing machine from the Textfield
     * 'm_turingMachineInputField'
     */
    class InputListener implements ActionListener {

        int err;
        String inputString;

        public void actionPerformed(ActionEvent e) {
            inputString = m_turingMachineInputField.getText();
            m_turingMachineData.m_turingMachineInput = inputString;
            m_turingMachineData.inputToTape();
            m_tapeGraphic.repaint();
        }

    } // inner class InputListener

    /**
     * Implements an ActionListener interface for getting the
     * input from the TextField 'm_numberOfVisibleCellsField'
     */
    class NumberOfVisibleCellsListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            int newNumber = 0;
            int oldNumber = m_numberOfVisibleCells;
            try {
                newNumber = Integer.parseInt(m_numberOfVisibleCellsField.getText());
            }
            catch(Exception exc) {
                m_numberOfVisibleCellsField.setText("" + oldNumber);
            }
            if((newNumber < 3) || (newNumber > 200)) {
                m_numberOfVisibleCellsField.setText("" + oldNumber);
                return;
            }
            m_numberOfVisibleCells = newNumber;
            if(newNumber > oldNumber) {
                int number = newNumber/m_turingMachineData.m_tapeBlockSize + 1;
                for(int i = 0; i < number; ++i) {
                    m_turingMachineData.expandMemoryTape();
                }
            }
            m_turingMachineData.m_numberOfVisibleCells = newNumber;
            m_tapeGraphic.repaint();
        }
    } // inner class NumberListener

} //class TuringMachineViewerWindow