////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : IndicatorBarProperty.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Thu May 03 14:42:20 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
package visalg.types;

import java.awt.Color;

/**
 * This class represents an indicator bar, that can be used to visualise the
 * values of e.g. pointers etc.
 *
 * An indicator bar is specified by its name, its position and its color, all
 * of which are initially set up at the constructor and can be changed at
 * runtime by using the appropriate methods.
 *
 * IndicatorBarProperty is recognised by the Array viewer.
 *
 * @see visalg.modules.ArrayViewer
 */
public class IndicatorBarProperty extends PropertyObject {

    /**
     * All IndicatorBarProperties own this name.
     */
    public static String INDICATOR_BAR_PROPERTY="Indicator Bar Property";

    /**
     * The name of the IndicatorBar.
     */
    private String m_indicatorname;

    /**
     * The Color of the IndicatorBar.
     */
    private Color m_color;

    /**
     * The position of the IndicatorBar.
     */
    private int m_position;

    /**
     * The relative position of the IndicatorBar.
     */
    private int m_relPos;

    /**
     * Sets up the initial values of the IndicatorBarObject. The name of this
     * object is automatically set to INDICATOR_BAR_PROPERTY in order to make
     * it available to QueryPropertyObjects.
     * To avoid overlapping bars, every indicator has a relative position, like
     * -1, 1, ... which determines the offset, this bar is painted relative to
     * the actual position.
     * Do not confuse the propertys name with the indicator bars name.
     *
     * @param indicatorname The name of the bar
     * @param color The color of the bar
     * @param position The position of the bar
     * @param relPos The relative position of the bar
     *
     * @SEE visalg.basics.DataModule.QueryPropertyObjects(String)
     */
    public IndicatorBarProperty(String indicatorname, Color color, int position, int relPos)
    {
        setPropertyName(INDICATOR_BAR_PROPERTY);

        m_indicatorname = indicatorname;

        m_color = color;

        m_position = position;

        m_relPos = relPos;
    }//IndicatorBarProperty(String, Color, int)

    /**
     * Returns the name of the indicator bar.
     *
     * @return The name of the bar
     */
    public String getIndicatorName()
    {
        return m_indicatorname;
    }//getIndicatorName

    /**
     * Sets the name of the indicator bar.
     *
     * @param name The name of the bar
     */
    public void setIndicatorName(String name)
    {
        m_indicatorname = name;
    }//setIndicatorName(String)

    /**
     * Returns the color of the indicator bar.
     *
     * @return The color of the bar
     */
    public Color getIndicatorColor()
    {
        return m_color;
    }//getIndicatorColor()

    /**
     * Sets the color of the indicator bar.
     *
     * @param color The color of the bar
     */
    public void setIndicatorColor(Color color)
    {
        m_color = color;
    }//setIndicatorColor(Color)

    /**
     * Returns the position of the indicator bar.
     *
     * @return The position of the bar
     */
    public int getIndicatorPosition()
    {
        return m_position;
    }//getIndicatorPosition()

    /**
     * Returns the relative position of the indicator bar.
     *
     * @return The relative position
     */
    public int getRelativePosition() {
        return m_relPos;
    }

    /**
     * Sets the relative position of the indicator bar.
     *
     * @param relPos The new relative position
     */
    public void setRelativePosition(int relPos) {
        m_relPos = relPos;
    }

    /**
     * Sets the position of the indicator bar.
     *
     * @param position The position of the bar.
     */
    public void setIndicatorPosition(int position)
    {
        m_position = position;
    }//setIndicatorPosition
}//class IndicatorBarProperty