////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgGraph.java
//  Description      :
//  Author           : Bonato, Braun, Schoebel
//  Email            :
//  Copyright        : University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on :
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date        Name            Changes/Extensions
//  ----        ----            ------------------
//
////////////////////////////////////////////////////////////////////////////////


package visalg.types.VisAlgGraph;


import visalg.basics.ModuleManager;
import visalg.basics.DataModule;
import visalg.modules.TSPGraphViewer;
import visalg.types.VisAlgGraph.*;
import visalg.types.VisAlgData;

import java.awt.Graphics;
import java.io.File;
import java.util.*;

import javax.swing.JComponent;
import javax.swing.JPanel;

/**
 * This class defines the VisAlgGraph- Objects used in this project.
 */
public class VisAlgGraph implements VisAlgData {


    // -----------------------------------------------------
    // ------- V a r i a b l e s ---------------------------
    // -----------------------------------------------------

    /**
      *
      */
    private Object[] m_nodes = null;
    private Object[] m_edges = null;
    private Object m_intervals = new Object();
    private int m_currentNode = -1;
    private int m_diffToX; // mouse difference to node- x- coordinate
    private int m_diffToY; // mouse difference to node- y- coordinate
    private int m_maxAbsoluteX = 0;
    private int m_minAbsoluteX = 0;
    private int m_maxAbsoluteY = 0;
    private int m_minAbsoluteY = 0;
    private boolean m_showingNames = true;
    private boolean m_mousePressed = false;
    private boolean m_leftOpen = true;
    private boolean m_rightOpen = true;
    private File m_tspgFile;
    private File m_tspFile;
    private File m_texFile;


    // -----------------------------------------------------
    // ------- M e t h o d s   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
    * constructor:
    */
    public VisAlgGraph(){
    }

    public VisAlgGraph(Object[] n, Object[] e){
        m_nodes = n;
        m_edges = e;
    }//end public VisAlgGraph(Object[] n, Object[] e)

    /**
     * setVisAlgNodes(Object[]) expects the Object[], where the
     * nodes are stored. See also VisAlgNode.
     * @param n   Object[] containing the VisAlgNodes
     */
    public void setVisAlgNodes(Object[] n) {
        m_nodes = n;
    }//end public void setVisAlgNodes(Object[] n)

    /**
     * getVisAlgNodes(); returns the Object[] m_nodes, that contains all nodes.
     * See also VisAlgNode.
     *
     * @return m_nodes VisAlgNode[] within VisAlgGraph- Object
     */
    public Object[] getVisAlgNodes() {
        return m_nodes;
    }//end public Object[] getVisAlgNodes()

    /**
     * setVisAlgInterval(Object) expects the Object, where the intervals are
     * stored. See also VisAlgInterval.
     * @param i   Object containing the VisAlgInterval[]
     */
    public void setVisAlgIntervals(Object i) {
        m_intervals = i;
        return;
    }//end public void setVisAlgIntervals(Object i)

    /**
     * getVisAlgIntervals(); returns the Object m_intervals, that contains the
     * object stored with setVisAlgIntervals(Object).
     * See also VisAlgInterval.
     *
     * @return m_intervals   Object, that only should contain VisAlgInterval[]
     */
    public Object getVisAlgIntervals() {
        return m_intervals;
    }//end public Object getVisAlgIntervals()

    /**
     * This function returns the node with the given ID.
     * @param id   ID
     *
     * @return VisAlgNode- Object if it worked,
     *         null Wrong instance.
     */
    public VisAlgNode getNodeAt(int id) {
        if (m_nodes[id] instanceof VisAlgNode) {
            return (VisAlgNode) m_nodes[id];
        }
        else {
            return null;
        }
    }//end public VisAlgNode getNodeAt(int id)

    /**
     * This function returns the edge with the given ID.
     * @param id   ID
     *
     * @return VisAlgEdge- Object if it worked,
     *         null Wrong instance.
     */
    public VisAlgEdge getEdgeAt(int id) {
        if (m_edges[id] instanceof VisAlgEdge) {
            return (VisAlgEdge) m_edges[id];
        }
        else {
            return null;
        }
    }//end public VisAlgNode getNodeAt(int id)

    /**
     * This function returns the interval with the given ID.
     * @param id   ID
     *
     * @return VisAlgInterval- Object if it worked,
     *         null Wrong instance.
     */
    public VisAlgInterval getIntervalAt(int id) {
        if (m_intervals instanceof VisAlgInterval[]) {
            VisAlgInterval[] vai = (VisAlgInterval[]) m_intervals;
            return vai[id];
        }//end if (m_intervals instanceof VisAlgInterval[])

        else if (m_intervals instanceof VisAlgInterval) {
            return (VisAlgInterval) m_intervals;
        }//end else if (m_intervals instanceof VisAlgInterval)

        return null;
    }//end public VisAlgInterval getIntervalAt(int id)

    /**
     * setVisAlgEdges(Object[]) expects the Object[], where the
     * Edges are stored. See also VisAlgEdge.
     * @param e   Object[] containing the VisAlgEdges
     */
    public void setVisAlgEdges(Object[] e) {
        m_edges = e;
    }//end public void setVisAlgEdges(Object[] e)

    /**
     * getVisAlgEdges(); returns the Object[] m_edges, that contains all edges.
     * See also VisAlgEdge.
     *
     * @return m_edges VisAlgEdge[] within VisAlgGraph- Object
     */
    public Object[] getVisAlgEdges() {
        return m_edges;
    }//end public Object[] getVisAlgEdges()

    /**
     * numberOfNodes(); returns the number of elements contained in the
     * m_nodes- Object[].
     *
     * @return m_nodes.length Number of nodes within the m_nodes[]
     */
    public int numberOfNodes() {
        if (m_nodes == null) {
            return -1;
        }
        return m_nodes.length;
    }//end public int numberOfNodes()

    /**
     * numberOfEdges(); returns the number of elements contained in the
     * m_edges- Object[].
     *
     * @return m_edges.length Number of edges within the m_edges[]
     */
    public int numberOfEdges() {
        if (m_edges == null) {
            return -1;
        }
        return m_edges.length;
    }//end public int numberOfEdges()

    /**
     * numberOfIntervals(); returns the number of elements contained in the
     * m_intervals- Object.
     *
     * @return length Number of nodes within the m_intervals- Object.
     */
    public int numberOfIntervals() {
        if (m_intervals instanceof VisAlgInterval[]) {
            VisAlgInterval[] vai = (VisAlgInterval[]) m_intervals;
            return vai.length;
        }//end if (m_intervals instanceof VisAlgInterval[])

        else if (m_intervals instanceof VisAlgInterval) {
            return 1;
        }//end else if (m_intervals instanceof VisAlgInterval)

        return 0;
    }//end public int numberOfIntervals()

    /**
     * This function sets the value of the m_mousePressed- variable.
     * @param m   Mouse pressed or not
     */
    public void setMousePressed(boolean m) {
        m_mousePressed = m;
    }//end public void setMousePressed(boolean m)

    /**
     * This function returns the value of the m_mousePressed- variable.
     *
     * @return m_mousePressed
     */
    public boolean isMousePressed() {
        return m_mousePressed;
    }//end public boolean isMousePressed()

    /**
     * This function returns the value of the m_showingNames- variable.
     *
     * @return m_showingNames
     */
    public boolean isShowing() {
        return m_showingNames;
    }//end public boolean isShowing()

    /**
     * This function sets the value of the m_showingNames- variable.
     * @param s   true for showing the names or false otherwise
     */
    public void setShowing(boolean s) {
        m_showingNames = s;
    }// public void setShowing(boolean s)

    /**
     * This function sets the value of the m_leftOpen- variable.
     * @param m   the intervals are left open
     */
    public void setLeftOpen(boolean lo) {
        m_leftOpen = lo;
    }// public void setMousePressed(boolean m)

    /**
     * This function returns the value of the m_leftOpen- variable.
     *
     * @return m_leftOpen
     */
    public boolean isLeftOpen() {
        return m_leftOpen;
    }// public boolean isMousePressed()

    /**
     * This function sets the value of the m_rightOpen- variable.
     * @param m   the intervals are right open
     */
    public void setRightOpen(boolean ro) {
        m_rightOpen = ro;
    }// public void setMousePressed(boolean m)

    /**
     * This function returns the value of the m_rightOpen- variable.
     *
     * @return m_rightOpen
     */
    public boolean isRightOpen() {
        return m_rightOpen;
    }// public boolean isMousePressed()

    /**
     * This function sets the value of the m_diffToX- variable.
     * @param dx   Horizontal distance between cursor and node
     */
    public void setMouseDifferenceToNodeX(int dx) {
        m_diffToX = dx;
    }// public void setMouseDifferenceToNodeX(int dx)

    /**
     * This function returns the value of the m_diffToX- variable.
     *
     * @return m_diffToX
     */
    public int getMouseDifferenceToNodeX() {
        return m_diffToX;
    }// public int getMouseDifferenceToNodeX()

    /**
     * This function sets the value of the m_diffToY- variable.
     * @param dy   Vertical distance between cursor and node
     */
    public void setMouseDifferenceToNodeY(int dy) {
        m_diffToY = dy;
    }// public void setMouseDifferenceToNodeY(int dy)

    /**
     * This function returns the value of the m_diffToY- variable.
     *
     * @return m_diffToY
     */
    public int getMouseDifferenceToNodeY() {
        return m_diffToY;
    }// public int getMouseDifferenceToNodeY()

    /**
     * This function sets the value of the m_currentNode- variable.
     * @param c   Current node.
     */
    public void setCurrentNode(int c) {
        m_currentNode = c;
    }// public void setCurrentNode(int c)

    /**
     * This function returns the value of the m_currentNode- variable.
     *
     * @return m_currentNode
     */
    public int getCurrentNode() {
        return m_currentNode;
    }// public int getCurrentNode()

    /**
     * Stores the file of the given TSPLIB- file.
     * @param f   the TSPLIB- file to store
     */
     public void setTSPLibFile(File f) {
        m_tspFile = f;
     } // public File setTSPLibFile()

    /**
     * Returns the stored TSPLIB- file.
     *
     * @return m_tspFile   TSPLIB- file
     */
     public File getTSPLibFile() {
        return m_tspFile;
     } // public File getTSPLibFile()

    /**
     * Returns the stored TSPGraph- file.
     *
     * @return m_tspFile   TSPGraph- file
     */
     public File getTSPGraphFile() {
        return m_tspgFile;
     } // public File getTSPGraphFile()

    /**
     * Stores the file of the given TSPGraph- file.
     * @param f   the TSPGraph- file to store
     */
     public void setTSPGraphFile(File f) {
        m_tspgFile = f;
     } // public File setTSPGraphFile()

    /**
     * Returns the stored Tex- output- file.
     *
     * @return m_texFile   Tex- output- file
     */
     public File getTexFile() {
        return m_texFile;
     } // public File getTexFile()

    /**
     * Stores the file of the given Tex- output- file.
     * @param f   the Tex- output- file to store
     */
     public void setTexFile(File f) {
        m_texFile = f;
     } // public File setTexFile()

    /**
     * This function returns the maximum  and minimum of all x/y- coordinates.
     * @return int[]   xy[0] = minX, xy[1] = maxX, xy[2] = minY, xy[3] = maxY
    */
    public int[] getMinMaxXY() {
        int[] v = {0, 0, 0, 0};

        v[1] = m_maxAbsoluteX;
        v[0] = m_minAbsoluteX;
        v[3] = m_maxAbsoluteY;
        v[2] = m_minAbsoluteY;

        return v;
    } // public void setMinMaxXY(int[] v)

    /**
     * This function sets the maximum  and minimum of all x/y- coordinates.
     * @param v[0]    smallest x- coordinate
     * @param v[1]    highest  x- coordinate
     * @param v[2]    smallest y- coordinate
     * @param v[3]    highest  y- coordinate
     */
    public void setMinMaxXY(int[] v) {

        if (v.length != 4) {
            System.err.println("Error in setMax(int[] v): four integers required.");
            return;
        }
        m_maxAbsoluteX = v[1];
        m_minAbsoluteX = v[0];
        m_maxAbsoluteY = v[3];
        m_minAbsoluteY = v[2];
    }

    /**
     * This function determines the coordinates, that are needed to draw the
     * nodes.
     * @param width    Frame- width
     * @param height   Frame- height
     */
    public void determineDrawCoordinates(int width, int height) {
        VisAlgNode van;

        for (int i = 0; i < m_nodes.length; i++) {

             //Calculate the X-Coordinate for drawing

            if (m_nodes[i] instanceof VisAlgNode){
                van = (VisAlgNode) m_nodes[i];
            }//end if (m_nodes.get(i) instanceof VisAlgNode)
            else {
                return;
            }//end else

            if (m_maxAbsoluteX == m_minAbsoluteX) {
               van.setXForDrawing( (int) (width / 2));
            }//end if (m_maxAbsoluteX == m_minAbsoluteX)
            else {
                van.setXForDrawing(
                    (int) (
                        (van.getAbsoluteX() - m_minAbsoluteX)
                        * (width - 40)
                        / (m_maxAbsoluteX - m_minAbsoluteX) + 20
                    )
                );
            }//end else

             //Calculate the Y-Coordinate for drawing

            if (m_maxAbsoluteY == m_minAbsoluteY) {
                van.setYForDrawing( (int) (height / 2));
            }//end if (m_maxAbsoluteY == m_minAbsoluteY)
            else {
                van.setYForDrawing(
                    (int) (
                        (van.getAbsoluteY() - m_minAbsoluteY)
                        * (height - 60)
                        / (m_maxAbsoluteY - m_minAbsoluteY) + 20
                    )
                );
            } // else
        } // for
    }//end public void determineXForDrawingOfAllNodesOnSize(int width,
     //                                                     int height)

    /**
     * This function uses the node- ID to find the belonging node- Object. Then
     * it sets the appropriate instance within the edge- Object.
     */
    public void setNodeToEdges() {
        int temporary = numberOfEdges();
        VisAlgNode van;
        VisAlgEdge vae;

        if (temporary == -1) {
            return;
        }//end if (temporary == -1)

        for (int i = 0; temporary > i; i++) {
            if (m_edges[i] instanceof VisAlgEdge) {
                vae = (VisAlgEdge) m_edges[i];

                if (m_nodes[vae.getStartID()] instanceof VisAlgNode) {
                    vae.setStart( (VisAlgNode) m_nodes[vae.getStartID()] );
                }//end if (m_nodes[vae.getStartID()] instanceof VisAlgNode)

                if (m_nodes[vae.getEndID()] instanceof VisAlgNode) {
                    vae.setEnd( (VisAlgNode) m_nodes[vae.getEndID()] );
                }//end if (m_nodes[vae.getEndID()] instanceof VisAlgNode)
            }//end if (m_edges[i] instanceof VisAlgEdge)
            else {
                return;
            }//end else
        }//end for (int i = 0; temporary > i; i++)
    }//end public void setNodeToEdges()

    /**
     * This function calculates the Absolute X,Y-coordinates. Given are mouse-
     * coordinates and the size.
     * @param drawX   node- x- coordinate for drawing
     * @param drawY   node- y- coordinate for drawing
     * @param width   panel- width
     * @param height  panel- height
     */
    public void determineAbsoluteXYOnSize(int drawX, int drawY, int width,
                                          int height) {

        //from mouseX to absoluteX
        //(width - 40) is never 0, because minimum Windows Size = 114
        // this is VisAlg based

        VisAlgNode van;

        if (m_nodes[m_currentNode] instanceof VisAlgNode) {
            van = (VisAlgNode) m_nodes[m_currentNode];
        }//end if (m_nodes.get(i) instanceof VisAlgNode)
        else {
            return;
        }//end else

        if ((width - 40) !=  0) {
            van.setAbsoluteX(
                (int) (
                    (drawX - 20)
                    * (m_maxAbsoluteX - m_minAbsoluteX)
                    / (width - 40) + m_minAbsoluteX
                )
            );
        } // if ((width - 40) !=  0)

        //from mouseY to absoluteY

        if ((height - 60) !=  0) {
            van.setAbsoluteY(
                (int) (
                    (drawY - 20)
                    * (m_maxAbsoluteY - m_minAbsoluteY)
                    / (height - 60) + m_minAbsoluteY
                )
            );
        } // if ((height - 60) !=  0)
    }//end public void determineAbsoluteXYOnSize(int drawX, int drawY,
     //                                          int width, int height)

    /**
     * This function looks for an existing node at the given mouse- position.
     * @param x   mouse- x- coordinate
     * @param y   mouse- y- coordinate
     *
     * @return  i Node- ID if node found,
     *         -1 No node found,
     *         -2 Wrong instance.
     */
    public int nodeFoundAtXY(int x,int y) {
        VisAlgNode van;

        for (int i = 0; i < m_nodes.length; i++) {
            if (m_nodes[i] instanceof VisAlgNode) {
                van = (VisAlgNode) m_nodes[i];
            }//end if (m_nodes.get(i) instanceof VisAlgNode)
            else {
                return -2;
            }//end else

            if (    (x >= (van.getXForDrawing() - van.getWidth() / 2))
                    && (x <= (van.getXForDrawing() + van.getWidth() / 2))
                    && (y >= (van.getYForDrawing() - van.getHeight() + 7))
                    && (y <= (van.getYForDrawing() + 7 )) ) {
                m_currentNode = i;
                return i;
            } // if for finding nodes
        }//end for (int i = 0; i <= getNumberOfNodes() - 1; i++)
        m_currentNode = -1;
        return -1;
    }//end public int nodeFoundAtXY(int x,int y)

    /**
     * This function determines the maximal node- ID.
     *
     * @return id Node-ID,
     *         -1 Wrong instance.
     */
    public int maxNodeID() {
        int id = -1;
        int temporary = m_edges.length;
        VisAlgEdge vae;

        for (int j = 0; temporary > j; j++) {
            if (m_edges[j] instanceof VisAlgEdge) {
                vae = (VisAlgEdge) m_edges[j];
                id = (vae.getStartID() > id) ? vae.getStartID() : id;
                id = (vae.getEndID() > id) ? vae.getEndID() : id;
            }//end if (m_edges[j] instanceof VisAlgEdge)
            else {
                return -1;
            }//end else
        }//end for (int j = 0; temporary > j; j++)
        return id;
    }//end public int maxNodeID()


    // ------- from interface VisAlgData ------------------

    /**
     * Dud- method
     */
    public JComponent getComponent() {
        return new JPanel();
    }

    /**
     * Dud- method
     */
    public void paintComponent(Graphics g, int x, int y) {}

    /**
     * The data must create all viewer module objects and register them at the
     * associated ModuleManager.
     *
     * @param mm The ModuleManager
     * @param dm The associated DataModule
     */
    public void registerViewerModules(ModuleManager mm, DataModule dm) {
        mm.addModule(new TSPGraphViewer(mm, dm));
    } // public void registerViewerModules(ModuleManager mm, DataModule dm)

    /**
     * Does nothing.
     */
    public void printData() {}

    /**
     * This data is not cloneable up to now.
     */
    public VisAlgData cloneData() {
        return null;
    }
}//end public class VisAlgGraph