////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgTuringMachineData.java
//  Description      : VisAlg Datatype representing a Turing machine
//  Author           : Heiner Violet
//  Email            : hviolet@ix.urz.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : 30.09.01
//  Last modified by :
//  Last modified on :
//  Update count     : V 1.0
//
///////////////////////////////////////////////////////////////////////////////
//
// Date          Name               Change/Extensions
// ----          ----               -----------------
//
////////////////////////////////////////////////////////////////////////////



package visalg.types;

import java.io.*;
import javax.swing.*;
import java.awt.Graphics;
import java.util.ArrayList;

import visalg.basics.ModuleManager;
import visalg.basics.DataModule;
import visalg.modules.TuringMachineViewer;

/**
 * This is a VisAlgData-type representing a Turing machine.
 * @see visalg.modules.TuringMachine
 */
public class VisAlgTuringMachineData implements VisAlgData {


    // -----------------------------------------------------
    // ------- V A R I A B L E S ---------------------------
    // -----------------------------------------------------

    /**
     * The the coded TM program
     */
    public ArrayList m_turingProgramCode;

    /**
     * The the programtext of the Turing machine program
     */
    public ArrayList m_turingProgramText;

    /**
     * Syntax type of the TM-program.
     */
    public int m_turingProgramSyntaxType;

    /**
     * Array for tape alphabet of the TM
     */
    public char[] m_tapeAlphabet;

    /**
     * Length of the tape alphabet of the TM
     */
    public int m_alphabetLength;

    /**
     * ArrayList for memory tape
     */
    public ArrayList m_memoryTape;

    /**
     * Minimum index of the memory tape
     */
    public int m_minTapeIndex;

    /**
     * Maximum index of the memory tape
     */
    public int m_maxTapeIndex;

    /**
     * Size of one block of the memory tape
     */
    public int m_tapeBlockSize;

    /**
     * Length of the memory tape of the TM in cells
     */
    public int m_tapeLength;

    /**
     * Number of visible cells(TM Viewer Window);
     */
    public int m_numberOfVisibleCells = 15;      // number of visible cells of the tape

     /**
     * Position of the head of the Turing machine
     */
    public int m_headPosition;

    /**
     * String for tape-input
     */
    public String m_turingMachineInput;

    /**
     * Current state of the TM-program. This state controls the TM-program flow
     */
    public int m_turingProgramCurrentState;

    /**
     * Contains -1 if there is no Turing machine module, 0 otherwise
     */
    public int m_turingProgramInitialError = -1;

    /**
     * Contains the type of error the Turing program might invoke while
     * running
     */
    public int m_turingProgramRunningError;

    /**
     * Counting the number of steps done during a computation
     */
    public int m_numberOfTuringProgramSteps;

    /**
     * Most extreme left position of the head during a computation
     */
    public int m_minHeadPosition;

    /**
     * Most extreme right position of the head during a computation
     */
    public int m_maxHeadPosition;

    // -----------------------------------------------------
    // ------- M E T H O D S   ( p u b l i c ) -------------
    // -----------------------------------------------------

    /**
     * Constructor
     */
    public VisAlgTuringMachineData(
                        ArrayList turingProgramCode,
                        int turingProgramSyntaxType,
                                char[] tapeAlphabet,
                              ArrayList turingProgramText) {

        m_turingProgramCode = turingProgramCode;   // coded program
                                          //      (integer-vectors)
        m_turingProgramSyntaxType = turingProgramSyntaxType;
        m_tapeAlphabet = tapeAlphabet;
	m_turingProgramText = turingProgramText;

        m_tapeBlockSize = 10;               // block size of the memory tape
        m_turingMachineInput = "";             // input string is empty at the beginning
        resetMachine();

    } // Constructor of VisAlgTuringMachineData

    /**
     * register viewer modules
     */
    public void registerViewerModules(ModuleManager mm, DataModule dm) {
        mm.addModule(new TuringMachineViewer(mm, dm));
    }

    public JComponent getComponent() { return null; }
    public void paintComponent(Graphics g, int x, int y) { }


    /**
     * Resets the tape and the head and the alphabet
     * of the Turing machine
     */
    public void resetMachine() {

        m_memoryTape = new ArrayList(); // Initialization of new tape-array
        m_tapeLength = 0;            // It is empty at the beginning
        m_maxTapeIndex = -1;
        m_minTapeIndex = 0;
        int number = m_numberOfVisibleCells/m_tapeBlockSize + 1;
        for(int i = 0; i < number; ++i) {
          expandMemoryTape();           // add two blocks to the memory tape
        }
        String s = new String(m_tapeAlphabet);  // determine the length of
        m_alphabetLength = s.length();      // the tape alphabet
        inputToTape();
        m_headPosition = 0;             // at the beginning the head is at cell 0

        m_numberOfTuringProgramSteps = 0;
        m_minHeadPosition = 0;
        m_maxHeadPosition = 0;

    } // method resetMachine

    /**
     * Returns the content of the ith cell of the tape
     */
    public char readFromTape(int i) {
        int blockNumber = (i + m_tapeLength/2) / 10;
        int position = (i + m_tapeLength/2) % 10;

        int[] block = (int[])m_memoryTape.get(blockNumber);
        int c = block[position];
        return (char)( (c == 0) ? '#' : c );
    } // method readFromTape(int i)

    /**
     * Returns the content of the cell under the head
     */
    public char readFromTape() {
        int blockNumber = (m_headPosition + m_tapeLength/2) / 10;
        int index = (m_headPosition + m_tapeLength/2) % 10;

        int[] block = (int[])m_memoryTape.get(blockNumber);
        int c = block[index];
        return (char)( (c == 0) ? '#' : c );
    } // method readFromTape()

    /**
     * Writes a char to the ith cell of the tape
     */
    public void writeToTape(int i, char s) {
        int blockNumber = (i + m_tapeLength/2) / 10;
        int position = (i + m_tapeLength/2) % 10;
        int[] block = (int[])m_memoryTape.get(blockNumber);

        if(s != '#') {
            block[position] = s;
        }
        else {
            block[position] = 0;
        }
    } // method writeToTape(int i, char s)

    /**
     * Writes a char to the cell under the head
     */
    public void writeToTape(char s) {

        int blockNumber = (m_headPosition + m_tapeLength/2) / 10;
        int index = (m_headPosition + m_tapeLength/2) % 10;
        int[] block = (int[])m_memoryTape.get(blockNumber);

        if(s != '#') {
            block[index] = s;
        }
        else {
            block[index] = 0;
        }

    } // method writeToTape(char s)

    /**
     * Moves the head on cell if possible, and returns -1 if not
     */
    public void moveHead(int direction) {

        int blockNumber = (m_headPosition + m_tapeLength/2) / 10;
        int index = (m_headPosition + m_tapeLength/2) % 10;

        if(direction == -1) {             // move one cell to the left
            checkIfMemorySuffices();
            m_headPosition -= 1;
            if(m_headPosition < m_minHeadPosition) {
                m_minHeadPosition = m_headPosition;
            }
        }
        else if(direction == 1) {          // move one cell to the right
            checkIfMemorySuffices();
            m_headPosition += 1;
            if(m_headPosition > m_maxHeadPosition) {
                m_maxHeadPosition = m_headPosition;
            }
        }
        else {                   // stand (nothing happens)
            m_headPosition += 0;
        }

    } // moveHead

    /**
     * Returns the ith line of the Turing program
     */
    public String getLine(int i) {
        if(i >= 0 && i < m_turingProgramText.size()) {
            return (String)m_turingProgramText.get(i);
        }
        else {
            System.out.println("VisAlgTMProgram.getLine(i): " +
                                       "Array Index out of bounds!");
            return null;
        }
    } // getLine

    /**
     * Reads the output beginning one cell right from the head
     */
    public String getOutput() {

        String s = "";

        char actualCharacter = readFromTape(m_headPosition);
            for(int i = m_headPosition+1; readFromTape(i) != '#'; ++i) {
            s = s + readFromTape(i);
        }
        return s;

    } // method getOutput

    /**
     * Returns the content of the whole tape as a String
     */
    public String getTape() {
        String s = "";
        for(int i = m_minTapeIndex; i < m_maxTapeIndex; ++i) {
            s += readFromTape(i);
        }
        return s;
    } // method getTape

    /**
     * Checks if the symbol c is contained in the tape alphabet
     */
    public boolean isInAlphabet(char c) {

        for (int i = 0; i < m_alphabetLength; ++i) {
            if (m_tapeAlphabet[i] == c) {
                return true;
            }
        }
        return false;

    } //isInAlphabet

    /**
     * Increases the variable 'm_numberOfTuringProgramSteps'
     */
    public void increaseNumberOfSteps() {
        ++m_numberOfTuringProgramSteps;
    } // increaseNumberOfSteps

    /**
     * Returns the number of cells visited during the whole computation
     */
    public int numberOfVisitedCells() {
        int nvc = m_maxHeadPosition - m_minHeadPosition + 1;
        return nvc;
    } // numberOfVisitedCells

    /**
     * Returns the number of the line with initial state s
     */
    public int numberOfLine(int state) {
        String line;
        String number;
        int num;
        char symbol = ' ';

        for(int i = 0; i < m_turingProgramText.size(); ++i) {
            number = "";
            line = (String)m_turingProgramText.get(i);
            line = line.trim();
            if(line.equals("")) {
                continue;
            }
            if (line.charAt(0) == '(') {     // finding a command line...
                int j = 1;
                while(line.charAt(j) != ',') {  // grabbing the 1. arg.
                    ++j;
                }
                number = line.substring(1,j);
                number = number.trim();
                num = Integer.parseInt(number);

                int jOld = j+1;
                while(line.charAt(++j) != ',') {  // grabbing the 2. arg.
                  ;
                }
                String s = line.substring(jOld, j);
                s = s.trim();
                symbol = s.charAt(0);

                if(m_turingProgramSyntaxType == 1) {
                    if(num == state) {
                        return i;
                    }
                }
                else if (m_turingProgramSyntaxType == 2) {
                    char cS = readFromTape(m_headPosition);
                    if((num == state) && (cS == symbol)) {
                        return i;
                    }
                }
            }
        }

        return -1;

    } // numberOfLine

    /**
     * Expands the memory tape by one block at each of its side.
     */
    public void expandMemoryTape() {
        m_memoryTape.add(0, new int[m_tapeBlockSize]); // adds one block to the
        m_memoryTape.add(new int[m_tapeBlockSize]);    // left and one to the
        m_tapeLength += 2 * m_tapeBlockSize;           // right end of the tape
        m_maxTapeIndex += m_tapeBlockSize;
        m_minTapeIndex -=m_tapeBlockSize;
    } // method expandMemoryTape

    /**
     * Puts the input string to the tape
     */
    public void inputToTape() {
        int index = 1;
        for(int i = 0; i < m_turingMachineInput.length()/m_tapeBlockSize; ++i) {
            expandMemoryTape();
        }
        clearTape();
        for(int i = 0; i < m_turingMachineInput.length(); ++i) {
            char s = m_turingMachineInput.charAt(i);
            if(isInAlphabet(s)) {
                writeToTape(index++, s);
            }
        }
    } // method inputToTape

    public void printData() {
        VisAlgTuringMachineData tmdata = this;
        int nol = tmdata.m_turingProgramText.size();
        for (int i = 0; i < nol; i++) {
            System.out.println(tmdata.getLine(i));
        }
    }//printData()

    public VisAlgData cloneData() {
        VisAlgTuringMachineData data = null;
        try {
            data = (VisAlgTuringMachineData) clone();
        }
        catch (Exception ex) {
            System.err.println(ex);
        }
        return data;
    }//cloneData()
    // -----------------------------------------------------
    // ------- M E T H O D S   ( p r i v a t e ) -----------
    // -----------------------------------------------------

    /**
     * Checks wether the memory tape has to been expanded or not
     */
    private void checkIfMemorySuffices() {
        int radius = m_numberOfVisibleCells/2 + 1;
        if((m_headPosition + radius + 1 >= m_maxTapeIndex) ||
                            (m_headPosition - radius - 1 <= m_minTapeIndex)) {
            expandMemoryTape();
        }
    } // method checkIfMemorySuffices

    /**
     * Fills the whole tape with '#' symbols
     */
    private void clearTape() {
        for(int i = m_minTapeIndex; i <= m_maxTapeIndex; ++i) {
            writeToTape(i, '#');
        }
    } // method clearTape

} // class VisAlgTuringMachineData