////////////////////////////////////// java ////////////////////////////////////
//
//  Module           : VisAlgVectorArray.java
//  Description      :
//  Author           : Matthias Vigelius
//  Email            :
//  Copyright        : Uni Heidelberg
//  Created on       : Mon Oct 29 11:45:00 2001
//  Last modified by :
//  Last modified on :
//  Update count     :
//  CVS Info         :
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	    Name		    Changes/Extensions
//  ----	    ----		    ------------------
///////////////////////////////////////////////////////////////////////////////


package visalg.types;

import java.util.Vector;
import java.util.Enumeration;
import java.lang.IllegalArgumentException;

import javax.swing.JComponent;
import javax.swing.JPanel;
import java.awt.Graphics;

import visalg.basics.ModuleManager;
import visalg.basics.DataModule;
import visalg.modules.ArrayViewer;
import visalg.modules.MatrixArrayViewer;

/**
 * This class represents all Vector structures of VisAlg. It replaces older
 * array classes. It can contain all type of VisAlgData, but it ensures, that
 * all data are of the same type. Once created, only data of the initial type
 * can be added.
 */
public class VisAlgVector implements VisAlgData {

    /**
     * The vector object containing the data.
     */
    private Vector m_data;

    /**
     * Contains the type of the data.
     */
    private Class m_classType;

    /**
     * At creation, the type of the data, that shall be contained in this
     * vector, must be omitted, in order to keep the vector type safe.
     *
     * @param dataType The type of the data that is contained here.
     */
    public VisAlgVector(VisAlgData dataType) {
        m_data = new Vector();

        // Store ClassType
        m_classType = dataType.getClass();
    }//VisAlgVector(VisAlgData)

    /**
     * Adds a new datum to the vector. It is checked, if the type of the
     * argument equals the type of the data which is hold here. If the
     * argument is not of the same type, an exception is thrown.
     *
     * @param VisAlgData The datum, that shall be added.
     * @exception IllegalArgumentException Will be thrown, if the type of the
     * argument does not match the type of the data.
     */
    public void addData(VisAlgData datum) throws IllegalArgumentException {
        // check if argument is of the right type
        if (datum.getClass().equals(m_classType)) {
            m_data.add(datum);
        } else {
            throw new IllegalArgumentException("Error in VisAlgVectorArray.addData(...) : Argument is not of same type !");
        }
    }//addData(VisAlgData)

    /**
     * Creates a clone of this class. In this clone method, all elements,
     * that are hold in the vector, are cloned by calling up clone().
     *
     * @return A copy of this class.
     */
    public VisAlgData cloneData() {
        VisAlgVector newVector = null;

        try {
            newVector = new VisAlgVector((VisAlgData) m_classType.newInstance());
        } catch(IllegalAccessException iae) {
            System.err.println(iae);
        } catch(InstantiationException ie) {
            System.err.println(ie);
        }

        for (Enumeration e=m_data.elements(); e.hasMoreElements(); ) {
            try {
                VisAlgData datum = (VisAlgData) e.nextElement();
                newVector.addData(datum.cloneData());
            } catch(IllegalArgumentException iae) {
                System.err.println(iae);
            }
        }

        return newVector;
    }//cloneData()

    /**
     * Returns a new JPanel object. This method should not be used anymore.
     */
    public JComponent getComponent(){return new JPanel();};

    /**
     * Does nothing. This method is subject to future changes.
     */
    public void paintComponent(Graphics g, int x, int y){};

    /**
     * Returns the elements of the vector as an array. The return type is an
     * array of the data types. E.G. : If the vector contains elements of type
     * VisAlgInt, the array will be of type VisAlgInt[]. Note : The array
     * contains references to the data elements, so changes to them will
     * affect the data itself.
     *
     * @return An array containing references to the data elements
     */
    public Object[] toArray() {
        Object[] returnArray;

        returnArray = (Object[]) java.lang.reflect.Array.newInstance(m_classType, m_data.size());

        m_data.toArray(returnArray);

        return returnArray;
    }//toArray()

    /**
     * The associated viewer module is ArrrayViewer.
     *
     * @see visalg.modules.ArrayViewer
     */
    public void registerViewerModules(ModuleManager mm, DataModule dm) {


        // Check, if type of hold data is equal to VisAlgComparableData
        if (isComparable()) {
            mm.addModule(new ArrayViewer(mm, dm));
            mm.addModule(new MatrixArrayViewer(mm, dm));
        }
    }//registerViewerModules(ModuleManager, DataModule)

    /**
     * Checks, if the data is of type VisAlgComparableData.
     *
     * @return True, if the data is of type VisAlgComparableData
     *
     * @see visalg.types.VisAlgComparableData
     */
    public boolean isComparable()
    {
        Class test=null;

        // Create a class object for the interface "visalg.types.VisAlgComparableData"
        try {
            test = m_classType.forName("visalg.types.VisAlgComparableData");
        }
        catch (Exception ex) {
            System.err.println(ex);
        }

        return test.isAssignableFrom(m_classType);
    }//isComparable()

    /**
     * Returns the data element at the specified position.
     *
     * @param index The position
     * @return The data element
     */
    public VisAlgData getDataElementAt(int index)
    {
        return (VisAlgData)m_data.get(index);
    }//getDataElementAt(int)

    /**
     * Swaps the elements at the specified positions.
     *
     * @param a,b The elements, that are to be swapped
     */
    public void swapValues(int a, int b) {
        Object oa = m_data.get(a);
        Object ob = m_data.get(b);
        m_data.set(a, ob);
        m_data.set(b, oa);
    }//swapValues(int, int)

    /**
     * Prints the data to the stdOut. For every element of this vector,
     * printData() is called.
     */
    public void printData() {
        System.out.println("Data Type : VisAlgVector of "+m_classType.toString());
        System.out.println();
        Enumeration enum = m_data.elements();
        while (enum.hasMoreElements()) {
            VisAlgData data = (VisAlgData) enum.nextElement();
            data.printData();
        }
    }//printData()
}//class VisAlgVector